# -*- coding: utf-8 -*-
# by einfall, murxer, pclin & Sven_H for DreamOS

from Screens.MessageBox import MessageBox
from Screens.Screen import Screen
from Screens.ChoiceBox import ChoiceBox
from Screens.Console import Console
from Components.ActionMap import ActionMap
from Components.MenuList import MenuList
from Components.Label import Label
from Components.Sources.StaticText import StaticText
from Components.MultiContent import MultiContentEntryText, MultiContentEntryTextAlphaBlend
from Components.Pixmap import Pixmap
from Components.FileList import FileList
from Components.ConfigList import ConfigList, ConfigListScreen
from Components.config import config, getConfigListEntry, ConfigText, ConfigSelection, ConfigSelectionNumber, \
    ConfigSubsection, NoSave, configfile
from Tools.BoundFunction import boundFunction
from enigma import gFont, ePoint, eSize, getDesktop, ePixmap, eListboxPythonMultiContent, RT_HALIGN_CENTER, \
    RT_VALIGN_CENTER, RT_WRAP, eListbox, eTimer, eConsoleAppContainer, eNetworkManager, eEnv
from skin import TemplatedListFonts, parseColor
from collections import OrderedDict
import subprocess, threading, os, json

sz_w = getDesktop(0).size().width()

hyperionremote_sh = "systemctl"
hyperioncontrol_version = "3.0"

json.encoder.FLOAT_REPR = lambda o: format(o, '.4f')

# == language-support ====================================
from Components.Language import language
from os import environ as os_environ, popen
import gettext
from Tools.Directories import resolveFilename, SCOPE_LANGUAGE, SCOPE_PLUGINS

lang = language.getLanguage()
os_environ["LANGUAGE"] = lang[:2]
gettext.bindtextdomain("enigma2", resolveFilename(SCOPE_LANGUAGE))
gettext.textdomain("enigma2")
gettext.bindtextdomain("enigma2-plugins", resolveFilename(SCOPE_LANGUAGE))
gettext.bindtextdomain("HyperionControl",
                       "%s%s" % (resolveFilename(SCOPE_PLUGINS), "Extensions/HyperionControl/locale"))


def _(txt):
    t = gettext.dgettext("HyperionControl", txt)
    if t == txt:
        t = gettext.gettext(txt)
    if t == txt:
        t = gettext.dgettext("enigma2-plugins", txt)
    return t


# == end language support ================================

class ColorNames():
    # english colornames
    itemslist_en = [(('Maroon', '#800000'),), (('Dark Red', '#8B0000'),), (('Brown', '#A52A2A'),),
                    (('Firebrick', '#B22222'),), (('Crimson', '#DC143C'),), (('Red', '#FF0000'),),
                    (('Tomato', '#FF6347'),), (('Coral', '#FF7F50'),), (('Indian Red', '#CD5C5C'),),
                    (('Light Coral', '#F08080'),), (('Dark Salmon', '#E9967A'),), (('Salmon', '#FA8072'),),
                    (('Light Salmon', '#FFA07A'),), (('Orange Red', '#FF4500'),), (('Dark Orange', '#FF8C00'),),
                    (('Orange', '#FFA500'),), (('Gold', '#FFD700'),), (('Dark Golden Rod', '#B8860B'),),
                    (('Golden Rod', '#DAA520'),), (('Pale Golden rod', '#EEE8AA'),), (('Dark Khaki', '#BDB76B'),),
                    (('Khaki', '#F0E68C'),), (('Olive', '#808000'),), (('Yellow', '#FFFF00'),),
                    (('Yellow Green', '#9ACD32'),), (('Dark Olive Green', '#556B2F'),), (('Olive Drab', '#6B8E23'),),
                    (('Lawn Green', '#7CFC00'),), (('Chart Reuse', '#7FFF00'),), (('Green Yellow', '#ADFF2F'),),
                    (('Dark Green', '#006400'),), (('Green', '#008000'),), (('Forest Green', '#228B22'),),
                    (('Lime', '#00FF00'),), (('Lime Green', '#32CD32'),), (('Light Green', '#90EE90'),),
                    (('Pale Green', '#98FB98'),), (('Dark Sea Green', '#8FBC8F'),),
                    (('Medium Spring Green', '#00FA9A'),), (('Spring Green', '#00FF7F'),), (('Sea Green', '#2E8B57'),),
                    (('Medium Aqua Marine', '#66CDAA'),), (('Medium Sea Green', '#3CB371'),),
                    (('Light Sea Green', '#20B2AA'),), (('Dark Slate Gray', '#2F4F4F'),), (('Teal', '#008080'),),
                    (('Dark Cyan', '#008B8B'),), (('Cyan', '#00FFFF'),), (('Light Cyan', '#E0FFFF'),),
                    (('Dark Turquoise', '#00CED1'),), (('Turquoise', '#40E0D0'),), (('Medium Turquoise', '#48D1CC'),),
                    (('Pale Turquoise', '#AFEEEE'),), (('Aqua Marine', '#7FFFD4'),), (('Powder Blue', '#B0E0E6'),),
                    (('Cadet Blue', '#5F9EA0'),), (('Steel Blue', '#4682B4'),), (('Corn Flower Blue', '#6495ED'),),
                    (('Deep Sky Blue', '#00BFFF'),), (('Dodger Blue', '#1E90FF'),), (('Light Blue', '#ADD8E6'),),
                    (('Sky Blue', '#87CEEB'),), (('Light Sky Blue', '#87CEFA'),), (('Midnight Blue', '#191970'),),
                    (('Navy', '#000080'),), (('Dark Blue', '#00008B'),), (('Medium Blue', '#0000CD'),),
                    (('Blue', '#0000FF'),), (('Royal Blue', '#4169E1'),), (('Blue Violet', '#8A2BE2'),),
                    (('Indigo', '#4B0082'),), (('Dark Slate Blue', '#483D8B'),), (('Slate Blue', '#6A5ACD'),),
                    (('Medium Slate Blue', '#7B68EE'),), (('Medium Purple', '#9370DB'),),
                    (('Dark Magenta', '#8B008B'),), (('Dark Violet', '#9400D3'),), (('Dark Orchid', '#9932CC'),),
                    (('Medium Orchid', '#BA55D3'),), (('Purple', '#800080'),), (('Thistle', '#D8BFD8'),),
                    (('Plum', '#DDA0DD'),), (('Violet', '#EE82EE'),), (('Magenta / Fuchsia', '#FF00FF'),),
                    (('Orchid', '#DA70D6'),), (('Medium Violet Red', '#C71585'),), (('Pale Violet Red', '#DB7093'),),
                    (('Deep Pink', '#FF1493'),), (('Hot Pink', '#FF69B4'),), (('Light Pink', '#FFB6C1'),),
                    (('Pink', '#FFC0CB'),), (('Antique White', '#FAEBD7'),), (('Beige', '#F5F5DC'),),
                    (('Bisque', '#FFE4C4'),), (('Blanched Almond', '#FFEBCD'),), (('Wheat', '#F5DEB3'),),
                    (('Corn Silk', '#FFF8DC'),), (('Lemon Chiffon', '#FFFACD'),),
                    (('Light Golden Rod Yellow', '#FAFAD2'),), (('Light Yellow', '#FFFFE0'),),
                    (('Saddle Brown', '#8B4513'),), (('Sienna', '#A0522D'),), (('Chocolate', '#D2691E'),),
                    (('Peru', '#CD853F'),), (('Sandy Brown', '#F4A460'),), (('Burly Wood', '#DEB887'),),
                    (('Tan', '#D2B48C'),), (('Rosy Brown', '#BC8F8F'),), (('Moccasin', '#FFE4B5'),),
                    (('Navajo White', '#FFDEAD'),), (('Peach Puff', '#FFDAB9'),), (('Misty Rose', '#FFE4E1'),),
                    (('Lavender Blush', '#FFF0F5'),), (('Linen', '#FAF0E6'),), (('Old Lace', '#FDF5E6'),),
                    (('Papaya Whip', '#FFEFD5'),), (('Sea Shell', '#FFF5EE'),), (('Mint Cream', '#F5FFFA'),),
                    (('Slate Gray', '#708090'),), (('Light Slate Gray', '#778899'),),
                    (('Light Steel Blue', '#B0C4DE'),), (('Lavender', '#E6E6FA'),), (('Floral White', '#FFFAF0'),),
                    (('Alice Blue', '#F0F8FF'),), (('Ghost White', '#F8F8FF'),), (('Honeydew', '#F0FFF0'),),
                    (('Ivory', '#FFFFF0'),), (('Azure', '#F0FFFF'),), (('Snow', '#FFFAFA'),),
                    (('Dim Gray', '#696969'),), (('Gray', '#808080'),), (('Dark gray', '#A9A9A9'),),
                    (('Silver', '#C0C0C0'),), (('Light Gray', '#D3D3D3'),), (('Gainsboro', '#DCDCDC'),),
                    (('White Smoke', '#F5F5F5'),), (('White', '#FFFFFF'),)]

    # german colornames
    itemslist_de = [(('Kastanienbraun', '#800000'),), (('Dunkelrot', '#8B0000'),), (('Braun', '#A52A2A'),),
                    (('Feuerrot', '#B22222'),), (('Purpur', '#DC143C'),), (('Rot', '#FF0000'),),
                    (('Tomatenrot', '#FF6347'),), (('Korallrot', '#FF7F50'),), (('Indianerrot', '#CD5C5C'),),
                    (('Helles Korallrot', '#F08080'),), (('Dunkles Lachsrosa', '#E9967A'),),
                    (('Lachsrosa', '#FA8072'),), (('Helles Lachsrosa', '#FFA07A'),), (('Orangerot', '#FF4500'),),
                    (('Dunkles Orangerot', '#FF8C00'),), (('Orangerot', '#FFA500'),), (('Gold', '#FFD700'),),
                    (('Dunkle Goldrute', '#B8860B'),), (('Goldrute', '#DAA520'),), (('Blaß Goldrute', '#EEE8AA'),),
                    (('Dunkler Erdton', '#BDB76B'),), (('Erdton', '#F0E68C'),), (('Olive', '#808000'),),
                    (('Gelb', '#FFFF00'),), (('Gelbgrün', '#9ACD32'),), (('Dunkles Olivegrün', '#556B2F'),),
                    (('Graugrün', '#6B8E23'),), (('Grasgrün', '#7CFC00'),), (('Chartreuse', '#7FFF00'),),
                    (('Grüngelb', '#ADFF2F'),), (('Dunkelgrün', '#006400'),), (('Grün', '#008000'),),
                    (('Waldgrün', '#228B22'),), (('Limone', '#00FF00'),), (('Limonengrün', '#32CD32'),),
                    (('Hellgrün', '#90EE90'),), (('Blaßgrün', '#98FB98'),), (('Dunkel Seegrün', '#8FBC8F'),),
                    (('Mittelfrühlingsgrün', '#00FA9A'),), (('Frühlingsgrün', '#00FF7F'),), (('Meergrün', '#2E8B57'),),
                    (('Mittelaquamarinblau', '#66CDAA'),), (('Mittelseegrün', '#3CB371'),),
                    (('Hellmeergrün', '#20B2AA'),), (('Dunkel Schiefergrau', '#2F4F4F'),), (('Blaugrün', '#008080'),),
                    (('Dunkles Zyan', '#008B8B'),), (('Zyan', '#00FFFF'),), (('Helles Zyan', '#E0FFFF'),),
                    (('Dunkeltürkis', '#00CED1'),), (('Türkis', '#40E0D0'),), (('Mitteltürkis', '#48D1CC'),),
                    (('Blaßtürkis', '#AFEEEE'),), (('Aquamarin', '#7FFFD4'),), (('Pulverblau', '#B0E0E6'),),
                    (('Kadettenblau', '#5F9EA0'),), (('Stahlblau', '#4682B4'),), (('Kornblumenblau', '#6495ED'),),
                    (('Dunkelhimmelblau', '#00BFFF'),), (('Persenningblau', '#1E90FF'),), (('Hellblau', '#ADD8E6'),),
                    (('Himmelblau', '#87CEEB'),), (('Hellhimmelblau', '#87CEFA'),), (('Mitternachtsblau', '#191970'),),
                    (('Navy', '#000080'),), (('Dunkelblau', '#00008B'),), (('Mittelblau', '#0000CD'),),
                    (('Blau', '#0000FF'),), (('Königsblau', '#4169E1'),), (('Blauviolett', '#8A2BE2'),),
                    (('Indigoblau', '#4B0082'),), (('Dunkles Schieferblau', '#483D8B'),),
                    (('Schlieferblau', '#6A5ACD'),), (('Mittelschieferblau', '#7B68EE'),), (('Mittellila', '#9370DB'),),
                    (('Dunkelmagenta', '#8B008B'),), (('Dunkelviolett', '#9400D3'),), (('Dunkel Orchidee', '#9932CC'),),
                    (('Mittelorchidee', '#BA55D3'),), (('Lila', '#800080'),), (('Distel', '#D8BFD8'),),
                    (('Pflaume', '#DDA0DD'),), (('Violett', '#EE82EE'),), (('Magenta', '#FF00FF'),),
                    (('Orchidee', '#DA70D6'),), (('Mittelviolettrot', '#C71585'),), (('Blaßviolettrot', '#DB7093'),),
                    (('Dunkelrosa', '#FF1493'),), (('Knallrosa', '#FF69B4'),), (('Hellrosa', '#FFB6C1'),),
                    (('Rosa', '#FFC0CB'),), (('Antikweiß', '#FAEBD7'),), (('Beige', '#F5F5DC'),),
                    (('Bisquit', '#FFE4C4'),), (('Mandelweiß', '#FFEBCD'),), (('Weizen', '#F5DEB3'),),
                    (('Kornseide', '#FFF8DC'),), (('Zitronenchiffon', '#FFFACD'),), (('Hellgoldrute Gelb', '#FAFAD2'),),
                    (('Hellgelb', '#FFFFE0'),), (('Sattelbraun', '#8B4513'),), (('Sienna', '#A0522D'),),
                    (('Schokolade', '#D2691E'),), (('Peru', '#CD853F'),), (('Sandbraun', '#F4A460'),),
                    (('Burlywood', '#DEB887'),), (('Hautfarben', '#D2B48C'),), (('Rosigbraun', '#BC8F8F'),),
                    (('Mokassin', '#FFE4B5'),), (('Navajo Weiß', '#FFDEAD'),), (('Pfirsich', '#FFDAB9'),),
                    (('Blaßrose', '#FFE4E1'),), (('Lavendelhauch', '#FFF0F5'),), (('Leinen', '#FAF0E6'),),
                    (('Alte Spitze', '#FDF5E6'),), (('Papaya Creme', '#FFEFD5'),), (('Muschelweiß', '#FFF5EE'),),
                    (('Pfefferminzcreme', '#F5FFFA'),), (('Schiefergrau', '#708090'),),
                    (('Hellschiefergrau', '#778899'),), (('Hellstahlblau', '#B0C4DE'),), (('Lavendel', '#E6E6FA'),),
                    (('Blütenweiß', '#FFFAF0'),), (('Aliceblau', '#F0F8FF'),), (('Geisterweiß', '#F8F8FF'),),
                    (('Honigtau', '#F0FFF0'),), (('Elfenbein', '#FFFFF0'),), (('Azur', '#F0FFFF'),),
                    (('Schneeweiß', '#FFFAFA'),), (('Blaßgrau', '#696969'),), (('Grau', '#808080'),),
                    (('Dunkelgrau', '#A9A9A9'),), (('Silber', '#C0C0C0'),), (('Hellgrau', '#D3D3D3'),),
                    (('Gainsboro', '#DCDCDC'),), (('Rauch', '#F5F5F5'),), (('Weiß', '#FFFFFF'),)]

    # gradient
    itemsverlauf = [(('', '#700000'),), (('', '#800000'),), (('', '#8D0000'),), (('', '#990000'),), (('', '#A60000'),),
                    (('', '#B30000'),), (('', '#C00000'),), (('', '#CC0000'),), (('', '#D90000'),), (('', '#E60000'),),
                    (('', '#F20000'),), (('', '#FF0000'),), (('', '#FF1010'),), (('', '#FF2020'),), (('', '#FF3030'),),
                    (('', '#FF3F3F'),), (('', '#FF4F4F'),), (('', '#FF5F5F'),), (('', '#FF6F6F'),), (('', '#FF7F7F'),),
                    (('', '#FF8F8F'),), (('', '#FF9F9F'),), (('', '#FFAFAF'),), (('', '#FFBEBE'),), (('', '#FFCECE'),),
                    (('', '#FFDEDE'),), (('', '#FFEEEE'),), (('', '#007000'),), (('', '#008000'),), (('', '#008D00'),),
                    (('', '#009900'),), (('', '#00A600'),), (('', '#00B300'),), (('', '#00C000'),), (('', '#00CC00'),),
                    (('', '#00D900'),), (('', '#00E600'),), (('', '#00F200'),), (('', '#00FF00'),), (('', '#10FF10'),),
                    (('', '#20FF20'),), (('', '#30FF30'),), (('', '#3FFF3F'),), (('', '#4FFF4F'),), (('', '#5FFF5F'),),
                    (('', '#6FFF6F'),), (('', '#7FFF7F'),), (('', '#8FFF8F'),), (('', '#9FFF9F'),), (('', '#AFFFAF'),),
                    (('', '#BEFFBE'),), (('', '#CEFFCE'),), (('', '#DEFFDE'),), (('', '#EEFFEE'),), (('', '#000070'),),
                    (('', '#000080'),), (('', '#00008D'),), (('', '#000099'),), (('', '#0000A6'),), (('', '#0000B3'),),
                    (('', '#0000C0'),), (('', '#0000CC'),), (('', '#0000D9'),), (('', '#0000E6'),), (('', '#0000F2'),),
                    (('', '#0000FF'),), (('', '#1010FF'),), (('', '#2020FF'),), (('', '#3030FF'),), (('', '#3F3FFF'),),
                    (('', '#4F4FFF'),), (('', '#5F5FFF'),), (('', '#6F6FFF'),), (('', '#7F7FFF'),), (('', '#8F8FFF'),),
                    (('', '#9F9FFF'),), (('', '#AFAFFF'),), (('', '#BEBEFF'),), (('', '#CECEFF'),), (('', '#DEDEFF'),),
                    (('', '#EEEEFF'),), (('', '#9E9E00'),), (('', '#ADAD00'),), (('', '#B5B500'),), (('', '#BDBD00'),),
                    (('', '#C6C600'),), (('', '#CECE00'),), (('', '#D6D600'),), (('', '#DEDE00'),), (('', '#E6E600'),),
                    (('', '#EFEF00'),), (('', '#F7F700'),), (('', '#FFFF00'),), (('', '#FFFF10'),), (('', '#FFFF20'),),
                    (('', '#FFFF30'),), (('', '#FFFF3F'),), (('', '#FFFF4F'),), (('', '#FFFF5F'),), (('', '#FFFF6F'),),
                    (('', '#FFFF7F'),), (('', '#FFFF8F'),), (('', '#FFFF9F'),), (('', '#FFFFAF'),), (('', '#FFFFBE'),),
                    (('', '#FFFFCE'),), (('', '#FFFFDE'),), (('', '#FFFFEE'),), (('', '#009E9E'),), (('', '#00ADAD'),),
                    (('', '#00B5B5'),), (('', '#00BDBD'),), (('', '#00C6C6'),), (('', '#00CECE'),), (('', '#00D6D6'),),
                    (('', '#00DEDE'),), (('', '#00E6E6'),), (('', '#00EFEF'),), (('', '#00F7F7'),), (('', '#00FFFF'),),
                    (('', '#10FFFF'),), (('', '#20FFFF'),), (('', '#30FFFF'),), (('', '#3FFFFF'),), (('', '#4FFFFF'),),
                    (('', '#5FFFFF'),), (('', '#6FFFFF'),), (('', '#7FFFFF'),), (('', '#8FFFFF'),), (('', '#9FFFFF'),),
                    (('', '#AFFFFF'),), (('', '#BEFFFF'),), (('', '#CEFFFF'),), (('', '#DEFFFF'),), (('', '#EEFFFF'),),
                    (('', '#4F004F'),), (('', '#5E005E'),), (('', '#6D006D'),), (('', '#7C007C'),), (('', '#8A008A'),),
                    (('', '#990099'),), (('', '#A700A7'),), (('', '#B600B6'),), (('', '#C500C5'),), (('', '#D300D3'),),
                    (('', '#E200E2'),), (('', '#FF00FF'),), (('', '#FF10FF'),), (('', '#FF20FF'),), (('', '#FF30FF'),),
                    (('', '#FF3FFF'),), (('', '#FF4FFF'),), (('', '#FF5FFF'),), (('', '#FF6FFF'),), (('', '#FF7FFF'),),
                    (('', '#FF8FFF'),), (('', '#FF9FFF'),), (('', '#FFAFFF'),), (('', '#FFBEFF'),), (('', '#FFCEFF'),),
                    (('', '#FFDEFF'),), (('', '#FFEEFF'),), (('', '#3F1F09'),), (('', '#46220A'),), (('', '#4D260B'),),
                    (('', '#53290C'),), (('', '#5A2D0D'),), (('', '#61300E'),), (('', '#68330E'),), (('', '#6F370F'),),
                    (('', '#753A10'),), (('', '#7C3E11'),), (('', '#834112'),), (('', '#8B4513'),), (('', '#924F1F'),),
                    (('', '#99592B'),), (('', '#A16337'),), (('', '#A86D42'),), (('', '#AF774E'),), (('', '#B6815A'),),
                    (('', '#BD8B66'),), (('', '#C59472'),), (('', '#CC9E7E'),), (('', '#D3A88A'),), (('', '#DAB296'),),
                    (('', '#E1BCA1'),), (('', '#E9C6AD'),), (('', '#F0D0B9'),), (('', '#F7DAC5'),), (('', '#C05000'),),
                    (('', '#C65000'),), (('', '#CD5D00'),), (('', '#D66700'),), (('', '#E07100'),), (('', '#E08100'),),
                    (('', '#E09100'),), (('', '#E39300'),), (('', '#E69500'),), (('', '#E99700'),), (('', '#EC9900'),),
                    (('', '#F09B00'),), (('', '#F39D00'),), (('', '#F69F00'),), (('', '#F9A100'),), (('', '#FCA300'),),
                    (('', '#FFA500'),), (('', '#FFAC14'),), (('', '#FFB329'),), (('', '#FFBB3D'),), (('', '#FFC252'),),
                    (('', '#FFC966'),), (('', '#FFD07A'),), (('', '#FFD78F'),), (('', '#FFDFA3'),), (('', '#FFE6B8'),),
                    (('', '#FFEDCC'),), (('', '#000000'),), (('', '#111111'),), (('', '#1B1B1B'),), (('', '#242424'),),
                    (('', '#2E2E2E'),), (('', '#373737'),), (('', '#414141'),), (('', '#4A4A4A'),), (('', '#545454'),),
                    (('', '#5D5D5D'),), (('', '#676767'),), (('', '#707070'),), (('', '#7A7A7A'),), (('', '#838383'),),
                    (('', '#8D8D8D'),), (('', '#969696'),), (('', '#A0A0A0'),), (('', '#A9A9A9'),), (('', '#B3B3B3'),),
                    (('', '#BCBCBC'),), (('', '#C6C6C6'),), (('', '#CFCFCF'),), (('', '#D9D9D9'),), (('', '#E2E2E2'),),
                    (('', '#ECECEC'),), (('', '#F5F5F5'),), (('', '#FFFFFF'),)]

    def getGradientList(self):
        return self.itemsverlauf

    def getColorList(self):
        if lang == "de_DE":
            return self.itemslist_de
        else:
            return self.itemslist_en

    def getColorName(self, colorValue=""):
        if lang == "de_DE":
            colorList = self.itemslist_de
        else:
            colorList = self.itemslist_en

        for color in colorList:
            if color[0][1] == colorValue:
                return color[0][0]

        if colorValue == "#000000":
            return _("None")
        return colorValue


class hyperionControlColors(Screen):
    if sz_w == 1920:
        skin = """
        <screen name="hyperionControlColors" position="center,100" size="900,940" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="20,860" scale="stretch" size="200,70" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="240,860" size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="460,860"  size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="680,860" size="200,70" scale="stretch"/>
            <eLabel font="Regular;30" foregroundColor="white" name="" position="20,860" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="460,860" size="200,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget source="key_blue" render="Label" font="Regular;30" foregroundColor="white" position="680,860" size="200,70" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="10,845" size="880,2" backgroundColor="grey" />
            <widget name="list" position="10,5" size="850,830" itemWidth="840" itemHeight="63" transparent="1" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""

    else:
        skin = """
        <screen name="hyperionControlColors" position="center,80" size="590,620" transparent="0">
            <ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
            <eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <widget source="key_yellow" render="Label" font="Regular;22" foregroundColor="white" position="295,570" size="140,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget source="key_blue" render="Label" font="Regular;22" foregroundColor="white" position="440,570" size="140,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
            <widget name="list" position="10,3" size="575,550" itemWidth="565" itemHeight="45" transparent="1" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""

    def __init__(self, session, OnlyReturnColor=False):
        Screen.__init__(self, session)
        self['actions'] = ActionMap(
            ['OkCancelActions', 'ColorActions', 'MenuActions', 'SetupActions', 'ListboxActions'], {
                'cancel': self.key_exit,
                'ok': self.key_ok,
                'red': self.key_red,
                'green': self.key_green,
                'yellow': self.key_yellow,
                'blue': self.key_blue,
                'previousSection': self.pageUp,
                'nextSection': self.pageDown,
                'left': self.moveLeft,
                'right': self.moveRight,
            })

        self.OnlyReturnColor = OnlyReturnColor

        tlf = TemplatedListFonts()

        self.ColorNames = ColorNames()
        self.litems = self.ColorNames.getColorList()
        self.ColorPage = "farbliste"

        if self.OnlyReturnColor:
            self["key_yellow"] = StaticText("")
            self["key_blue"] = StaticText("")
        else:
            self["key_yellow"] = StaticText(_("gradient"))
            self["key_blue"] = StaticText(_("Image Mode"))

        if sz_w == 1920:
            self.itemHeight = 63
            self.itemWidth = 840
            self.itemSize = 90
            self.margin = ePoint(10, 10)
        else:
            self.itemHeight = 45
            self.itemWidth = 565
            self.itemSize = 60
            self.margin = ePoint(5, 5)

        self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent, itemSize=self.itemHeight, margin=ePoint(0, 0), selectionZoom=1.1)
        self.ml.l.setFont(0, gFont(tlf.face(tlf.SMALL), tlf.size(tlf.SMALL)))
        self.ml.l.setFont(1, gFont(tlf.face(tlf.MEDIUM), tlf.size(tlf.MEDIUM)))
        self.ml.l.setBuildFunc(self.showList, True)
        self['list'] = self.ml

        self.onLayoutFinish.append(self.buildList)
        self.setTitle(_("Hyperion Control - Colors"))

    def buildList(self):
        self["list"].setList(self.litems)

    def moveLeft(self):
        if self.ColorPage == "farbverlauf":
            self["list"].instance.moveSelection(self["list"].instance.moveLeft)
        else:
            self.pageUp()

    def moveRight(self):
        if self.ColorPage == "farbverlauf":
            self["list"].instance.moveSelection(self["list"].instance.moveRight)
        else:
            self.pageDown()

    def pageUp(self):
        self["list"].pageUp()

    def pageDown(self):
        self["list"].pageDown()

    def showList(self, item, selected):
        (text, hcol) = item
        col = parseColor(hcol).argb()
        if selected:
            textcol = parseColor("#000000").argb()
            if col == textcol:
                textcol = parseColor("#FFFFFF").argb()
        else:
            textcol = parseColor("#FFFFFF").argb()
            color = [int(hcol[1:3], 16), int(hcol[3:5], 16), int(hcol[5:7], 16)]
            if (color[0] > 191 and color[1] > 181) or (color[1] > 206 and color[2] > 208):
                textcol = parseColor("#696969").argb()

        res = [item]

        try:
            if self.ColorPage == "farbverlauf":
                EntrySize = (self.itemSize, self.itemSize)
                if sz_w == 1920:
                    pos = (7, 7)
                else:
                    pos = (5, 5)
                if selected:
                    pos = (0, 0)
            elif self.ColorPage == "farbliste":
                if sz_w == 1920:
                    margin = 5
                    pos = (margin + 15, margin)
                    EntrySize = (self.itemWidth - margin * 2 - 25, self.itemHeight - margin * 2)
                else:
                    margin = 4
                    pos = (margin + 10, margin)
                    EntrySize = (self.itemWidth - margin * 2 - 15, self.itemHeight - margin * 2)

            if selected:
                res.append(MultiContentEntryTextAlphaBlend(pos=pos, size=EntrySize, font=1, text=text,
                                                           flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
                                                           color=textcol, color_sel=textcol, backcolor=col,
                                                           backcolor_sel=col))
            else:
                res.append(MultiContentEntryTextAlphaBlend(pos=pos, size=EntrySize, font=0, text=text,
                                                           flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER | RT_WRAP,
                                                           color=textcol, color_sel=textcol, backcolor=col,
                                                           backcolor_sel=col))
            return res

        except:
            import traceback
            traceback.print_exc()
            return None

    def key_red(self):
        self.key_exit()

    def key_green(self):
        pass

    def key_yellow(self):
        if self.OnlyReturnColor:
            return
        if self.ColorPage == "farbverlauf":
            self.ColorPage = "farbliste"
            self["key_yellow"].setText(_("gradient"))
            # self["list"].instance.setMode(eListbox.layoutVertical)
            self["list"].instance.setItemHeight(self.itemHeight)
            self["list"].instance.setItemWidth(self.itemWidth)
            self["list"].instance.setMargin(ePoint(0, 0))
            self.litems = self.ColorNames.getColorList()
        else:
            self.ColorPage = "farbverlauf"
            self["key_yellow"].setText(_("color list"))
            # self["list"].instance.setMode(eListbox.layoutGrid)
            self["list"].instance.setItemHeight(self.itemSize)
            self["list"].instance.setItemWidth(self.itemSize)
            self["list"].instance.setMargin(self.margin)
            self.litems = self.ColorNames.getGradientList()

        index = self["list"].getSelectionIndex()
        self["list"].moveToIndex(0)
        self.buildList()
        self["list"].moveToIndex(index)

    def key_blue(self):
        if self.OnlyReturnColor:
            return
        try:
            if config.plugins.hyperioncontrol.lastpicpath.value:
                path = config.plugins.hyperioncontrol.lastpicpath.value
            else:
                path = "/"
            self.session.openWithCallback(self.selectPictureCallback, HyperionControlPictureBrowser, path)
        except:
            import traceback
            traceback.print_exc()

    def selectPictureCallback(self, retvalue):
        if retvalue:
            cmd = "hyperion-remote --clearall && hyperion-remote -i " + retvalue
            print "[HyperionControl] cmd", cmd
            command = Command(cmd)
            command.run(timeout=3)
            print "[HyperionControl] cmd_out:", command.out

    def key_ok(self):
        exist = self['list'].getCurrent()
        if exist == None:
            return
        print "=== index ok", self["list"].getSelectionIndex()
        (name, color) = self['list'].getCurrent()[0]
        print name, color

        if self.OnlyReturnColor:
            self.close((name, color))
            return
        cmd = "-c '%s'" % color
        send_CMD(cmd)

    def key_exit(self):
        self.close()


# to get consolen-output with timeout-function
class Command(object):
    def __init__(self, cmd):
        self.cmd = cmd
        self.process = None
        self.out = ""
        self.err = ""

    def run(self, timeout):
        def target():
            self.process = subprocess.Popen(self.cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
            (self.out, self.err) = self.process.communicate()

        thread = threading.Thread(target=target)
        thread.start()
        self.timeout = False

        thread.join(float(timeout))
        if thread.is_alive():
            self.process.terminate()
            self.timeout = True
            self.process = None
            thread = None
            return


# =============================================

class ConfigListHC(ConfigList):
    def __init__(self, list, session=None):
        ConfigList.__init__(self, list, session=session)

    def jumpToPreviousSection(self):
        index = self.getCurrentIndex() - 1
        maxlen = len(self._ConfigList__list)
        while index >= 0 and maxlen > 0:
            index -= 1
            # fix jump to PreviousSection on empty lines in ConfigList
            if index in self._headers and self.list[index] != ("",):
                if index + 1 < maxlen:
                    self.setCurrentIndex(index + 1)
                    return
                else:
                    self.setCurrentIndex(index - 1)
                    return
        self.pageUp()


# =========================================================================

class hyperionControlSetup(Screen, ConfigListScreen):
    if sz_w == 1920:
        skin = """
        <screen name="hyperionControlSetup" position="center,100" size="900,950" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="20,870" scale="stretch" size="200,70" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="240,870" size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="460,870"  size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="680,870" size="200,70" scale="stretch"/>
            <eLabel name="" position="10,850" size="880,2" backgroundColor="grey" />
            <eLabel font="Regular;30" foregroundColor="white" name="" position="20,870" size="200,70" text="Config Creator" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel font="Regular;30" foregroundColor="white" name="" position="240,870" size="200,70" text="Save" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
            <widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="460,870" size="200,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget source="key_blue" render="Label" font="Regular;30" foregroundColor="white" position="680,870" size="200,70" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <widget name="config" seperation="100" position="10,10" size="880,810" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""
    else:
        skin = """
        <screen name="hyperionControlSetup" position="center,80" size="590,620">
            <ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
            <eLabel font="Regular;18" foregroundColor="white" name="" position="5,570" size="140,40" text="Config Creator" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel font="Regular;18" foregroundColor="white" name="" position="150,570" size="140,40" text="Save" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
            <widget source="key_yellow" render="Label" font="Regular;18" foregroundColor="white" position="295,570" size="140,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget source="key_blue" render="Label" font="Regular;18" foregroundColor="white" position="440,570" size="140,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
            <widget name="config" position="10,10" seperation="100" size="570,545" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'InfobarEPGActions'], {
            'cancel': self.key_exit,
            'red': self.key_red,
            'green': self.key_green,
            'yellow': self.key_yellow,
            'blue': self.key_blue,
            'menu': self.menu,
            'showEventInfo': self.info,
            # 'ok': self.key_ok,
        })

        self["key_yellow"] = StaticText()
        self["key_blue"] = StaticText(_("Options"))
        self.setupPage = "device"

        self.list = []
        ConfigListScreen.__init__(self, self.list, session=session, on_change=self.changed)
        self["config"] = ConfigListHC(self.list, session=session)
        self.setTitle(_("Hyperion Control - Setup"))
        self["key_yellow"].setText(_("Remote/Action"))

        # build choices
        setChoicesHyperionRemote()

        self.v4l2Device = readV4l2_Device()
        # no grabber found load new choices
        if not self.v4l2Device:
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                config.plugins.hyperioncontrol.hyperionModeType.value = config.plugins.hyperioncontrol.hyperionModeType.default
            config.plugins.hyperioncontrol.hyperionModeType = ConfigSelection(choices=[("hyperion", "Hyperion Boblight/KodiServer"), ("hyperion-aml", _("Hyperion-internal grabber"))], default="hyperion-aml")
            if config.plugins.hyperioncontrol.onIdleAction.value == "v4l2":
                config.plugins.hyperioncontrol.onIdleAction.value = config.plugins.hyperioncontrol.onIdleAction.default
            config.plugins.hyperioncontrol.onIdleAction = ConfigSelection(choices=[("color", _("Color")), ("effect", _("Effect")), ("off", _("Hyperion off")), ("nothing", _("nothing"))], default="nothing")
        else:
            config.plugins.hyperioncontrol.hyperionModeType = ConfigSelection(choices=[("hyperion", "Hyperion Boblight/KodiServer"), ("hyperion-aml", _("Hyperion-internal grabber")), ("hyperion-v4l2", _("Hyperion-external grabber"))], default="hyperion-aml")
            config.plugins.hyperioncontrol.onIdleAction = ConfigSelection(choices=[("color", _("Color")), ("effect", _("Effect")), ("off", _("Hyperion off")), ("nothing", _("nothing")), ("v4l2", _("V4l2"))], default="nothing")

        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            config.plugins.hyperioncontrol.hyperionRemoteType.value = "hyperion-v4l2"
        elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
            config.plugins.hyperioncontrol.hyperionRemoteType.value = "hyperion-aml"
        else:
            config.plugins.hyperioncontrol.hyperionRemoteType.value = "hyperion"
        
        self.onShown.append(self.readData)
        self.lastLEDcountTxt = None

        self.ColorNames = ColorNames()

        # reload ConfigText to ConfigSelection for Idle-Color and Idle-Effects
        color = config.plugins.hyperioncontrol.onIdleColor.value
        colorName = self.ColorNames.getColorName(color)
        config.plugins.hyperioncontrol.onIdleColor = ConfigSelection(choices=[(color, colorName)], default=color)
        color = config.plugins.hyperioncontrol.onIdlebackColor.value
        colorName = self.ColorNames.getColorName(color)
        config.plugins.hyperioncontrol.onIdlebackColor = ConfigSelection(choices=[(color, colorName)], default=color)
        value = config.plugins.hyperioncontrol.onIdleEffect.value
        config.plugins.hyperioncontrol.onIdleEffect = ConfigSelection(choices=[(value, value)], default=value)
        value = config.plugins.hyperioncontrol.onIdlebackEffect.value
        config.plugins.hyperioncontrol.onIdlebackEffect = ConfigSelection(choices=[(value, value)], default=value)

    def readData(self):
        self.onShown.remove(self.readData)
        self.readValuesFromConfig()
        readValuesFromRemote(self)
        self.buildConfig()

    def saveRemoteJson(self):
        try:
            if config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-above":
                config_file_path = "/etc/hyperion/hyperion-above.config.json"
            elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-below":
                config_file_path = "/etc/hyperion/hyperion-below.config.json"
            elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-left":
                config_file_path = "/etc/hyperion/hyperion-left.config.json"
            elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-right":
                config_file_path = "/etc/hyperion/hyperion-right.config.json"
            elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-v4l2":
                config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
            else:
                config_file_path = "/etc/hyperion/hyperion.config.json"

            if os.path.exists(config_file_path):
                valuesFromConfig = open(config_file_path).read()
                lines = valuesFromConfig.split("\n")
                newlines = []
                for line in lines:
                    cleanline = line.replace("\t", "")
                    if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
                        newlines.append(line)
                valuesFromConfig = "\n".join(newlines)
                jsonConfigRemote = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)

                # hsv
                jsonConfigRemote['color']['transform'][0]['hsv']['saturationGain'] = int(config.plugins.hyperioncontrol.saturationGain.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['hsv']['valueGain'] = int(config.plugins.hyperioncontrol.valueGain.value) / 100.0

                # hsl
                jsonConfigRemote['color']['transform'][0]['hsl'] = {}
                jsonConfigRemote['color']['transform'][0]['hsl']['saturationLGain'] = int(config.plugins.hyperioncontrol.saturationLGain.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['hsl']['luminanceGain'] = int(config.plugins.hyperioncontrol.luminanceGain.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['hsl']['luminanceMinimum'] = int(config.plugins.hyperioncontrol.luminanceMinimum.value) / 100.0

                # red
                jsonConfigRemote['color']['transform'][0]['red']['gamma'] = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['red']['threshold'] = int(config.plugins.hyperioncontrol.thresholdRed.value) / 100.0

                # set as new key
                jsonConfigRemote['color']['temperature'] = [{}]
                jsonConfigRemote['color']['temperature'][0]['id'] = "default"
                jsonConfigRemote['color']['temperature'][0]['leds'] = "*"
                jsonConfigRemote['color']['temperature'][0]['correctionValues'] = {}

                jsonConfigRemote['color']['temperature'][0]['correctionValues']['red'] = int(config.plugins.hyperioncontrol.temperatureRed.value)

                # set as new key
                jsonConfigRemote['color']['channelAdjustment'] = [{}]
                jsonConfigRemote['color']['channelAdjustment'][0]['id'] = "default"
                jsonConfigRemote['color']['channelAdjustment'][0]['leds'] = "*"
                jsonConfigRemote['color']['channelAdjustment'][0]['pureRed'] = {}
                jsonConfigRemote['color']['channelAdjustment'][0]['pureGreen'] = {}
                jsonConfigRemote['color']['channelAdjustment'][0]['pureBlue'] = {}

                jsonConfigRemote['color']['channelAdjustment'][0]['pureRed']['redChannel'] = int(config.plugins.hyperioncontrol.redAdjustRed.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureGreen']['redChannel'] = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureBlue']['redChannel'] = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
                jsonConfigRemote['color']['transform'][0]['red']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['red']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0

                # green
                jsonConfigRemote['color']['transform'][0]['green']['gamma'] = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['green']['threshold'] = int(config.plugins.hyperioncontrol.thresholdGreen.value) / 100.0
                jsonConfigRemote['color']['temperature'][0]['correctionValues']['green'] = int(config.plugins.hyperioncontrol.temperatureGreen.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureRed']['greenChannel'] = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureGreen']['greenChannel'] = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureBlue']['greenChannel'] = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
                jsonConfigRemote['color']['transform'][0]['green']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['green']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0

                # blue
                jsonConfigRemote['color']['transform'][0]['blue']['gamma'] = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['blue']['threshold'] = int(config.plugins.hyperioncontrol.thresholdBlue.value) / 100.0
                jsonConfigRemote['color']['temperature'][0]['correctionValues']['blue'] = int(config.plugins.hyperioncontrol.temperatureBlue.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureRed']['blueChannel'] = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureGreen']['blueChannel'] = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
                jsonConfigRemote['color']['channelAdjustment'][0]['pureBlue']['blueChannel'] = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
                jsonConfigRemote['color']['transform'][0]['blue']['whitelevel'] = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
                jsonConfigRemote['color']['transform'][0]['blue']['blacklevel'] = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0

                if os.path.exists(config_file_path):
                    # write values in config.json
                    with open(config_file_path, "w") as write_file:
                        json.dump(jsonConfigRemote, write_file, ensure_ascii=False, indent=4)
                    self.session.open(MessageBox, _("The settings were saved in '%s'." % config_file_path), MessageBox.TYPE_INFO, timeout=10)
                else:
                    self.session.open(MessageBox, _("File '%s' not found!" % config_file_path), MessageBox.TYPE_INFO, timeout=5)
            else:
                self.session.open(MessageBox, _("File '%s' not found!" % config_file_path),  MessageBox.TYPE_INFO, timeout=5)

        except:
            import traceback, sys
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
            self.session.open(MessageBox, _("When saving the 'hyperion.config.json' an error has occurred!\n\nError:\n%s") % error, MessageBox.TYPE_INFO, timeout=5)

    def saveJson(self):
        try:
            self.setAutostart()
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
            else:
                config_file_path = "/etc/hyperion/hyperion.config.json"

            if os.path.exists(config_file_path):
                # write values in config.json
                with open(config_file_path, "w") as write_file:
                    json.dump(self.jsonConfig, write_file, ensure_ascii=False, indent=4)

                # daemon reload und services neu starten
                os.system("systemctl daemon-reload")
                if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion":
                    os.system("systemctl -q is-active hyperionaml  && systemctl stop hyperionaml")
                    os.system("systemctl -q is-active hyperion-v4l2  && systemctl stop hyperion-v4l2")
                    cmd = "systemctl restart hyperion"
                    txt = _("The settings were saved in '/etc/hyperion/hyperion.config.json'.\n\nHyperion were restarted.")
                elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
                    os.system("systemctl -q is-active hyperion-v4l2  && systemctl stop hyperion-v4l2")
                    cmd = "systemctl restart hyperion && systemctl restart hyperionaml"
                    txt = _("The settings were saved in '/etc/hyperion/hyperion.config.json'.\n\nHyperion and internal grabber were restarted.")
                else:  # "hyperion-v4l2"
                    os.system("systemctl -q is-active hyperion  && systemctl stop hyperion")
                    os.system("systemctl -q is-active hyperionaml  && systemctl stop hyperionaml")
                    cmd = "systemctl restart hyperion-v4l2"
                    txt = _("The settings were saved in '/etc/hyperion/hyperion.config-v4l2.json'.\n\nexternal grabber were restarted.")

                os.system(cmd)
                self.readValuesFromConfig()
                readValuesFromRemote(self)
                setChoicesHyperionRemote()
                self.session.openWithCallback(self.saveJsonMessageCallback, MessageBox, txt, MessageBox.TYPE_INFO, timeout=10)
            else:
                self.session.open(MessageBox, _("File '/etc/hyperion/hyperion.config.json' not found!"),  MessageBox.TYPE_INFO, timeout=5)

            self.saveAll()  # save all config-values which don't use NoSave()
            config.plugins.hyperioncontrol.save()
            configfile.save()
        except:
            import traceback, sys
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
            self.session.open(MessageBox, _("When saving the 'hyperion.config.json' an error has occurred!\n\nError:\n%s") % error, MessageBox.TYPE_INFO, timeout=5)

    def saveJsonMessageCallback(self, ret=None):
        # check installed packages for device-type
        if config.plugins.hyperioncontrol.deviceType.value in ("sedu", "adalight", "AdalightApa102", "atmo", "karate"):
            command = Command("apt list --installed kernel-module-usbserial kernel-module-ch341 kernel-module-ftdi-sio")
            command.run(timeout=5)

            # read real settings from hyperion-remote -l
            if command.timeout or command.process.returncode != 0:
                print "[HyperionControl] timeout/error on check packages"
            else:
                cmdValue = command.out.strip().split("\n")
                if len(cmdValue) < 4:
                    self.session.openWithCallback(self.installPackages, MessageBox, _("You need more kernel-moduls for the selected device-type:\n  - kernel-module-usbserial\n  - kernel-module-ch341\n  - kernel-module-ftdi-sio\n\nDo you want to install this needed modules?"), MessageBox.TYPE_YESNO)

    def installPackages(self, ret=None):
        if ret:
            self.session.openWithCallback(self.installPackagesCallback, Console, _("HyperionControl - install needed modules"), ["apt-get -y --assume-yes install kernel-module-usbserial kernel-module-ch341 kernel-module-ftdi-sio", "modprobe usbserial", "modprobe ch341", "modprobe ftdi-sio", "systemctl restart hyperion && systemctl restart hyperionaml"])

    def installPackagesCallback(self, ret=None):
        self.session.open(MessageBox, _("Needed modules was installed and loaded.\n\nHyperion and internal grabber were restarted."), MessageBox.TYPE_INFO)

    def setAutostart(self):
        value = getHyperionInfo(mode="autostart")
        if config.plugins.hyperioncontrol.hyperionAutostart.value:
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion":
                if value[1]:
                    cmd = "%s disable systemd-hyperionaml.timer" % hyperionremote_sh
                    os.system(cmd)
                if value[2]:
                    cmd = "%s disable hyperion-v4l2.service" % hyperionremote_sh
                    os.system(cmd)
                if not value[0]:
                    cmd = "%s enable hyperion.service" % hyperionremote_sh
                    os.system(cmd)

            elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
                if value[0]:
                    cmd = "%s disable hyperion.service" % hyperionremote_sh
                    os.system(cmd)
                if value[2]:
                    cmd = "%s disable hyperion-v4l2.service" % hyperionremote_sh
                    os.system(cmd)
                if not value[1]:
                    cmd = "%s enable systemd-hyperionaml.timer" % hyperionremote_sh
                    os.system(cmd)

            elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                if value[0]:
                    cmd = "%s disable hyperion.service" % hyperionremote_sh
                    os.system(cmd)
                if value[1]:
                    cmd = "%s disable systemd-hyperionaml.timer" % hyperionremote_sh
                    os.system(cmd)
                if not value[2]:
                    cmd = "%s enable hyperion-v4l2.service" % hyperionremote_sh
                    os.system(cmd)
        else:
            if value[0]:
                cmd = "%s disable hyperion.service" % hyperionremote_sh
                os.system(cmd)
            if value[1]:
                cmd = "%s disable systemd-hyperionaml.timer" % hyperionremote_sh
                os.system(cmd)
            if value[2]:
                cmd = "%s disable hyperion-v4l2.service" % hyperionremote_sh
                os.system(cmd)

        if config.plugins.hyperioncontrol.extraLamp.value:
            if not config.plugins.hyperioncontrol.hyperionLeftAutostart.value:
                cmd = "%s disable hyperion-left" % hyperionremote_sh
            else:
                cmd = "%s enable hyperion-left" % hyperionremote_sh
            os.system(cmd)
            if not config.plugins.hyperioncontrol.hyperionRightAutostart.value:
                cmd = "%s disable hyperion-right" % hyperionremote_sh
            else:
                cmd = "%s enable hyperion-right" % hyperionremote_sh
            os.system(cmd)
            if not config.plugins.hyperioncontrol.hyperionAboveAutostart.value:
                cmd = "%s disable hyperion-above" % hyperionremote_sh
            else:
                cmd = "%s enable hyperion-above" % hyperionremote_sh
            os.system(cmd)
            if not config.plugins.hyperioncontrol.hyperionBelowAutostart.value:
                cmd = "%s disable hyperion-below" % hyperionremote_sh
            else:
                cmd = "%s enable hyperion-below" % hyperionremote_sh
            os.system(cmd)
        else:
            for conf in ["above", "below", "left", "right"]:
                cmd = "%s disable hyperion-%s" % (hyperionremote_sh, conf)
                os.system(cmd)

    def importJsonBak(self, retval):
        try:
            if not retval:
                return

            # read values from config.json.bak
            if not self.jsonConfig:
                self.session.open(MessageBox, _(
                    "There are no data in '/etc/hyperion/hyperion.config.json.bak'!\n\nAn import is therefore not possible."), MessageBox.TYPE_INFO, timeout=5)
                return

            self.jsonConfigBak = ""
            config_file_path = "/etc/hyperion/hyperion.config.json.bak"
            if os.path.exists(config_file_path):
                # json.bak einlesen
                valuesFromConfig = open(config_file_path).read()
                lines = valuesFromConfig.split("\n")
                newlines = []
                for line in lines:
                    cleanline = line.replace("\t", "")
                    if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
                        newlines.append(line)
                valuesFromConfig = "\n".join(newlines)
                self.jsonConfigBak = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)

                # alle Hauptkeys aus json.bak in json übernehmen
                for (k, v) in self.jsonConfigBak.items():
                    # print "===", k
                    key = str(k)
                    if key in self.jsonConfig:
                        self.jsonConfig[key] = self.jsonConfigBak[key]

                # reload configs from jsonConfigBak
                # hsv
                setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, self.jsonConfigBak, ['color', 'transform', 0, 'hsv', 'saturationGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, self.jsonConfigBak, ['color', 'transform', 0, 'hsv', 'valueGain'], 100)
                # hsl
                setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, self.jsonConfigBak, ['color', 'transform', 0, 'hsl', 'saturationLGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, self.jsonConfigBak, ['color', 'transform', 0, 'hsl', 'luminanceGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, self.jsonConfigBak, ['color', 'transform', 0, 'hsl', 'luminanceMinimum'], 100)
                # red
                setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, self.jsonConfigBak, ['color', 'transform', 0, 'red', 'gamma'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, self.jsonConfigBak, ['color', 'transform', 0, 'red', 'threshold'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, self.jsonConfigBak, ['color', 'temperature', 0, 'correctionValues', 'red'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureRed', 'redChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureGreen', 'redChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureBlue', 'redChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, self.jsonConfigBak, ['color', 'transform', 0, 'red', 'whitelevel'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, self.jsonConfigBak, ['color', 'transform', 0, 'red', 'blacklevel'], 100)

                # green
                setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, self.jsonConfigBak, ['color', 'transform', 0, 'green', 'gamma'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, self.jsonConfigBak, ['color', 'transform', 0, 'green', 'threshold'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, self.jsonConfigBak, ['color', 'temperature', 0, 'correctionValues', 'green'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureRed', 'greenChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureGreen', 'greenChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureBlue', 'greenChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, self.jsonConfigBak, ['color', 'transform', 0, 'green', 'whitelevel'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, self.jsonConfigBak, ['color', 'transform', 0, 'green', 'blacklevel'], 100)

                # blue
                setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, self.jsonConfigBak, ['color', 'transform', 0, 'blue', 'gamma'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, self.jsonConfigBak, ['color', 'transform', 0, 'blue', 'threshold'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, self.jsonConfigBak, ['color', 'temperature', 0, 'correctionValues', 'blue'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureRed', 'blueChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureGreen', 'blueChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, self.jsonConfigBak, ['color', 'channelAdjustment', 0, 'pureBlue', 'blueChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, self.jsonConfigBak, ['color', 'transform', 0, 'blue', 'whitelevel'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, self.jsonConfigBak, ['color', 'transform', 0, 'blue', 'blacklevel'], 100)

                # device
                setConfigValueFromJson(config.plugins.hyperioncontrol.deviceType, self.jsonConfigBak, ['device', 'type'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.configName, self.jsonConfigBak, ['device', 'name'])
                if config.plugins.hyperioncontrol.deviceType.value in ("udpraw", "philipshue", "philipshueentertainment"):
                    outputIP = setConfigValueFromJson(config.plugins.hyperioncontrol.outputIP, self.jsonConfigBak, ['device', 'output'], onlyReturnValue=True)
                    outputIP = outputIP.replace(":19446", "").split(".")
                    outputIP = [int(d) for d in outputIP]
                    config.plugins.hyperioncontrol.outputIP.value = outputIP
                setConfigValueFromJson(config.plugins.hyperioncontrol.colorOrder, self.jsonConfigBak, ['device', 'colorOrder'])
                if config.plugins.hyperioncontrol.deviceType.value == "philipshue":
                    setConfigValueFromJson(config.plugins.hyperioncontrol.phe_username, self.jsonConfigBak, ['device', 'username'])
                    lightIDs = setConfigValueFromJson(config.plugins.hyperioncontrol.phe_lightIds, self.jsonConfigBak, ['device', 'lightIds'], onlyReturnValue=True)
                if config.plugins.hyperioncontrol.deviceType.value == "philipshueentertainment":
                    setConfigValueFromJson(config.plugins.hyperioncontrol.phe_username, self.jsonConfigBak, ['device', 'username'])
                    setConfigValueFromJson(config.plugins.hyperioncontrol.phe_clientkey, self.jsonConfigBak, ['device', 'clientkey'])
                    lightIDs = setConfigValueFromJson(config.plugins.hyperioncontrol.phe_lightIds, self.jsonConfigBak, ['device', 'lightIds'], onlyReturnValue=True)
                    config.plugins.hyperioncontrol.phe_lightIds.value = ', '.join(map(str, lightIDs))
                    setConfigValueFromJson(config.plugins.hyperioncontrol.phe_groupId, self.jsonConfigBak, ['device', 'groupId'])
                    setConfigValueFromJson(config.plugins.hyperioncontrol.delayafterconnect, self.jsonConfigBak, ['device', 'delayafterconnect'])
                if config.plugins.hyperioncontrol.deviceType.value in ("philipshue", "philipshueentertainment"):
                    self.jsonConfig['device']['switchOffOnBlack'] = str( config.plugins.hyperioncontrol.phe_switchOffOnBlack.value)
                else:
                    setConfigValueFromJson(config.plugins.hyperioncontrol.baudrate, self.jsonConfigBak, ['device', 'rate'], calculate=1)
                if config.plugins.hyperioncontrol.deviceType.value in ("udpraw"):
                    setConfigValueFromJson(config.plugins.hyperioncontrol.protocol, self.jsonConfigBak, ['device', 'protocol'])

                # outputType
                if config.plugins.hyperioncontrol.deviceType.value in ("sedu", "adalight", "AdalightApa102", "atmo", "karate"):
                    setConfigValueFromJson(config.plugins.hyperioncontrol.outputTYPE, self.jsonConfigBak, ['device', 'output'])

                # Framegrabber
                width = setConfigValueFromJson(None, self.jsonConfigBak, ['framegrabber', 'width'], None, True, "160")
                height = setConfigValueFromJson(None, self.jsonConfigBak, ['framegrabber', 'height'], None, True, "160")
                config.plugins.hyperioncontrol.framegrabberSize.value = str(width) + "," + str(height)
                setConfigValueFromJson(config.plugins.hyperioncontrol.framegrabberFreq, self.jsonConfigBak, ['framegrabber', 'frequency_Hz'], 1)

                # Smoothing
                setConfigValueFromJson(config.plugins.hyperioncontrol.smoothingupdateFrequency, self.jsonConfigBak, ['smoothing', 'updateFrequency'], 1)

                # BlackborderDetector(not in remote only in json)
                bbdEnable = setConfigValueFromJson(config.plugins.hyperioncontrol.bbdEnable, self.jsonConfigBak, ['blackborderdetector', 'enable'], calculate=None, onlyReturnValue=True)
                if str(bbdEnable).lower() == "true":
                    config.plugins.hyperioncontrol.bbdEnable.value = True
                else:
                    config.plugins.hyperioncontrol.bbdEnable.value = False
                setConfigValueFromJson(config.plugins.hyperioncontrol.bbdThreshold, self.jsonConfigBak, ['blackborderdetector', 'threshold'], calculate=100)

                # webif
                webConfigEnable = setConfigValueFromJson(config.plugins.hyperioncontrol.webConfigEnable, self.jsonConfigBak, ['webConfig', 'enable'], calculate=None, onlyReturnValue=True)
                if str(webConfigEnable).lower() == "true":
                    config.plugins.hyperioncontrol.webConfigEnable.value = True
                else:
                    config.plugins.hyperioncontrol.webConfigEnable.value = False
                setConfigValueFromJson(config.plugins.hyperioncontrol.webConfigPort, self.jsonConfigBak, ['webConfig', 'port'], calculate=1)

                # bootsequence
                effect = setConfigValueFromJson(config.plugins.hyperioncontrol.bootseqEffect, self.jsonConfigBak, ['bootsequence', 'effect'], calculate=None, onlyReturnValue=True)
                color = setConfigValueFromJson(config.plugins.hyperioncontrol.bootseqColor, self.jsonConfigBak, ['bootsequence', 'color'], calculate=None, onlyReturnValue=True)
                if effect:
                    config.plugins.hyperioncontrol.bootseqStart.value = "effect"
                    config.plugins.hyperioncontrol.bootseqEffect.choices.choices = [(effect, effect)]
                    config.plugins.hyperioncontrol.bootseqEffect.value = str(effect)
                else:
                    if color == [0, 0, 0]:
                        config.plugins.hyperioncontrol.bootseqStart.value = "off"
                    else:
                        config.plugins.hyperioncontrol.bootseqStart.value = "color"
                    effect = config.plugins.hyperioncontrol.bootseqEffect.default
                    config.plugins.hyperioncontrol.bootseqEffect.choices.choices = [(effect, effect)]
                    config.plugins.hyperioncontrol.bootseqEffect.value = str(effect)

                if color:
                    color = "#" + '{:02x}'.format(color[0]).upper() + '{:02x}'.format( color[1]).upper() + '{:02x}'.format(color[2]).upper()
                    colorName = self.ColorNames.getColorName(color)
                    config.plugins.hyperioncontrol.bootseqColor.choices.choices = [(color, colorName)]
                    config.plugins.hyperioncontrol.bootseqColor.value = str(color)
                setConfigValueFromJson(config.plugins.hyperioncontrol.bootseqDuration, self.jsonConfigBak, ['bootsequence', 'duration_ms'], calculate=1)

                self.jsonConfigBak = ""

                # reload ConfigList with new values
                self.buildConfig()
                self.session.open(MessageBox, _("The import from the file '/etc/hyperion/hyperion.config.json.bak' has been successfully completed."), MessageBox.TYPE_INFO, timeout=5)

            else:
                self.session.open(MessageBox, _("Import failed!\n\nFile '/etc/hyperion/hyperion.config.json.bak' not found!"), MessageBox.TYPE_INFO, timeout=5)

        except:
            import traceback
            traceback.print_exc()
            self.session.open(MessageBox, _("Error importing the 'hyperion.config.json.bak'!"), MessageBox.TYPE_INFO,
                              timeout=5)
            self.jsonConfigBak = ""

    def readValuesFromConfig(self):
        # read values from config.json
        self.jsonConfig = ""
        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
        else:
            # Hyperion, Hyperion-aml
            config_file_path = "/etc/hyperion/hyperion.config.json"

        try:
            if os.path.exists(config_file_path):
                valuesFromConfig = open(config_file_path).read()
                lines = valuesFromConfig.split("\n")
                newlines = []
                for line in lines:
                    cleanline = line.replace("\t", "")
                    if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
                        newlines.append(line)
                valuesFromConfig = "\n".join(newlines)
                self.jsonConfig = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)


                # bootsequence
                effect = self.jsonConfig['bootsequence']['effect']
                color = self.jsonConfig['bootsequence']['color']
                if effect:
                    config.plugins.hyperioncontrol.bootseqStart.value = "effect"
                    config.plugins.hyperioncontrol.bootseqEffect.choices.choices = [(effect, effect)]
                    config.plugins.hyperioncontrol.bootseqEffect.value = str(effect)
                else:
                    if color == [0, 0, 0]:
                        config.plugins.hyperioncontrol.bootseqStart.value = "off"
                    else:
                        config.plugins.hyperioncontrol.bootseqStart.value = "color"
                    effect = config.plugins.hyperioncontrol.bootseqEffect.default
                    config.plugins.hyperioncontrol.bootseqEffect.choices.choices = [(effect, effect)]
                    config.plugins.hyperioncontrol.bootseqEffect.value = str(effect)
                if color:
                    color = "#" + '{:02x}'.format(color[0]).upper() + '{:02x}'.format(
                        color[1]).upper() + '{:02x}'.format(color[2]).upper()
                    colorName = self.ColorNames.getColorName(color)
                    config.plugins.hyperioncontrol.bootseqColor.choices.choices = [(color, colorName)]
                    config.plugins.hyperioncontrol.bootseqColor.value = str(color)
                config.plugins.hyperioncontrol.bootseqDuration.value = str( self.jsonConfig['bootsequence']['duration_ms'])
        except:
            import traceback, sys
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))

            self.session.open(MessageBox, _("Error loading the '%(file)s'!\n\nError:\n%(err)s") % ({'file': config_file_path, 'err': error}), MessageBox.TYPE_INFO, timeout=5)

    def buildConfig(self):

        self.list = []
        if self.setupPage == "transform":
            self.list.append(getConfigListEntry(_("Hyperion-Remote"), ))
            self.list.append(getConfigListEntry(_("Hyperion remote type"), NoSave(config.plugins.hyperioncontrol.hyperionRemoteType)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("HSV (0-500, default=100)"), ))
            self.list.append(getConfigListEntry(_("Saturation Gain"), NoSave(config.plugins.hyperioncontrol.saturationGain)))
            self.list.append(getConfigListEntry(_("Value Gain"), NoSave(config.plugins.hyperioncontrol.valueGain)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("HSL"), ))
            self.list.append(getConfigListEntry(_("SaturationL Gain (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.saturationLGain)))
            self.list.append(getConfigListEntry(_("Luminance Gain (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.luminanceGain)))
            self.list.append(getConfigListEntry(_("Luminance Minimum (0-50, default=0)"), NoSave(config.plugins.hyperioncontrol.luminanceMinimum)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Gamma (0-500, default=250)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.gammaRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.gammaGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.gammaBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Threshold (0-100, default=0)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.thresholdRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.thresholdGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.thresholdBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Temperature (0-255, default=255)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.temperatureRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.temperatureGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.temperatureBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Red Adjust (0-255, default=255, 0, 0)"), ))
            self.list.append(getConfigListEntry(_("red (fix)"), NoSave(config.plugins.hyperioncontrol.redAdjustRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.redAdjustGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.redAdjustBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Green Adjust (0-255, default=0, 255, 0)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.greenAdjustRed)))
            self.list.append(getConfigListEntry(_("green (fix)"), NoSave(config.plugins.hyperioncontrol.greenAdjustGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.greenAdjustBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Blue Adjust (0-255, default=0, 0, 255)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blueAdjustRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blueAdjustGreen)))
            self.list.append(getConfigListEntry(_("blue (fix)"), NoSave(config.plugins.hyperioncontrol.blueAdjustBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Whitelevel (0-100, default=100)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.whitelevelRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.whitelevelGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.whitelevelBlue)))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Blacklevel (0-50, default=0)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blacklevelRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blacklevelGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.blacklevelBlue)))
            self.list.append(getConfigListEntry("", ))

        elif self.setupPage == "device":
            self.list.append(getConfigListEntry(_("Hyperion-Setup"), ))
            self.list.append(getConfigListEntry(_("Hyperion-Type"), config.plugins.hyperioncontrol.hyperionModeType))
            self.list.append(getConfigListEntry(_("Hyperion-Autostart"), config.plugins.hyperioncontrol.hyperionAutostart))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Extra lamp-Setup"), ))
            self.list.append(getConfigListEntry(_("Support extra lamp"), config.plugins.hyperioncontrol.extraLamp))
            if config.plugins.hyperioncontrol.extraLamp.value:
                self.list.append(getConfigListEntry(_("Hyperion Above Autostart"), config.plugins.hyperioncontrol.hyperionAboveAutostart))
                self.list.append(getConfigListEntry(_("Hyperion Below Autostart"), config.plugins.hyperioncontrol.hyperionBelowAutostart))
                self.list.append(getConfigListEntry(_("Hyperion Left Autostart"), config.plugins.hyperioncontrol.hyperionLeftAutostart))
                self.list.append(getConfigListEntry(_("Hyperion Right Autostart"), config.plugins.hyperioncontrol.hyperionRightAutostart))
            self.list.append(getConfigListEntry("", ))

            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                self.list.append(getConfigListEntry(_("External Grabber setup"), ))
#                self.list.append(getConfigListEntry(_("HDR compensation"), config.plugins.hyperioncontrol.externalGrabberHDR))
                self.list.append(getConfigListEntry("", ))
            
#            self.list.append(getConfigListEntry(_("Timer actions"), ))
#            self.list.append(getConfigListEntry("", ))

#            self.list.append(getConfigListEntry(_("User Profiles"), ))
#            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Idle actions"), ))
            self.list.append(getConfigListEntry(_("on Idle action"), config.plugins.hyperioncontrol.onIdleAction))
            if config.plugins.hyperioncontrol.onIdleAction.value == "effect":
                self.list.append(getConfigListEntry(_("... select idle effect (with ok key)"), config.plugins.hyperioncontrol.onIdleEffect))
            elif config.plugins.hyperioncontrol.onIdleAction.value == "color":
                self.list.append(getConfigListEntry(_("... select idle color (with ok key)"), config.plugins.hyperioncontrol.onIdleColor))
            elif config.plugins.hyperioncontrol.onIdleAction.value == "v4l2":
                self.list.append(getConfigListEntry(_("Device-Check"), config.plugins.hyperioncontrol.v4l2CheckOnOff))
                if config.plugins.hyperioncontrol.v4l2CheckOnOff.value:
                    self.list.append(getConfigListEntry(_("Device-Check-IP"), config.plugins.hyperioncontrol.v4l2CheckIP))

            if config.plugins.hyperioncontrol.onIdleAction.value == "off":
                if config.plugins.hyperioncontrol.hyperionAutostart.value:
                    self.list.append(getConfigListEntry(_("WLED TV off"), config.plugins.hyperioncontrol.hyperionTVIdle))
                if config.plugins.hyperioncontrol.extraLamp.value:
                    if config.plugins.hyperioncontrol.hyperionAboveAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED above off"), config.plugins.hyperioncontrol.hyperionAboveIdle))
                    if config.plugins.hyperioncontrol.hyperionBelowAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED below off"), config.plugins.hyperioncontrol.hyperionBelowIdle))
                    if config.plugins.hyperioncontrol.hyperionLeftAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED left off"), config.plugins.hyperioncontrol.hyperionLeftIdle))
                    if config.plugins.hyperioncontrol.hyperionRightAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED right off"), config.plugins.hyperioncontrol.hyperionRightIdle))
                    if config.plugins.hyperioncontrol.hyperionTVIdle.value == False:
                       if config.plugins.hyperioncontrol.hyperionAboveIdle.value == False or config.plugins.hyperioncontrol.hyperionBelowIdle.value == False or config.plugins.hyperioncontrol.hyperionLeftIdle.value == False or config.plugins.hyperioncontrol.hyperionRightIdle.value == False:
                           self.list.append(getConfigListEntry(_("Sync with WLED from TV"), config.plugins.hyperioncontrol.hyperionSyncIdle))
                    self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("leave Idle action"), config.plugins.hyperioncontrol.onIdlebackAction))
            if config.plugins.hyperioncontrol.onIdlebackAction.value == "effect":
                self.list.append(getConfigListEntry(_("... select start effect (with ok key)"), config.plugins.hyperioncontrol.onIdlebackEffect))
            elif config.plugins.hyperioncontrol.onIdlebackAction.value == "color":
                self.list.append(getConfigListEntry(_("... select start color (with ok key)"), config.plugins.hyperioncontrol.onIdlebackColor))
            self.list.append(getConfigListEntry("", ))
            
            if config.plugins.hyperioncontrol.creatorOutputWLED.value:
                self.list.append(getConfigListEntry(_("Standby actions"), ))
                if config.plugins.hyperioncontrol.hyperionAutostart.value:
                    self.list.append(getConfigListEntry(_("WLED TV off"), config.plugins.hyperioncontrol.hyperionTVStandby))
                if config.plugins.hyperioncontrol.extraLamp.value:
                    if config.plugins.hyperioncontrol.hyperionAboveAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED above off"), config.plugins.hyperioncontrol.hyperionAboveStandby))
                    if config.plugins.hyperioncontrol.hyperionBelowAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED below off"), config.plugins.hyperioncontrol.hyperionBelowStandby))
                    if config.plugins.hyperioncontrol.hyperionLeftAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED left off"), config.plugins.hyperioncontrol.hyperionLeftStandby))
                    if config.plugins.hyperioncontrol.hyperionRightAutostart.value:
                        self.list.append(getConfigListEntry(_("WLED right off"), config.plugins.hyperioncontrol.hyperionRightStandby))
            self.list.append(getConfigListEntry("", ))

        self["config"].list = self.list
        self["config"].l.setList(self.list)

    def getConfigName(self, subsection, configelement):
        for item in subsection.dict():
            conf = subsection.__getattr__(item)
            if conf == configelement:
                return item

    def changed(self):
        current = self["config"].getCurrent()
        configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])
        # print "[HyperionControl] - change:", configName

        if configName in ("extraLamp", "hyperionModeType", "hyperionRemoteType", "bootseqStart", "onIdleAction", "onIdlebackAction", "v4l2CheckOnOff", "hyperionAutostart", "hyperionAboveAutostart", "hyperionBelowAutostart", "hyperionLeftAutostart", "hyperionRightAutostart", "hyperionTVIdle", "hyperionAboveIdle", "hyperionBelowIdle", "hyperionLeftIdle", "hyperionRightIdle", "timerHyperion", "timerMondayOnOff", "timerTuesdayOnOff"):
            if configName == "hyperionModeType":
                if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                    config.plugins.hyperioncontrol.hyperionRemoteType.value = "hyperion-v4l2"
                elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
                    config.plugins.hyperioncontrol.hyperionRemoteType.value = "hyperion-aml"
                else:
                    config.plugins.hyperioncontrol.hyperionRemoteType.value = "hyperion"
                self.readValuesFromConfig()
                readValuesFromRemote(self)
                setChoicesHyperionRemote()
            elif configName == "extraLamp":
                setChoicesHyperionRemote()
            elif configName == "hyperionRemoteType":
                readValuesFromRemote(self)
            self.buildConfig()
            return

        cmd = ""
        if configName == "saturationGain":
            value = int(config.plugins.hyperioncontrol.saturationGain.value) / 100.0
            cmd = "-s %s" % value
        elif configName == "valueGain":
            value = int(config.plugins.hyperioncontrol.valueGain.value) / 100.0
            cmd = "-v %s" % value
        elif configName == "saturationLGain":
            value = int(config.plugins.hyperioncontrol.saturationLGain.value) / 100.0
            cmd = "-u %s" % value
        elif configName == "luminanceGain":
            value = int(config.plugins.hyperioncontrol.luminanceGain.value) / 100.0
            cmd = "-m %s" % value
        elif configName == "luminanceMinimum":
            value = int(config.plugins.hyperioncontrol.luminanceMinimum.value) / 100.0
            cmd = "-n %s" % value
        elif configName.startswith("gamma"):
            Red = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
            cmd = "-g '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("threshold"):
            Red = int(config.plugins.hyperioncontrol.thresholdRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.thresholdGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.thresholdBlue.value) / 100.0
            cmd = "-t '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("temperature"):
            Red = int(config.plugins.hyperioncontrol.temperatureRed.value)
            Green = int(config.plugins.hyperioncontrol.temperatureGreen.value)
            Blue = int(config.plugins.hyperioncontrol.temperatureBlue.value)
            cmd = "-Z '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("redAdjust"):
            Red = int(config.plugins.hyperioncontrol.redAdjustRed.value)
            Green = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
            Blue = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
            cmd = "-R '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("greenAdjust"):
            Red = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
            Green = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
            Blue = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
            cmd = "-G '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("blueAdjust"):
            Red = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
            Green = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
            Blue = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
            cmd = "-B '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("whitelevel"):
            Red = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
            cmd = "-w '%s %s %s'" % (Red, Green, Blue)
        elif configName.startswith("blacklevel"):
            Red = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0
            cmd = "-b '%s %s %s'" % (Red, Green, Blue)

        if cmd:
            send_CMD(cmd)

    def menu(self):
        list = []
        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            list.append((_("import complete config from hyperion.config-v4l2.json.bak"), "importCompleteConfig"))
            list.append((_("import LED-config from hyperion.config.json.ledimport"), "importLEDConfig"))
            list.append((_("save current config-file as hyperion.config-v4l2.json.bak"), "saveAsBak"))
            list.append((_("show content of current hyperion.config-v4l2.json"), "showConfig"))
        else:
            list.append((_("import complete config from hyperion.config.json.bak"), "importCompleteConfig"))
            list.append((_("import LED-config from hyperion.config.json.ledimport"), "importLEDConfig"))
            list.append((_("save current config-file as hyperion.config.json.bak"), "saveAsBak"))
            list.append((_("show content of current hyperion.config.json"), "showConfig"))

        self.session.openWithCallback(self.menuCallback, ChoiceBox, title=_("Hyperion Control - Setup-Options"), list=list, )

    def menuCallback(self, ret):
        ret = ret and ret[1]
        if ret:
            if ret == "importCompleteConfig":
                if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                    message_txt = _("Should import all data from existing 'hyperion.config-v4l2.json.bak'?\n\nAll setup values will be overwritten with the data from the 'hyperion.config-v4l2.json.bak'!\n\nThe read values can then be changed via the green button saved in the current 'hyperion.config-v4l2.json'.")
                else:
                    message_txt = _("Should import all data from existing 'hyperion.config.json.bak'?\n\nAll setup values will be overwritten with the data from the 'hyperion.config.json.bak'!\n\nThe read values can then be changed via the green button saved in the current 'hyperion.config.json'.")
                self.session.openWithCallback(self.importJsonBak, MessageBox, message_txt, MessageBox.TYPE_YESNO, default=False)
            elif ret == "importLEDConfig":
                if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                    message_txt = _("Should the LED config data import from exist 'hyperion.config.json.ledimport' ?\n\nAll LED values are taken with the data from the 'hyperion.config.json.ledimport' overwritten!\n\nThe read-in values can then be saved via the green button in the current 'hyperion.config-v4l2.json'.")
                else:
                    message_txt = _("Should the LED config data import from exist 'hyperion.config.json.ledimport' ?\n\nAll LED values are taken with the data from the 'hyperion.config.json.ledimport' overwritten!\n\nThe read-in values can then be saved via the green button in the current 'hyperion.config.json'.")
                self.session.openWithCallback(self.importLEDConfig, MessageBox, message_txt, MessageBox.TYPE_YESNO, default=False)
            elif ret == "saveAsBak":
                config_bakfile_path = "/etc/hyperion/hyperion.config.json.bak"
                message_txt = _("Should the current config file be backed up as 'hyperion.config.json.bak'?")
                if os.path.exists(config_bakfile_path):
                    message_txt += _("\n\nThe already existing 'hyperion.config.json.bak' will be overwritten!")
                self.session.openWithCallback(self.saveConfigAsBak, MessageBox, message_txt, MessageBox.TYPE_YESNO, default=False)
            elif ret == "showConfig":
                if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                    config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
                else:
                    config_file_path = "/etc/hyperion/hyperion.config.json"
                self.session.open(Console, _("Hyperion config file"), ["cat %s" % config_file_path])

    def saveConfigAsBak(self, ret):
        if ret:
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
                config_bakfile_path = "/etc/hyperion/hyperion.config-v4l2.json.bak"
            else:
                config_file_path = "/etc/hyperion/hyperion.config.json"
                config_bakfile_path = "/etc/hyperion/hyperion.config.json.bak"
            if os.path.exists(config_file_path):
                os.system('cp %s %s' % (config_file_path, config_bakfile_path))
                print "[HyperionControl] configfile saved as .bak"
                message_txt = _("The current config file was successful saved as '%s'." % config_bakfile_path)
                self.session.open(MessageBox, message_txt, MessageBox.TYPE_INFO, timeout=5)
            else:
                self.session.open(MessageBox, _("Backup as .bak failed!\n\nFile '%s' not found!" % config_file_path), MessageBox.TYPE_INFO, timeout=5)

    def importLEDConfig(self, ret):
        if not ret:
            return
        try:
            if not self.jsonConfig:
                self.session.open(MessageBox, _("There are no data from '/etc/hyperion/hyperion.config.json'!\n\nAn import is therefore not possible."), MessageBox.TYPE_INFO, timeout=5)
                return

            self.jsonConfigImport = ""
            config_file_path = "/etc/hyperion/hyperion.config.json.ledimport"
            if os.path.exists(config_file_path):
                # json.bak einlesen
                valuesFromConfig = open(config_file_path).read()
                lines = valuesFromConfig.split("\n")
                newlines = []
                for line in lines:
                    cleanline = line.replace("\t", "").lstrip()
                    if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
                        newlines.append(line)
                valuesFromConfig = "\n".join(newlines)
                print "=== config", valuesFromConfig
                self.jsonConfigImport = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)

                # LED-key aus import json in json übernehmen
                if "leds" in self.jsonConfigImport:
                    self.jsonConfig["leds"] = self.jsonConfigImport["leds"]
                    # print "=== LED-Config-Import erfolgreich"
                    self.session.open(MessageBox, _("The LED Config import was completed successfully.\n\nWhen saving via the green button, the acquired LED information can be saved in the current config."), MessageBox.TYPE_INFO, timeout=10)
                else:
                    self.session.open(MessageBox, _("The LED config import failed. No LED config could be found in the import file!"), MessageBox.TYPE_INFO, timeout=5)
                self.jsonConfigImport = ""
            else:
                self.session.open(MessageBox, _("Import failed!\n\nFile '/etc/hyperion/hyperion.config.json.ledimport' not found!"), MessageBox.TYPE_INFO, timeout=5)

        except:
            import traceback, sys
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
            self.session.open(MessageBox, _("Error when importing the LED config!\n\nError:\n%s") % error,
                              MessageBox.TYPE_INFO, timeout=5)
            self.jsonConfigImport = ""

    def info(self):
        current = self["config"].getCurrent()
        configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])

        message_txt = ""
        if configName == ("hyperionRemoteType"):
            message_txt = _("All active Hyperion are offered for live input of the values. In the color settings, you can use the OK button to display colored backgrounds. So it is very easy and quick to find optimal settings.\nStart with the white level, gamma and temperature. Finally, the colors. It is recommended to start the settings several times from the beginning.")
        elif configName in ("saturationGain", "saturationLGain"):
            message_txt = _("Saturation describes the relationship between color and gray content. It is expressed in percentages between 0% (gray) and 100% (fully saturated color).")
        elif configName in ("valueGain", "luminanceGain"):
            message_txt = _("With the brightness you adjust the brightness of the LED's.")
        elif configName == "luminanceMinimum":
            message_txt = _("With 'Luminance Minimum' you can  regulate the minimum brightness of the LEDs.")
        elif configName.startswith("whitelevel"):
            message_txt = _(
                "Whitelevel\n\nThe led strips channels dont have always the same perceived amount of color, that's why when we see a white image the leds are a bit reddish, greenish or bluish. To correct that behaviour, we need to limit the output or one or two channels so we perceive a perfect white.\n\nChange the 'Red', 'Green', 'Blue' values until your 'led white' fits your 'TV white'.\n\nHINT: Don't adjust the brightness of your leds with this setting!")
        elif configName.startswith("gamma"):
            message_txt = _(
                "Gamma correction\nEach gamma channel must be set so that the 'gray' of the LEDs does not look greenish/reddish/bluish.\nThis should be checked with each test image (calibration via Ok button in the setup option).")
        elif configName.startswith("threshold"):
            message_txt = _(
                "With this setting you delete the color information of the smallest values - so they are interpreted as black. If you, for example, set 'red' to a threshold of 10, all red values below are interpreted as black.")
        elif configName.startswith("temperature"):
            message_txt = _(
                "The 'temperature' changes the perception of all colors, either they are 'warmer' or 'colder'. With certain combinations you can set the Kelvin values for the color temperature as with LED lamps.\n\nExamples:\nDirect sunlight = 6,000K = 255, 255, 255\nCandle light = 1,900K = 255, 147, 41\nHalogen light = 3,200K = 255, 241, 224")
        elif "Adjust" in configName:
            message_txt = _(
                "Color correction\n\nIn this case, each color channel must be adjusted to match the color TV output in order to correct any color casts in the LEDs.\nThis should be checked with the test images (calibration via the Ok button for the respective setup option).")
        elif configName.startswith("blacklevel"):
            message_txt = _(
                "The black level determines how bright the LEDs are in black. This can be checked via the test pattern (calibration via the OK key in the setup option).")
        elif configName.startswith("framegrabber"):
            message_txt = _(
                "The video grabber image size specifies the width and height of the image which is sent to Hyperion and evaluated.\nThe frequency indicates the rhythm at which the pictures are sent (e.g., 10Hz = 10 frames per second).\nA larger image and a higher frequency results in a higher CPU load on the Dreambox.")
        elif configName.startswith("bbd"):
            message_txt = _(
                "With this option, the black border detection can be activated and adjusted. Black frames are ignored (as with some movies) and the image areas after that are used for the color definition of the LEDs.\n\nhint: If the LEDs do not show anything on black border when the black border detection is activated, the threshold value may be too low. If the threshold is set too high, even dark areas of the image in movies can be classified as black borders.")
        elif configName == "onIdleAction":
            message_txt = _(
                "Here you can set what should happen when activating or exiting idle mode.\nYou can set a color, an effect or do nothing. When leaving you can still choose the live mode (grabber).\nIf you have connected an external grabber, V4l2 is still available (external grabber). With this it is also possible to easily switch between internal and external grabber (1-click) if you have selected Hyperion type 'Hyperion internal grabber and set live mode when exiting. With hyperion off you can switch selcted hyperion to off.")
        elif configName == "v4l2CheckOnOff":
            message_txt = _(
                "Switch device test on/off.\nA TV device can be checked whether it is on or off.\nSwitch on your TV, the LEDs are started with external grabber. If the TV is switched off, the LEDs are switched off.")
        elif configName == "v4l2CheckIP":
            message_txt = _(
                "Here you can enter the IP address of your TV. It is then checked whether the device is switched on/off.")
        elif configName == "onIdlebackAction":
            message_txt = _(
                "Here you can set what should happen when activating or exiting idle mode.\nYou can set a color, an effect or do nothing. When leaving you can still choose the live mode (grabber).\nIf you have connected an external grabber, V4l2 is still available (external grabber). With this it is also possible to easily switch between internal and external grabber (1-click) if you have selected Hyperion type 'Hyperion internal grabber and set live mode when exiting.")
        elif configName == "hyperionModeType":
            message_txt = _(
                "Here you can set which Hyperion-Type Default should be used.")
        elif configName == "hyperionAutostart":
            message_txt = _(
                "If activated, the default Hyperion type is automatically started when the box is started.")
        elif configName == "extraLamp":
            message_txt = _(
                "If, in addition to the stripes on the TV, you also want to control lamps to match the TV set to yes, additional options will then be made available.")
        elif configName in ("hyperionAboveAutostart", "hyperionBelowAutostart", "hyperionLeftAutostart", "hyperionRightAutostart"):
            message_txt = _(
                "This option affects the overall behavior of the extra lamps. If Yes is selected, the respective lamp is taken into account. The WLED functions also depend on this.")
        elif configName in ("hyperionTVIdle", "hyperionAboveIdle", "hyperionBelowIdle", "hyperionLeftIdle", "hyperionRightIdle"):
            message_txt = _(
                "Select which lamps should be switched off when the box goes into idle mode.")
        elif configName in ("hyperionTVStandby", "hyperionAboveStandby", "hyperionBelowStandby", "hyperionLeftStandby", "hyperionRightStandby"):
            message_txt = _(
                "Select which lamps should be switched off when the box goes into idle mode.")
        elif configName == "hyperionSyncIdle":
            message_txt = _(
                "If the option is selected, the active lamps are synchronized with the effects of the WLED on the TV.")
        if message_txt:
            self.session.open(MessageBox, message_txt, MessageBox.TYPE_INFO, windowTitle="Hyperion Control Info")

    def keyOK(self):
        current = self["config"].getCurrent()
        configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])

        if configName.startswith(
                ("gamma", "whitelevel", "blacklevel", "redAdjust", "greenAdjust", "blueAdjust", "temperature")):
            self.session.open(HyperionControlColorSetup, configName, current[0])
        elif configName in ("bootseqColor", "onIdleColor", "onIdlebackColor"):
            self.session.openWithCallback(boundFunction(self.colorCallback, current[1]), hyperionControlColors, True)
        elif configName in ("bootseqEffect", "onIdleEffect", "onIdlebackEffect"):
            self.session.openWithCallback(boundFunction(self.effectsCallback, current[1]), hyperionControlEffects, True)

    def colorCallback(self, configObj=None, retvalue=False):
        if retvalue and configObj:
            configObj.choices.choices = [(retvalue[1], retvalue[0])]
            configObj.value = str(retvalue[1])

    def effectsCallback(self, configObj=None, retvalue=False):
        if retvalue:
            configObj.choices.choices = [(retvalue, retvalue)]
            configObj.value = str(retvalue)

    def creatorCallback(self):
        self.readValuesFromConfig()
        readValuesFromRemote(self)
        self.buildConfig()

    def key_red(self):
        self.session.openWithCallback(self.creatorCallback, hyperionControlCreatorSetup)

    def key_green(self):
        if self.setupPage == "transform":
            # Save Remote
            self.saveRemoteJson()
        else:
            self.saveJson()
            self.saveAll()  # save all config-values which don't use NoSave()
            config.plugins.hyperioncontrol.save()
            configfile.save()

    def key_yellow(self):
        if self.setupPage == "transform":
            self.setupPage = "device"
            self["key_yellow"].setText(_("Remote/Action"))
            self.setTitle(_("Hyperion Control - Setup"))
        else:
            self.setupPage = "transform"
            self["key_yellow"].setText(_("Setup"))
            self.setTitle(_("Hyperion Control - Remote/Action"))
        self.buildConfig()

    def key_blue(self):
        self.menu()

    def key_exit(self):
        self.saveAll()  # save all config-values which don't use NoSave()
        config.plugins.hyperioncontrol.save()
        configfile.save()
        self.close(self.session, "exit")


class hyperionControlSystem(Screen):
    if sz_w == 1920:
        skin = """
        <screen name="hyperionControlSystem" position="center,200" size="900,800" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="20,720" scale="stretch" size="200,70" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="240,720" size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="460,720"  size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="680,720" size="200,70" scale="stretch"/>
            <eLabel font="Regular;30" foregroundColor="white" name="" position="20,720" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel font="Regular;30" foregroundColor="white" name="" position="680,720" size="200,70" text="About" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="10,700" size="880,2" backgroundColor="grey" />
            <widget name="list" position="10,10" size="880,680" itemWidth="880" itemHeight="60" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""
    else:
        skin = """
        <screen name="hyperionControlSystem" position="center,80" size="590,620" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
            <eLabel font="Regular;18" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel font="Regular;18" foregroundColor="white" name="" position="440,570" size="140,40" text="About" valign="center" halign="center" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
            <widget name="list" position="10,10" size="575,540" itemWidth="575" itemHeight="40" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)

        self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'SetupActions'], {
            'cancel': self.key_exit,
            'ok': self.key_ok,
            'red': self.key_red,
            'green': self.key_green,
            'yellow': self.key_yellow,
            'blue': self.key_blue,
            'previousSection': self.pageUp,
            'nextSection': self.pageDown,
            'left': self.pageUp,
            'right': self.pageDown,
        })

        self.litems = []
        tlf = TemplatedListFonts()
        self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent,
                           margin=ePoint(0, 0), selectionZoom=1.3)
        self.ml.l.setFont(0, gFont(tlf.face(tlf.BIG), tlf.size(tlf.BIG)))
        self['list'] = self.ml
        self.onLayoutFinish.append(self.buildList)
        self.setTitle(_("Hyperion Control - SystemCTL"))

    def buildList(self):
        self.ml.l.setBuildFunc(self.showList, True)
        self.litems = []
        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            # external grabber
            self.litems.append(((_("stop external grabber-v4l2"), "stop-v4l2"),))
            self.litems.append(((_("start/restart external grabber-v4l2"), "restart-v4l2"),))
            self.litems.append(((_("status external grabber-v4l2"), "status-v4l2"),))
        elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
            # internal grabber
            self.litems.append(((_("stop internal grabber"), "stop-aml"),))
            self.litems.append(((_("start/restart internal grabber"), "restart-aml"),))
            self.litems.append(((_("status internal grabber"), "status-aml"),))
            self.litems.append(((_("status hyperion"), "status-hyperion"),))
        else:
            # hyperion
            self.litems.append(((_("stop hyperion"), "stop-hyperion"),))
            self.litems.append(((_("start/restart hyperion"), "restart-hyperion"),))
            self.litems.append(((_("status hyperion"), "status-hyperion"),))

        if config.plugins.hyperioncontrol.extraLamp.value:
            self.litems.append(((_("Lamp left start/stop"), "lampleft"), ))
            self.litems.append(((_("Lamp right start/stop"), "lampright"), ))
            self.litems.append(((_("Lamp above start/stop"), "lampabove"), ))
            self.litems.append(((_("Lamp below start/stop"), "lampbelow"), ))
        self.litems.append(((_("Start all LED"), "start-all"),))
        self.litems.append(((_("Stop all LED"), "stop-all"),))

        self["list"].setList(self.litems)

    def showList(self, item, selected):
        res = [item]
        (name, cmd) = item
        if sz_w == 1920:
            res.append(MultiContentEntryText(pos=(0, 0), size=(880, 60), font=0, text=name,
                                             flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
        else:
            res.append(MultiContentEntryText(pos=(0, 0), size=(575, 40), font=0, text=name,
                                             flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
        return res

    def pageUp(self):
        self["list"].pageUp()

    def pageDown(self):
        self["list"].pageDown()

    def key_red(self):
        self.close()

    def key_green(self):
        pass

    def key_yellow(self):
        pass

    def key_blue(self):
        about(self)

    def key_exit(self):
        self.close(self.session, "exit")

    def key_ok(self):
        exist = self['list'].getCurrent()
        if exist == None:
            return
        (name, returnValue) = self['list'].getCurrent()[0]
        if returnValue == "start-aml":
            self.session.open(Console, _("start internal grabber"), ["%s start hyperionaml" % hyperionremote_sh])
        elif returnValue == "stop-aml":
            self.session.open(Console, _("stop internal grabber"), ["%s stop hyperion && systemctl stop hyperionaml" % hyperionremote_sh])
        elif returnValue == "restart-aml":
            self.session.open(Console, _("restart internal grabber"), ["%s restart hyperion && systemctl restart hyperionaml" % hyperionremote_sh])
        elif returnValue == "status-aml":
            self.session.open(Console, _("status internal grabber"), ["%s status hyperionaml" % hyperionremote_sh])
        elif returnValue == "start-hyperion":
            self.session.open(Console, _("start hyperion"), ["%s start hyperion" % hyperionremote_sh])
        elif returnValue == "stop-hyperion":
            self.session.open(Console, _("stop hyperion"), ["%s stop hyperion" % hyperionremote_sh])
        elif returnValue == "restart-hyperion":
            self.session.open(Console, _("restart hyperion"), ["%s restart hyperion" % hyperionremote_sh])
        elif returnValue == "status-hyperion":
            self.session.open(Console, _("status hyperion"), ["%s status hyperion" % hyperionremote_sh])
        elif returnValue == "disable-hyperion":
            self.session.open(Console, _("disable hyperion"), ["%s disable hyperion" % hyperionremote_sh])
        elif returnValue == "enable-auto":
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                self.session.open(Console, _("enable external grabber"),
                                  ["%s enable hyperion-v4l2.service" % hyperionremote_sh])
            elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion":
                self.session.open(Console, _("enable hyperion Kodi/Boblight server"),
                                  ["%s enable hyperion.service" % hyperionremote_sh])
            else:
                self.session.open(Console, _("enable internal grabber"),
                                  ["%s enable systemd-hyperionaml.timer" % hyperionremote_sh])
        elif returnValue == "disable-auto":
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                self.session.open(Console, _("disable external grabber"),
                                  ["%s disable systemd-hyperion-v4l2.timer" % hyperionremote_sh])
            else:
                self.session.open(Console, _("disable internal grabber"),
                                  ["%s disable systemd-hyperionaml.timer" % hyperionremote_sh])
        elif returnValue == "status-auto":
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                self.session.open(Console, _("status internal grabber"),
                                  ["%s status hyperion-v4l2.service" % hyperionremote_sh])
            elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion":
                self.session.open(Console, _("status hyperion Kodi/Boblight server"),
                                  ["%s status hyperion.service" % hyperionremote_sh])
            else:
                self.session.open(Console, _("status internal grabber"),
                                  ["%s status systemd-hyperionaml.timer" % hyperionremote_sh])
        elif returnValue == "start-v4l2":
            self.session.open(Console, _("start internal grabber"), ["%s start hyperion-v4l2" % hyperionremote_sh])
        elif returnValue == "stop-v4l2":
            self.session.open(Console, _("stop internal grabber"), ["%s stop hyperion-v4l2" % hyperionremote_sh])
        elif returnValue == "restart-v4l2":
            self.session.open(Console, _("restart internal grabber"), ["%s restart hyperion-v4l2" % hyperionremote_sh])
        elif returnValue == "status-v4l2":
            self.session.open(Console, _("status internal grabber"), ["%s status hyperion-v4l2" % hyperionremote_sh])

        elif returnValue is "lampleft":
            if readLampLeft_Device():
                self.session.open(Console, "stop hyperion-left", ["%s stop hyperion-left" % hyperionremote_sh])
            else:
                self.session.open(Console, "restart hyperion-left", ["%s restart hyperion-left" % hyperionremote_sh])
        elif returnValue is "lampright":
            if readLampRight_Device():
                self.session.open(Console, "stop hyperion-right", ["%s stop hyperion-right" % hyperionremote_sh])
            else:
                self.session.open(Console, "restart hyperion-right", ["%s restart hyperion-right" % hyperionremote_sh])
        elif returnValue is "lampabove":
            if readLampAbove_Device():
                self.session.open(Console, "stop hyperion-above", ["%s stop hyperion-above" % hyperionremote_sh])
            else:
                self.session.open(Console, "restart hyperion-above", ["%s restart hyperion-above" % hyperionremote_sh])
        elif returnValue is "lampbelow":
            if readLampBelow_Device():
                self.session.open(Console, "stop hyperion-below", ["%s stop hyperion-below" % hyperionremote_sh])
            else:
                self.session.open(Console, "restart hyperion-below", ["%s restart hyperion-below" % hyperionremote_sh])

        elif returnValue is "lampleft_auto":
            if readLampLeftAuto_Device():
                self.session.open(Console, "disable hyperion-left", ["%s disable hyperion-left" % hyperionremote_sh])
            else:
                self.session.open(Console, "enable hyperion-left", ["%s enable hyperion-left" % hyperionremote_sh])
        elif returnValue is "lampright_auto":
            if readLampRightAuto_Device():
                self.session.open(Console, "disable hyperion-right", ["%s disable hyperion-right" % hyperionremote_sh])
            else:
                self.session.open(Console, "enable hyperion-right", ["%s enable hyperion-right" % hyperionremote_sh])
        elif returnValue is "lampabove_auto":
            if readLampAboveAuto_Device():
                self.session.open(Console, "disable hyperion-above", ["%s disable hyperion-above" % hyperionremote_sh])
            else:
                self.session.open(Console, "enable hyperion-above", ["%s enable hyperion-above" % hyperionremote_sh])
        elif returnValue is "lampbelow_auto":
            if readLampBelowAuto_Device():
                self.session.open(Console, "disable hyperion-below", ["%s disable hyperion-below" % hyperionremote_sh])
            else:
                self.session.open(Console, "enable hyperion-below", ["%s enable hyperion-below" % hyperionremote_sh])
        elif returnValue == "stop-all":
            cmd_list = ["%s stop hyperion" % hyperionremote_sh,
                        "%s stop hyperionaml" % hyperionremote_sh,
                        "%s stop hyperion-v4l2" % hyperionremote_sh,
                        "%s stop hyperion-left" % hyperionremote_sh,
                        "%s stop hyperion-right" % hyperionremote_sh,
                        "%s stop hyperion-above" % hyperionremote_sh,
                        "%s stop hyperion-below" % hyperionremote_sh
                        ]
            for cmd in cmd_list:
                os.system(cmd)
            self.session.open(MessageBox, _("All Hyperion types have been stopped"), MessageBox.TYPE_INFO, windowTitle="Hyperion Control System")
        elif returnValue == "start-all":
            cmd = ""
            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
                cmd += "%s restart hyperion-v4l2" % hyperionremote_sh
            elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
                cmd += "%s restart hyperion && systemctl restart hyperionaml" % hyperionremote_sh
            else:
                cmd += "%s restart hyperion" % hyperionremote_sh
            if config.plugins.hyperioncontrol.extraLamp.value:
                if config.plugins.hyperioncontrol.hyperionAboveAutostart.value:
                    cmd += " && %s restart hyperion-above" % hyperionremote_sh
                if config.plugins.hyperioncontrol.hyperionBelowAutostart.value:
                    cmd += " && %s restart hyperion-below" % hyperionremote_sh
                if config.plugins.hyperioncontrol.hyperionRightAutostart.value:
                    cmd += " && %s restart hyperion-right" % hyperionremote_sh
                if config.plugins.hyperioncontrol.hyperionLeftAutostart.value:
                    cmd += " && %s restart hyperion-left" % hyperionremote_sh
            os.system(cmd)
            self.session.open(MessageBox, _("All Hyperion types have started"), MessageBox.TYPE_INFO, windowTitle="Hyperion Control System")


class hyperionControlEffects(Screen):
    if sz_w == 1920:
        skin = """
        <screen name="hyperionControlEffects" position="center,200" size="900,800" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="20,720" scale="stretch" size="200,70" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="240,720" size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="460,720"  size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="680,720" size="200,70" scale="stretch"/>
            <eLabel font="Regular;30" foregroundColor="white" name="" position="20,720" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="10,700" size="880,2" backgroundColor="grey" />
            <widget name="list" position="10,10" size="880,680" itemWidth="880" itemHeight="60" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""
    else:
        skin = """
        <screen name="hyperionControlEffects" position="center,80" size="590,620" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
            <eLabel font="Regular;22" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
            <widget name="list" position="10,10" size="575,540" itemWidth="575" itemHeight="40" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""

    def __init__(self, session, OnlyReturnEffect=False):
        Screen.__init__(self, session)
        self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'SetupActions', ], {
            'cancel': self.key_exit,
            'ok': self.key_ok,
            'red': self.key_red,
            'green': self.key_green,
            'yellow': self.key_yellow,
            'blue': self.key_blue,
            'previousSection': self.pageUp,
            'nextSection': self.pageDown,
            'left': self.pageUp,
            'right': self.pageDown,
        })
        self.setTitle(_("Hyperion Control - Effects"))
        self.OnlyReturnEffect = OnlyReturnEffect

        tlf = TemplatedListFonts()
        self.litems = [('Blue mood blobs',), ('Cinema brighten lights',), ('Cinema dim lights',), ('Cold mood blobs',),
                       ('Color traces',), ('Full color mood blobs',), ('Green mood blobs',), ('Knight rider',),
                       ('Police Lights Single',), ('Police Lights Solid',), ('Rainbow mood',), ('Rainbow swirl',),
                       ('Rainbow swirl fast',), ('Random',), ('Red mood blobs',), ('Running dots',), ('Snake',),
                       ('Sparks',), ('Sparks Color',), ('Strobe Raspbmc',), ('Strobe blue',), ('Strobe white',),
                       ('Warm mood blobs',), ('X-Mas',), ('UDP listener',), ('UDP multicast listener',),
                       ('System Shutdown',)]
        self.ml = MenuList(self.litems, mode=eListbox.layoutGrid, content=eListboxPythonMultiContent,
                           margin=ePoint(0, 0), selectionZoom=1.2)
        self.ml.l.setFont(0, gFont(tlf.face(tlf.BIG), tlf.size(tlf.BIG)))
        self['list'] = self.ml

        self.onLayoutFinish.append(self.buildList)

    def buildList(self):
        self.ml.l.setBuildFunc(self.showList, True)
        self["list"].setList(self.litems)

    def showList(self, item, selected):
        res = [item]
        if sz_w == 1920:
            res.append(MultiContentEntryText(pos=(0, 0), size=(880, 60), font=0, text=item,
                                             flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
        else:
            res.append(MultiContentEntryText(pos=(0, 0), size=(575, 40), font=0, text=item,
                                             flags=RT_HALIGN_CENTER | RT_VALIGN_CENTER))
        return res

    def pageUp(self):
        self["list"].pageUp()

    def pageDown(self):
        self["list"].pageDown()

    def key_red(self):
        self.close()

    def key_green(self):
        pass

    def key_yellow(self):
        pass

    def key_blue(self):
        pass

    def key_ok(self):
        exist = self['list'].getCurrent()
        if exist == None:
            return
        effect = self['list'].getCurrent()[0]
        print effect
        if self.OnlyReturnEffect:
            self.close(effect)
            return
        cmd = "--effect '%s'" % effect
        send_CMD(cmd)

    def key_exit(self):
        self.close()


class hyperionStart(Screen):
    if sz_w == 1920:
        skin = """
        <screen name="hyperionStart" position="center,center" size="900,650">
            <widget name="above" position="416,5" size="68,80" scale="stretch" />
            <widget name="above_auto" position="486,75" size="30,25" scale="stretch" />
            <widget name="below" position="416,420" size="68,80" scale="stretch" />
            <widget name="below_auto" position="486,495" size="30,25" scale="stretch" />
            <widget name="left" position="82,300" size="68,80" scale="stretch" />
            <widget name="left_auto" position="150,375" size="30,25" scale="stretch" />
            <widget name="right" position="750,300" size="68,80" scale="stretch" />
            <widget name="right_auto" position="818,375" size="30,25" scale="stretch" />
            <widget name="status" position="220,120" size="460,270" font="Regular;28" foregroundColor="black" backgroundColor="white" valign="center" halign="center"/>
            <widget name="stripes" position="216,116" size="468,278" scale="stretch" />
            <widget name="stripes_auto" position="645,360" size="30,25" scale="stretch" />
            <widget name="logo" position="595,125" size="80,80" />
            <eLabel name="" position="10,550" size="880,2" backgroundColor="grey" />
            <ePixmap pixmap="skin_default/buttons/red.png" position="10,560" size="210,70" scale="stretch" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="230,560" size="210,70" scale="stretch" />
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="450,560" size="210,70" scale="stretch" />
            <ePixmap pixmap="skin_default/buttons/blue.png" position="670,560" size="210,70" scale="stretch" />
            <widget source="key_red" render="Label" backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,560"  size="210,70" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" transparent="1" valign="center" zPosition="1" />
            <widget source="key_green" render="Label" backgroundColor="#1f771f" font="Regular;30" halign="center" position="230,560" size="210,70" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"  transparent="1" valign="center" zPosition="1" />
            <widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="450,560" size="210,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget source="key_blue" render="Label"  font="Regular;30" foregroundColor="white" position="670,560" size="210,70" valign="center" halign="center" zPosition="1" backgroundColor="#18188b" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <widget name="version" font="Regular;30" position="15,500" size="400,40" />
            <ePixmap pixmap="skin_default/buttons/key_menu.png" position="785,490" size="90,45" scale="stretch" />
        </screen>"""
    else:
        skin = """
        <screen name="hyperionStart" position="center,center" size="600,500">
            <widget name="above" position="277,3" size="45,53" scale="stretch" />
            <widget name="above_auto" position="324,50" size="20,16" scale="stretch" />
            <widget name="below" position="277,280" size="45,53" scale="stretch" />
            <widget name="below_auto" position="324,330" size="20,16" scale="stretch" />
            <widget name="left" position="54,200" size="45,53" scale="stretch" />
            <widget name="left_auto" position="100,250" size="20,16" scale="stretch" />
            <widget name="right" position="500,200" size="45,53" scale="stretch" />
            <widget name="right_auto" position="545,250" size="20,16" scale="stretch" />
            <widget name="status" position="146,80" size="306,180" font="Regular;16" foregroundColor="black" backgroundColor="white" valign="center" halign="center"/>
            <widget name="stripes" position="144,77" size="311,185" scale="stretch" />
            <widget name="stripes_auto" position="430,240" size="20,16" scale="stretch" />
            <widget name="logo" position="396,83" size="53,53" />
            <eLabel name="" position="5,440" size="590,2" backgroundColor="grey" />
            <ePixmap pixmap="skin_default/buttons/red.png" position="5,450" size="145,40" scale="stretch" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="155,450" size="145,40" scale="stretch" />
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="305,450" size="145,40" scale="stretch" />
            <ePixmap pixmap="skin_default/buttons/blue.png" position="455,450" size="145,40" scale="stretch" />
            <widget source="key_red" render="Label" backgroundColor="#9f1313" font="Regular;18" halign="center" position="5,450"  size="145,40" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" transparent="1" valign="center" zPosition="1" />
            <widget source="key_green" render="Label" position="155,450" size="145,40" zPosition="1" font="Regular;18" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
            <widget source="key_yellow" render="Label" font="Regular;18" foregroundColor="white" position="305,450" size="145,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget source="key_blue" render="Label" font="Regular;18" foregroundColor="white" position="455,450" size="145,40" valign="center" halign="center" backgroundColor="#18188b" zPosition="1" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <widget name="version" font="Regular;22" position="15,400" size="300,25" />
            <ePixmap pixmap="skin_default/buttons/key_menu.png" position="497,395" size="72,36" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)

        self["above"] = Pixmap()
        self["above_auto"] = Pixmap()
        self["below"] = Pixmap()
        self["below_auto"] = Pixmap()
        self["left"] = Pixmap()
        self["left_auto"] = Pixmap()
        self["right"] = Pixmap()
        self["right_auto"] = Pixmap()
        self["logo"] = Pixmap()
        self["stripes"] = Pixmap()
        self["stripes_auto"] = Pixmap()
        self["version"] = Label(_("v.") + "" + hyperioncontrol_version)
        self["status"] = Label(_("state:"))
        self["key_red"] = StaticText(_("Colors"))
        self["key_green"] = StaticText(_("Effects"))
        self["key_yellow"] = StaticText(_("Setup"))
        self["key_blue"] = StaticText(_("Live Mode"))

        self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'InfobarEPGActions'],
                                    {
                                        'cancel': self.close,
                                        'red': self.key_red,
                                        'green': self.key_green,
                                        'yellow': self.key_yellow,
                                        'blue': self.key_blue,
                                        'menu': self.menu,
                                        'showEventInfo': self.about,
                                    })

        self.onLayoutFinish.append(self.layoutFinished)
        self.onShown.append(self.refreshStatus)
        self.runfirst = True

    def refreshStatus(self):
        if config.plugins.hyperioncontrol.extraLamp.value:
            png_off = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/LED-off.png"
            png_on = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/LED-on.png"

            png_off_auto = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/autostart_off.png"
            png_on_auto = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/autostart_on.png"
            for running in ["above", "below", "left", "right"]:
                auto = running + "_auto"
                if running == "above":
                    png = png_on if readLampAbove_Device() else png_off
                    png_auto = png_on_auto if readLampAboveAuto_Device() else png_off_auto
                elif running == "below":
                    png = png_on if readLampBelow_Device() else png_off
                    png_auto = png_on_auto if readLampBelowAuto_Device() else png_off_auto
                elif running == "left":
                    png = png_on if readLampLeft_Device() else png_off
                    png_auto = png_on_auto if readLampLeftAuto_Device() else png_off_auto
                else:
                    png = png_on if readLampRight_Device() else png_off
                    png_auto = png_on_auto if readLampRightAuto_Device() else png_off_auto
                self[running].instance.setPixmapFromFile(png)
                self[running].show()
                self[auto].instance.setPixmapFromFile(png_auto)
                self[auto].show()
        else:
            for running in ["above", "below", "left", "right"]:
                auto = running + "_auto"
                self[running].hide()
                self[auto].hide()

        status_txt = _("CPU") + "  %s\n\n"
        print "[HyperionControl] refresh status"

        aml_active = False
        fg_fps = "10"
        fg_size_w = "160"
        fg_size_h = "160"

        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            # hyperion-v4l2-status
            command = Command("systemctl status hyperion-v4l2")
            command.run(timeout=3)
            cmd_out = command.out
            if "Active: active" in cmd_out:
                status_txt += "  Hyperion-v4l2: %s\n" % _("active")
                png = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/stripes_on_468x278.png"
                pngl = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/Hyperion-on.png"
            else:
                status_txt += "  Hyperion-v4l2: %s\n" % _("inactive")
                png = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/stripes_off_468x278.png"
                pngl = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/Hyperion-off.png"
            self["logo"].instance.setPixmapFromFile(pngl)
            self["stripes"].instance.setPixmapFromFile(png)

        else:
            # hyperion-status
            command = Command("systemctl status hyperion")
            command.run(timeout=3)
            cmd_out = command.out
            if "Active: active" in cmd_out:
                status_txt += "  Hyperion: %s\n" % _("active")
                png = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/stripes_on_468x278.png"
                pngl = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/Hyperion-on.png"
            else:
                status_txt += "  Hyperion: %s\n" % _("inactive")
                png = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/stripes_off_468x278.png"
                pngl = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/Hyperion-off.png"
            self["logo"].instance.setPixmapFromFile(pngl)
            self["stripes"].instance.setPixmapFromFile(png)

            if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
                # hyperionaml-status
                command = Command("systemctl status hyperionaml")
                command.run(timeout=3)
                cmd_out = command.out
                if "Active: active" in cmd_out:
                    status_txt += _("  Internal grabber: %s\n") % _("active")
                    aml_active = True
                else:
                    status_txt += _("  Internal grabber: %s\n") % _("inactive")

                pos1 = cmd_out.find("hyperion-aml -f ")
                if pos1 > 0:
                    pos2 = cmd_out.find("\n", pos1 + 5)
                    if pos2 > 0:
                        row_txt_list = cmd_out[pos1:pos2].split(" ")
                        if len(row_txt_list) > 1 and row_txt_list[1] == "-f":
                            fg_fps = row_txt_list[2]
                        if len(row_txt_list) > 3 and row_txt_list[3] == "--width":
                            fg_size_w = row_txt_list[4]
                        if len(row_txt_list) > 5 and row_txt_list[5] == "--height":
                            fg_size_h = row_txt_list[6]

        # systemd-hyperionaml.timer or hyperion-v4l2.service
        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            cmd = "systemctl status hyperion-v4l2.service"
        elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
            cmd = "systemctl status systemd-hyperionaml.timer"
        else:
            cmd = "systemctl status hyperion.service"
        command = Command(cmd)
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            png = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/autostart_on.png"
        else:
            png = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/autostart_off.png"
        self["stripes_auto"].instance.setPixmapFromFile(png)

        # Modus
        self.jsonRemote = readValuesFromRemote(self, self.runfirst, False)
        self.runfirst = False
        if self.jsonRemote:
            if self.jsonRemote["activeEffects"] != []:
                status_txt += "  " + _("Mode: ") + _("Effects") + "\n\n"
            elif self.jsonRemote["activeLedColor"] != []:
                status_txt += "  " + _("Mode: ") + _("Colors") + "\n\n"
            elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion":
                status_txt += "  " + _("Mode: ") + _("Hyperion server") + "\n\n"
            else:
                status_txt += "  " + _("Mode: ") + _("Live Mode") + " " + _("or") + " " + _("Image Mode") + "\n\n"

        if aml_active:
            # get fps and size from framegrabber
            fg_size = "%sx%s" % (fg_size_w, fg_size_h)
            status_txt += "  " + _("Framegrabber") + ": " + fg_size + " " + _("FPS") + " " + fg_fps + "\n"
        elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            status_txt += "  " + _("Framegrabber") + ": V4l2"

        # use for CPU-usage
        self.lastTotal = 0
        self.lastIdle = 0
        cpu_value = self.getCPU()
        self.status_txt = status_txt
        self["status"].setText(self.status_txt % cpu_value)

        # CPU-refresh-Timer
        self.cpuTimer = eTimer()
        self.cpuTimer_conn = self.cpuTimer.timeout.connect(self.refreshCPU)
        self.cpuTimer.start(1000)

    def refreshCPU(self):
        try:
            cpu_value = self.getCPU()
            self["status"].setText(self.status_txt % cpu_value)
        except:
            import traceback, sys
            traceback.print_exc()

    def getCPU(self):
        try:
            if os.path.isfile("/proc/stat"):
                fields = [float(column) for column in os.popen("cat /proc/stat").readline().strip().split()[1:]]
                idle, total = fields[3], fields[0] + fields[1] + fields[2] + fields[3]
                idle_delta, total_delta = idle - self.lastIdle, total - self.lastTotal
                self.lastIdle, self.lastTotal = idle, total
                utilisation = 100.0 * (1.0 - idle_delta / total_delta)
                return format(round(utilisation, 2), "0.1f") + "%"
            else:
                return "No info"

        except:
            import traceback, sys
            traceback.print_exc()
            return "No info"

    def layoutFinished(self):
        self.setTitle(_("Hyperion Control"))

    def about(self):
        about(self)

    def key_red(self):
        self.session.open(hyperionControlColors)

    def key_green(self):
        self.session.open(hyperionControlEffects)

    def key_yellow(self):
        self.session.open(hyperionControlSetup)

    def key_blue(self):
        if self["key_blue"].getText():
            send_CMD('--clearall')
            self.session.open(MessageBox, _("The Live Mode has been activated."), MessageBox.TYPE_INFO, timeout=5)

    def menu(self):
        self.session.open(hyperionControlSystem)


class HyperionControlPictureBrowser(Screen):
    if sz_w == 1920:
        skin = """
        <screen name="HyperionControlPictureBrowser" position="center,170" size="1500,740" title="Hyperion Control Picture Browser">
            <ePixmap pixmap="Default-FHD/skin_default/buttons/red.svg" position="10,5" size="300,70" />
            <ePixmap pixmap="Default-FHD/skin_default/buttons/green.svg" position="310,5" size="300,70" />
            <widget backgroundColor="#9f1313" font="Regular;30" halign="center" position="10,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="300,70" source="key_red" transparent="1" valign="center" zPosition="1" />
            <widget backgroundColor="#1f771f" font="Regular;30" halign="center" position="310,5" render="Label" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2" size="300,70" source="key_green" transparent="1" valign="center" zPosition="1" />
            <eLabel backgroundColor="grey" position="10,80" size="900,1" />
            <eLabel backgroundColor="grey" position="920,5" size="1,725" />
            <widget name="picpreview" position="960,90" size="500,500" />
            <widget enableWrapAround="1" name="filelist" position="10,90" scrollbarMode="showOnDemand" size="900,630" />
        </screen>"""
    else:
        skin = """
        <screen name="HyperionControlPictureBrowser" position="center,120" size="920,430" title="Hyperion Control Picture Browser">
            <ePixmap pixmap="skin_default/buttons/red.png" position="10,5" size="200,40"/>
            <ePixmap pixmap="skin_default/buttons/green.png" position="210,5" size="200,40"/>
            <widget source="key_red" render="Label" position="10,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#9f1313" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
            <widget source="key_green" render="Label" position="210,5" size="200,40" zPosition="1" font="Regular;20" halign="center" valign="center" backgroundColor="#1f771f" transparent="1" foregroundColor="white" shadowColor="black" shadowOffset="-2,-2"/>
            <eLabel position="10,50" size="590,1" backgroundColor="grey"/>
            <eLabel position="610,5" size="1,420" backgroundColor="grey"/>
            <widget name="picpreview" position="620,80" size="290,290" />
            <widget name="filelist" position="10,60" size="590,360" enableWrapAround="1" scrollbarMode="showOnDemand"/>
        </screen>"""

    def __init__(self, session, current_path=None):
        Screen.__init__(self, session)
        if current_path:
            currDir = current_path
            if not os.path.exists(currDir):
                currDir = "/"
        else:
            currDir = None  # start with mountpoints-list

        self.filelist = FileList(currDir, matchingPattern="(?i)^.*\.(jpg|png)")
        self["filelist"] = self.filelist
        self["picpreview"] = Pixmap()
        self["filelist"].onSelectionChanged.append(self.selectionChanged)

        self["FilelistActions"] = ActionMap(["SetupActions"],
                                            {
                                                "save": self.ok,
                                                "ok": self.ok,
                                                "cancel": self.exit
                                            })
        self["key_red"] = StaticText(_("Cancel"))
        self["key_green"] = StaticText(_("OK"))

        self.ThumbTimer = eTimer()
        self.ThumbTimer_conn = self.ThumbTimer.timeout.connect(self.showThumb)

        self.onShown.append(self.setWindowTitle)

    def setWindowTitle(self):
        self.setTitle(_("Hyperion Control Picture Browser"))

    def ok(self):
        if self["filelist"].canDescent():  # isDir
            self["filelist"].descent()
        else:
            file_name = self["filelist"].getFilename()
            path = self["filelist"].getCurrentDirectory()
            self.exit(path + file_name)

    def showThumb(self):
        if not self["filelist"].canDescent():
            file_name = self["filelist"].getFilename()
            path = self["filelist"].getCurrentDirectory()
            # print "== picname", path + file_name
            self["picpreview"].instance.setPixmapFromFile(path + file_name)
            self["picpreview"].show()

    def selectionChanged(self):
        if self["picpreview"] is None:
            return

        if not self["filelist"].canDescent():
            self.ThumbTimer.start(400, True)
        else:
            self["picpreview"].hide()

    def exit(self, retvalue=None):
        self.ThumbTimer.stop()
        self.ThumbTimer_conn = None
        if self.filelist.getCurrentDirectory() is None:
            config.plugins.hyperioncontrol.lastpicpath.value = "/"
        else:
            config.plugins.hyperioncontrol.lastpicpath.value = self["filelist"].getCurrentDirectory()
        config.plugins.hyperioncontrol.save()

        self["picpreview"] = None
        self.close(retvalue)


class HyperionControlInfoScreen(Screen):
    IS_DIALOG = True

    if sz_w == 1920:
        skin = """
        <screen name="HyperionControlInfoScreen" position="center,150" size="1400,840">
            <widget name="infopicture" position="50,50" size="1300,740"/>
            <widget name="infotext" position="50,50" size="800,300"/>
        </screen>"""
    else:
        skin = """
        <screen name="HyperionControlInfoScreen" position="center,120" size="920,530">
            <widget name="infopicture" position="20,20" size="880,500"/>
            <widget name="infotext" position="50,50" size="800,300"/>
        </screen>"""

    def __init__(self, session, info_txt=""):
        Screen.__init__(self, session)

        self["infopicture"] = Pixmap()
        self["infotext"] = Label()
        self.info_txt = info_txt

        self["infopicture"].hide()
        self["infotext"].hide()

        self["FilelistActions"] = ActionMap(["SetupActions"],
                                            {
                                                "ok": self.ok,
                                                "cancel": self.exit
                                            })

        self.onShown.append(self.setWindowTitle)

    def setWindowTitle(self):
        self.setTitle(_("Hyperion Control Info"))
        path = "/usr/lib/enigma2/python/Plugins/Extensions/HyperionControl/pictures/"

        if self.info_txt.endswith("Deep"):
            filename = "image_area_depth_de.jpg"

        elif self.info_txt.endswith(("LedBegin", "LedDirection")):
            filename = "image_area_begin_de.jpg"

        elif self.info_txt.endswith("Dist"):
            filename = "image_area_distance_de.jpg"

        self["infopicture"].instance.setPixmapFromFile(path + filename)
        self["infopicture"].show()

    def ok(self):
        self.close()

    def exit(self):
        self.close()


class HyperionControlColorSetup(Screen, ConfigListScreen):
    if sz_w == 1920:
        skin = """
        <screen transparent="0" position="0,0" size="1920,1080" backgroundColor="#00FFFFFF" flags="wfNoBorder">
            <widget name="config" seperation="100" foregroundColor="#000000" backgroundColor="#FFFFFF" transparent="0" backgroundColorSelected="#004176b6" position="center,center" size="450,200" enableWrapAround="1" scrollbarMode="showOnDemand" />
            <widget name="pixmap" position="0,0" size="1920,1080" zPosition="-1" />
            <widget name="infotext" position="center,800" size="1920,100" font="Regular;30" foregroundColor="#000000" halign="center" transparent="1" />
        </screen>"""
    else:
        skin = """
        <screen position="0,0" size=" 1280,720" backgroundColor="#00FFFFFF" flags="wfNoBorder">
            <widget name="config" seperation="100" foregroundColor="#000000" backgroundColor="#FFFFFF" transparent="0" backgroundColorSelected="#004176b6" position="center,center" size="300,130" enableWrapAround="1" scrollbarMode="showOnDemand" />
            <widget name="pixmap" position="0,0" size="1280,720" zPosition="-1" />
            <widget name="infotext" position="center,550" size="1280,100" font="Regular;22" foregroundColor="#000000" halign="center" transparent="1" />
        </screen>"""

    def __init__(self, session, configName="", configText=""):
        Screen.__init__(self, session)

        self.configName = configName
        self.configText = configText

        # disable osd_alpha (GUI Transparenz)
        self.osd = config.av.osd_alpha.value
        config.av.osd_alpha.value = 255
        self["pixmap"] = Pixmap()
        self["pixmap"].hide()
        self["infotext"] = Label(_("change backgroundcolor with 'ok' key and close with 'exit' key"))

        self.list = []
        ConfigListScreen.__init__(self, self.list, session=session, on_change=self.changed)
        self["config"] = ConfigListHC(self.list, session=session)
        self.buildConfig()

        self["FilelistActions"] = ActionMap(["SetupActions", "ColorActions"],
                                            {
                                                "ok": self.keyOK,
                                                "cancel": self.exit,
                                            })

        if self.configName.startswith("whitelevel"):
            title = _("whitelevel")
            self["infotext"].setText(_("close with 'exit' key"))
        elif self.configName.startswith("blacklevel"):
            title = _("blacklevel")
            self["infotext"].setText(_("close with 'exit' key"))
        elif self.configName.startswith("redAdjust"):
            title = _("redAdjust")
        elif self.configName.startswith("greenAdjust"):
            title = _("greenAdjust")
        elif self.configName.startswith("blueAdjust"):
            title = _("blueAdjust")
        elif self.configName.startswith("gamma"):
            title = _("gamma")
        elif self.configName.startswith("temperature"):
            title = _("temperature")
        self.setTitle(title)

        self.onShown.append(self.setWindowTitle)

    def setWindowTitle(self):
        self.ColorIndex = 0
        if self.configName.startswith("whitelevel"):
            self.Colors = ("#FFFFFF",)
        if self.configName.startswith("blacklevel"):
            self.Colors = ("#000000",)
            self["infotext"].instance.setForegroundColor(parseColor("#FFFFFF"))
        elif self.configName.startswith("redAdjust"):
            self.Colors = ("#FF0000", "#FF00FF", "#FFFF00")
        elif self.configName.startswith("greenAdjust"):
            self.Colors = ("#00FF00", "#00FFFF", "#FFFF00")
        elif self.configName.startswith("blueAdjust"):
            self.Colors = ("#0000FF", "#00FFFF", "#FF00FF")
        elif self.configName.startswith("temperatureRed"):
            self.Colors = ("#FFFFFF", "#FF0000", "#00FF00", "#0000FF", "#00FFFF", "#FF00FF", "#FFFF00")
        elif self.configName.startswith("temperatureGreen"):
            self.Colors = ("#FFFFFF", "#00FF00", "#FF0000", "#0000FF", "#00FFFF", "#FF00FF", "#FFFF00")
        elif self.configName.startswith("temperatureBlue"):
            self.Colors = ("#FFFFFF", "#0000FF", "#FF0000", "#00FF00", "#00FFFF", "#FF00FF", "#FFFF00")
        elif self.configName.startswith("gamma"):
            self.Colors = ("#3F3F3F", "#7F7F7F", "#BFBFBF", "#000000")

        color = self.Colors[self.ColorIndex]
        self.instance.setBackgroundColor(parseColor(color))
        self["infotext"].instance.setBackgroundColor(parseColor(color))
        self["config"].instance.setBackgroundColor(parseColor("#7F7F7F"))

        cmd = "-c '%s'" % color
        send_CMD(cmd, master=True)

        # set selected config from setup-screen
        index = 0
        for configentry in self.list:
            if configentry[0] == self.configText:
                break
            index += 1
        self["config"].setCurrentIndex(index)

    def buildConfig(self):

        self.list = []
        if self.configName.startswith("whitelevel"):
            self.list.append(getConfigListEntry(_("Whitelevel (0-100, default=100)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.whitelevelRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.whitelevelGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.whitelevelBlue)))
        elif self.configName.startswith("blacklevel"):
            self.list.append(getConfigListEntry(_("Blacklevel (0-50, default=0)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blacklevelRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blacklevelGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.blacklevelBlue)))
        elif self.configName.startswith("temperature"):
            self.list.append(getConfigListEntry(_("temperature"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.temperatureRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.temperatureGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.temperatureBlue)))
        elif self.configName.startswith("redAdjust"):
            self.list.append(getConfigListEntry(_("Red Adjust (0-255, default=255, 0, 0)"), ))
            self.list.append(getConfigListEntry(_("red (fix)"), NoSave(config.plugins.hyperioncontrol.redAdjustRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.redAdjustGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.redAdjustBlue)))
        elif self.configName.startswith("greenAdjust"):
            self.list.append(getConfigListEntry(_("Green Adjust (0-255, default=0, 255, 0)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.greenAdjustRed)))
            self.list.append(getConfigListEntry(_("green (fix)"), NoSave(config.plugins.hyperioncontrol.greenAdjustGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.greenAdjustBlue)))
        elif self.configName.startswith("blueAdjust"):
            self.list.append(getConfigListEntry(_("Blue Adjust (0-255, default=0, 0, 255)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.blueAdjustRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.blueAdjustGreen)))
            self.list.append(getConfigListEntry(_("blue (fix)"), NoSave(config.plugins.hyperioncontrol.blueAdjustBlue)))
        elif self.configName.startswith("gamma"):
            self.list.append(getConfigListEntry(_("Gamma (0-500, default=250)"), ))
            self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.gammaRed)))
            self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.gammaGreen)))
            self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.gammaBlue)))

        self["config"].list = self.list
        self["config"].l.setList(self.list)

    def changed(self):
        current = self["config"].getCurrent()
        print "[HyperionControl] - change:", self.configName

        cmd = ""
        if self.configName.startswith("whitelevel"):
            Red = int(config.plugins.hyperioncontrol.whitelevelRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.whitelevelGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.whitelevelBlue.value) / 100.0
            cmd = "-w '%s %s %s'" % (Red, Green, Blue)
        elif self.configName.startswith("temperature"):
            Red = int(config.plugins.hyperioncontrol.temperatureRed.value)
            Green = int(config.plugins.hyperioncontrol.temperatureGreen.value)
            Blue = int(config.plugins.hyperioncontrol.temperatureBlue.value)
            cmd = "-Z '%s %s %s'" % (Red, Green, Blue)
        elif self.configName.startswith("redAdjust"):
            Red = int(config.plugins.hyperioncontrol.redAdjustRed.value)
            Green = int(config.plugins.hyperioncontrol.redAdjustGreen.value)
            Blue = int(config.plugins.hyperioncontrol.redAdjustBlue.value)
            cmd = "-R '%s %s %s'" % (Red, Green, Blue)
        elif self.configName.startswith("greenAdjust"):
            Red = int(config.plugins.hyperioncontrol.greenAdjustRed.value)
            Green = int(config.plugins.hyperioncontrol.greenAdjustGreen.value)
            Blue = int(config.plugins.hyperioncontrol.greenAdjustBlue.value)
            cmd = "-G '%s %s %s'" % (Red, Green, Blue)
        elif self.configName.startswith("blueAdjust"):
            Red = int(config.plugins.hyperioncontrol.blueAdjustRed.value)
            Green = int(config.plugins.hyperioncontrol.blueAdjustGreen.value)
            Blue = int(config.plugins.hyperioncontrol.blueAdjustBlue.value)
            cmd = "-B '%s %s %s'" % (Red, Green, Blue)
        elif self.configName.startswith("gamma"):
            Red = int(config.plugins.hyperioncontrol.gammaRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.gammaGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.gammaBlue.value) / 100.0
            cmd = "-g '%s %s %s'" % (Red, Green, Blue)
        elif self.configName.startswith("blacklevel"):
            Red = int(config.plugins.hyperioncontrol.blacklevelRed.value) / 100.0
            Green = int(config.plugins.hyperioncontrol.blacklevelGreen.value) / 100.0
            Blue = int(config.plugins.hyperioncontrol.blacklevelBlue.value) / 100.0
            cmd = "-b '%s %s %s'" % (Red, Green, Blue)

        if cmd:
            send_CMD(cmd)

    def keyOK(self):
        # switch colors for gamma-settings
        if self.configName.startswith(("gamma", "redAdjust", "greenAdjust", "blueAdjust", "temperature")):
            self.ColorIndex += 1
            if not self.configName.startswith("temperature") and self.ColorIndex in (4, 5):
                if self.ColorIndex == 4: direction = ePixmap.GRADIENT_HORIZONTAL
                if self.ColorIndex == 5: direction = ePixmap.GRADIENT_VERTICAL
                color = "#000000"
                color1 = "#000000"
                color2 = "#FFFFFF"
            elif not self.configName.startswith("temperature") and self.ColorIndex in (6, 7):
                if self.ColorIndex == 6: direction = ePixmap.GRADIENT_HORIZONTAL
                if self.ColorIndex == 7: direction = ePixmap.GRADIENT_VERTICAL
                color = "#000000"
                color2 = "#000000"
                color1 = "#FFFFFF"

            if self.configName.startswith("temperature"):
                if self.ColorIndex > 6: self.ColorIndex = 0
            elif not self.configName.startswith("gamma"):
                if self.ColorIndex > 2: self.ColorIndex = 0
            elif self.ColorIndex > 7:
                self.ColorIndex = 0

            if self.configName.startswith("temperature"):
                color = self.Colors[self.ColorIndex]
                self.instance.setBackgroundColor(parseColor(color))
                self["infotext"].instance.setBackgroundColor(parseColor(color))
                cmd = "-c '%s'" % color
                send_CMD(cmd)

            # if first 4 normal colors
            elif self.ColorIndex < 4:
                self["pixmap"].hide()
                color = self.Colors[self.ColorIndex]
                if color in ("#FF000000", "#000000"):
                    self["infotext"].instance.setForegroundColor(parseColor("#FFFFFF"))
                else:
                    self["infotext"].instance.setForegroundColor(parseColor("#000000"))
                self.instance.setBackgroundColor(parseColor(color))
                self["infotext"].instance.setBackgroundColor(parseColor(color))
                cmd = "-c '%s'" % color
                send_CMD(cmd)

            # if 4 gradient pixmap colors
            else:
                self.instance.setBackgroundColor(parseColor(color))
                self["infotext"].instance.setBackgroundColor(parseColor(color))
                self["pixmap"].instance.setGradient(parseColor(color1), parseColor(color2), direction)
                self["pixmap"].show()
                self["infotext"].hide()
                self["config"].hide()

                self.container = eConsoleAppContainer()
                width = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[0])
                height = int(str(config.plugins.hyperioncontrol.framegrabberSize.value).split(",")[1])
                cmd = 'dreamboxctl screenshot -f /tmp/HyperionOSD.jpg -iw %s -ih %s && hyperion-remote -i /tmp/HyperionOSD.jpg' % (
                    width, height)
                self.container_appClosed_conn = self.container.appClosed.connect(self.eConsoleFinished)
                self.container.execute(cmd)
            self.instance.invalidate()

    def eConsoleFinished(self, status):
        self["infotext"].show()
        self["config"].show()

    def exit(self):
        # reenable osd_alpha (GUI Transparenz)
        config.av.osd_alpha.value = self.osd
        send_CMD("--clearall", master=True)
        self.close()


class hyperionControlCreatorSetup(Screen, ConfigListScreen):
    if sz_w == 1920:
        skin = """
        <screen name="hyperionControlCreatorSetup" position="center,100" size="900,950" >
            <ePixmap pixmap="skin_default/buttons/red.png" position="20,870" scale="stretch" size="200,70" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="240,870" size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="460,870"  size="200,70" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="680,870" size="200,70" scale="stretch"/>
            <eLabel name="" position="10,850" size="880,2" backgroundColor="grey" />
            <eLabel font="Regular;30" foregroundColor="white" name="" position="20,870" size="200,70" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel font="Regular;30" foregroundColor="white" name="" position="240,870" size="200,70" text="Save" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
            <widget source="key_yellow" render="Label" font="Regular;30" foregroundColor="white" position="460,870" size="200,70" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <widget name="config" seperation="100" position="10,10" size="880,808" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""
    else:
        skin = """
        <screen name="hyperionControlCreatorSetup" position="center,80" size="590,620">
            <ePixmap pixmap="skin_default/buttons/red.png" position="5,570" scale="stretch" size="140,40" />
            <ePixmap pixmap="skin_default/buttons/green.png" position="150,570" size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/yellow.png" position="295,570"  size="140,40" scale="stretch"/>
            <ePixmap pixmap="skin_default/buttons/blue.png" position="440,570" size="140,40" scale="stretch"/>
            <eLabel font="Regular;18" foregroundColor="white" name="" position="5,570" size="140,40" text="Exit" valign="center" halign="center" backgroundColor="#9f1313" shadowColor="black" shadowOffset="-2,-2" transparent="1" />
            <eLabel font="Regular;18" foregroundColor="white" name="" position="150,570" size="140,40" text="Save" valign="center" halign="center" backgroundColor="#1f771f" transparent="1" shadowColor="black" shadowOffset="-2,-2" />
            <widget source="key_yellow" render="Label" font="Regular;18" foregroundColor="white" position="295,570" size="140,40" valign="center" halign="center" backgroundColor="#a08500" shadowColor="black" shadowOffset="-2,-2" zPosition="1" transparent="1" />
            <eLabel name="" position="5,560" size="580,2" backgroundColor="grey" />
            <widget name="config" position="10,10" seperation="100" size="570,545" enableWrapAround="1" scrollbarMode="showOnDemand" />
        </screen>"""

    def __init__(self, session):
        Screen.__init__(self, session)
        self['actions'] = ActionMap(['OkCancelActions', 'ColorActions', 'MenuActions', 'InfobarEPGActions'], {
            'cancel': self.key_exit,
            'red': self.key_red,
            'green': self.key_green,
            'yellow': self.restartHyperionService,
            'showEventInfo': self.info,
            # 'ok': self.key_ok,
        })

        self.list = []
        ConfigListScreen.__init__(self, self.list, session=session, on_change=self.changed)
        self["config"] = ConfigListHC(self.list, session=session)
        self.setTitle(_("Hyperion Control - Config Creator"))
        self["key_yellow"] = StaticText(_("Restart"))

        # load new choices
        choices = [("hyperion", "Hyperion Boblight/KodiServer"), ("hyperion-aml", _("Hyperion-internal grabber"))]
        if readV4l2_Device():
            choices.append(("hyperion-v4l2", _("Hyperion-external grabber")))
        if config.plugins.hyperioncontrol.extraLamp.value:
            for item in [("hyperion-above", _("Hyperion Above")), ("hyperion-below", _("Hyperion Below")), ("hyperion-left", _("Hyperion Left")), ("hyperion-right", _("Hyperion Right"))]:
                choices.append(item)
        config.plugins.hyperioncontrol.creatorHyperionModeType = ConfigSelection(choices=choices, default="hyperion-aml")

        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            config.plugins.hyperioncontrol.creatorHyperionModeType.value = "hyperion-v4l2"
        elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-aml":
            config.plugins.hyperioncontrol.creatorHyperionModeType.value = "hyperion-aml"
        else:
            config.plugins.hyperioncontrol.creatorHyperionModeType.value = "hyperion"

        self.onShown.append(self.readData)
        self.lastLEDcountTxt = None

        self.ColorNames = ColorNames()

    def readData(self):
        print "=== onshown"
        self.onShown.remove(self.readData)
        self.readValuesFromConfig()
        self.buildConfig()

    def buildConfig(self):

        self.list = []

        self.list.append(getConfigListEntry(_("Hyperion-Setup"), ))
        self.list.append(getConfigListEntry(_("Hyperion-Type"), config.plugins.hyperioncontrol.creatorHyperionModeType))
        self.list.append(getConfigListEntry("", ))
        self.list.append(getConfigListEntry(_("Device-Setup"), ))
        self.list.append(getConfigListEntry(_("Device-Typ"), config.plugins.hyperioncontrol.creatorDeviceType))
        self.list.append(getConfigListEntry(_("WLED-Wifi"), config.plugins.hyperioncontrol.creatorOutputWLED))
        self.list.append(getConfigListEntry(_("Config-Name"), config.plugins.hyperioncontrol.creatorConfigName))
        if config.plugins.hyperioncontrol.creatorDeviceType.value == "udpraw":
            self.list.append(
                getConfigListEntry(_("Device-IP (fix port: 19446)"), config.plugins.hyperioncontrol.creatorOutputIP))
        elif config.plugins.hyperioncontrol.creatorDeviceType.value in ("philipshue", "philipshueentertainment"):
            self.list.append(getConfigListEntry(_("Device-IP"), config.plugins.hyperioncontrol.creatorOutputIP))
        if config.plugins.hyperioncontrol.creatorDeviceType.value in (
        "sedu", "adalight", "AdalightApa102", "atmo", "karate"):
            self.list.append(getConfigListEntry(_("Output"), config.plugins.hyperioncontrol.creatorOutputTYPE))
            self.list.append(getConfigListEntry(_("delayafterconnect (0-20, default=0)"),
                                                config.plugins.hyperioncontrol.creatorDelayafterconnect))
        if config.plugins.hyperioncontrol.creatorDeviceType.value == "philipshue":
            self.list.append(getConfigListEntry(_("UserName"), config.plugins.hyperioncontrol.creatorPhe_username))
            self.list.append(getConfigListEntry(_("LightIDs"), config.plugins.hyperioncontrol.creatorPhe_lightIds))
            self.list.append(getConfigListEntry(_("Switch Off On Black"),
                                                config.plugins.hyperioncontrol.creatorPhe_switchOffOnBlack))
        if config.plugins.hyperioncontrol.creatorDeviceType.value == "philipshueentertainment":
            self.list.append(getConfigListEntry(_("UserName"), config.plugins.hyperioncontrol.creatorPhe_username))
            self.list.append(getConfigListEntry(_("ClientKey"), config.plugins.hyperioncontrol.creatorPhe_clientkey))
            self.list.append(getConfigListEntry(_("LightIDs"), config.plugins.hyperioncontrol.creatorPhe_lightIds))
            self.list.append(getConfigListEntry(_("GroupID"), config.plugins.hyperioncontrol.creatorPhe_groupId))
            self.list.append(getConfigListEntry(_("Switch Off On Black"),
                                                config.plugins.hyperioncontrol.creatorPhe_switchOffOnBlack))
        if config.plugins.hyperioncontrol.creatorDeviceType.value in ("philipshue", "philipshueentertainment"):
            self.list.append(getConfigListEntry(_("colorOrder"), config.plugins.hyperioncontrol.creatorColorOrder))
        else:
            self.list.append(getConfigListEntry(_("colorOrder"), config.plugins.hyperioncontrol.creatorColorOrder))
            self.list.append(getConfigListEntry(_("Baudrate"), config.plugins.hyperioncontrol.creatorBaudrate))
        if config.plugins.hyperioncontrol.creatorDeviceType.value in ("udpraw"):
            self.list.append(getConfigListEntry(_("Protocol"), config.plugins.hyperioncontrol.creatorProtocol))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("HSV (0-500, default=100)"), ))
        self.list.append(getConfigListEntry(_("Saturation Gain"), NoSave(config.plugins.hyperioncontrol.creatorSaturationGain)))
        self.list.append(getConfigListEntry(_("Value Gain"), NoSave(config.plugins.hyperioncontrol.creatorValueGain)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("HSL"), ))
        self.list.append(getConfigListEntry(_("SaturationL Gain (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.creatorSaturationLGain)))
        self.list.append(getConfigListEntry(_("Luminance Gain (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.creatorLuminanceGain)))
        self.list.append(getConfigListEntry(_("Luminance Minimum (0-50, default=0)"), NoSave(config.plugins.hyperioncontrol.creatorLuminanceMinimum)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Gamma (0-500, default=250)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorGammaRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorGammaGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorGammaBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Threshold (0-100, default=0)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorThresholdRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorThresholdGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorThresholdBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Temperature (0-255, default=255)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorTemperatureRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorTemperatureGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorTemperatureBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Red Adjust (0-255, default=255, 0, 0)"), ))
        self.list.append(getConfigListEntry(_("red (fix)"), NoSave(config.plugins.hyperioncontrol.creatorRedAdjustRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorRedAdjustGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorRedAdjustBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Green Adjust (0-255, default=0, 255, 0)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorGreenAdjustRed)))
        self.list.append(
            getConfigListEntry(_("green (fix)"), NoSave(config.plugins.hyperioncontrol.creatorGreenAdjustGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorGreenAdjustBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Blue Adjust (0-255, default=0, 0, 255)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorBlueAdjustRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorBlueAdjustGreen)))
        self.list.append(
            getConfigListEntry(_("blue (fix)"), NoSave(config.plugins.hyperioncontrol.creatorBlueAdjustBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Whitelevel (0-100, default=100)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorWhitelevelRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorWhitelevelGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorWhitelevelBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Blacklevel (0-50, default=0)"), ))
        self.list.append(getConfigListEntry(_("red"), NoSave(config.plugins.hyperioncontrol.creatorBlacklevelRed)))
        self.list.append(getConfigListEntry(_("green"), NoSave(config.plugins.hyperioncontrol.creatorBlacklevelGreen)))
        self.list.append(getConfigListEntry(_("blue"), NoSave(config.plugins.hyperioncontrol.creatorBlacklevelBlue)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("LED-Setup"), ))
        self.ledSetupIndex = self.list.index((_("LED-Setup"),))
        # LED

        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
            pos_txt = "Above"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
            pos_txt = "Below"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
            pos_txt = "Left"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
            pos_txt = "Right"
        else:
            # hyperion and aml, v4l2
            pos_txt = "Default"

        led_begin = config.plugins.hyperioncontrol.__getattr__("creator%sLedBegin" % pos_txt)
        led_direction = config.plugins.hyperioncontrol.__getattr__("creator%sLedDirection" % pos_txt)
        led_top = config.plugins.hyperioncontrol.__getattr__("creator%sLedTop" % pos_txt)
        led_bottom = config.plugins.hyperioncontrol.__getattr__("creator%sLedDown" % pos_txt)
        led_right = config.plugins.hyperioncontrol.__getattr__("creator%sLedRight" % pos_txt)
        led_left = config.plugins.hyperioncontrol.__getattr__("creator%sLedLeft" % pos_txt)
        led_gap = config.plugins.hyperioncontrol.__getattr__("creator%sLedBottomGap" % pos_txt)

        led_corner_left_top = config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerTopLeft" % pos_txt)
        led_corner_right_top = config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerTopRight" % pos_txt)
        led_corner_left_bottom = config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerBottomLeft" % pos_txt)
        led_corner_right_bottom = config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerBottomRight" % pos_txt)

        h_pic_deep = config.plugins.hyperioncontrol.__getattr__("creator%sHPicDeep" % pos_txt)
        v_pic_deep = config.plugins.hyperioncontrol.__getattr__("creator%sVPicDeep" % pos_txt)
        h_pic_dist = config.plugins.hyperioncontrol.__getattr__("creator%sHPicDist" % pos_txt)
        v_pic_dist = config.plugins.hyperioncontrol.__getattr__("creator%sVPicDist" % pos_txt)
        p_pic_overlap = config.plugins.hyperioncontrol.__getattr__("creator%sPicOverlap" % pos_txt)

        self.list.append(getConfigListEntry(_("LED begin"), led_begin))
        if led_begin.value != "0":
            self.list.append(getConfigListEntry(_("LED direction"), led_direction))
            self.list.append(getConfigListEntry(_("Number of LEDs top"), led_top))
            self.list.append(
                getConfigListEntry(_("Number of LEDs down"), led_bottom))
            self.list.append(
                getConfigListEntry(_("Number of LEDs right"), led_right))
            self.list.append(
                getConfigListEntry(_("Number of LEDs left"), led_left))
            self.list.append(
                getConfigListEntry(_("Number of LEDs gap bottom"), led_gap))
            self.list.append(
                getConfigListEntry(_("Corner LED left top"), led_corner_left_top))
            self.list.append(
                getConfigListEntry(_("Corner LED right top"), led_corner_right_top))
            self.list.append(
                getConfigListEntry(_("Corner LED left bottom"), led_corner_left_bottom))
            self.list.append(getConfigListEntry(_("Corner LED right bottom"), led_corner_right_bottom))

            self.list.append(getConfigListEntry("", ))
            self.list.append(getConfigListEntry(_("LED-Image Area"), ))
            self.list.append(
                getConfigListEntry(_("horizontal depth (default=8%)"), h_pic_deep))
            self.list.append(
                getConfigListEntry(_("vertical depth (default=5%)"), v_pic_deep))
            self.list.append(getConfigListEntry(_("horizontal distance (default=0%)"), h_pic_dist))
            self.list.append(getConfigListEntry(_("vertical distance (default=0%)"), v_pic_dist))
            self.list.append(getConfigListEntry(_("overlap (default=0%)"), p_pic_overlap))
            self.writeLedCount()

            self.list.append(getConfigListEntry("", ))
            self.list.append(getConfigListEntry(_("Smoothing"), ))
            self.list.append(getConfigListEntry(_("Update Frequency (default=20 Hz)"), NoSave(config.plugins.hyperioncontrol.creatorSmoothingupdateFrequency)))
        self.list.append(getConfigListEntry("", ))

        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-aml":
            self.list.append(getConfigListEntry(_("Framegrabber internal"), ))
            self.list.append(getConfigListEntry(_("Size (default=160x160)"), NoSave(config.plugins.hyperioncontrol.creatorFramegrabberSize)))
            self.list.append(getConfigListEntry(_("Frequency in Hz (5-25, default=10)"), NoSave(config.plugins.hyperioncontrol.creatorFramegrabberFreq)))
            self.list.append(getConfigListEntry("", ))

        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-v4l2":
            self.list.append(getConfigListEntry(_("Framegrabber external"), ))
            self.list.append(getConfigListEntry(_("FrameDecimation (default=2)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2frameDecimation)))
            self.list.append(getConfigListEntry(_("SizeDecimation (default=8)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2sizeDecimation)))
            self.list.append(getConfigListEntry(_("CropLeft (default=0)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2cropLeft)))
            self.list.append(getConfigListEntry(_("CropRight (default=0)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2cropRight)))
            self.list.append(getConfigListEntry(_("CropTop (default=0)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2cropTop)))
            self.list.append(getConfigListEntry(_("CropBottom (default=0)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2cropBottom)))
            self.list.append(getConfigListEntry(_("RedSignalThreshold Adjust (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2redSignalThreshold)))
            self.list.append(getConfigListEntry(_("GreenSignalThreshold Adjust (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2greenSignalThreshold)))
            self.list.append(getConfigListEntry(_("BlueSignalThreshold Adjust (0-100, default=100)"), NoSave(config.plugins.hyperioncontrol.creatorV4l2blueSignalThreshold)))
            self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("BlackBorderDetection"), ))
        self.list.append(getConfigListEntry(_("enabled"), NoSave(config.plugins.hyperioncontrol.creatorBbdEnable)))
        self.list.append(getConfigListEntry(_("threshold (1-20, default=4)"), NoSave(config.plugins.hyperioncontrol.creatorBbdThreshold)))
        self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Webif"), ))
        self.webConfigIndex = self.list.index((_("Webif"),))
        self.list.append(
            getConfigListEntry(_("enabled"), NoSave(config.plugins.hyperioncontrol.creatorWebConfigEnable)))
        self.list.append(
            getConfigListEntry(_("port (default=8099)"), NoSave(config.plugins.hyperioncontrol.creatorWebConfigPort)))
        self.list.append(getConfigListEntry("", ))
        self.writeWepifIP()

        # xbmc
        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion":
            self.list.append(getConfigListEntry(_("Kodi Video Checker"), ))
            self.list.append(getConfigListEntry(_("enabled"), config.plugins.hyperioncontrol.creatorXbmcVideoChecker))
            if config.plugins.hyperioncontrol.creatorXbmcVideoChecker.value:
                self.list.append(getConfigListEntry(_("Kodi-IP"), config.plugins.hyperioncontrol.creatorXbmcIP))
            self.list.append(getConfigListEntry("", ))

            self.list.append(getConfigListEntry(_("Boblight Server"), ))
            self.list.append(getConfigListEntry(_("enabled"), config.plugins.hyperioncontrol.creatorBoblightServer))
            if config.plugins.hyperioncontrol.creatorBoblightServer.value:
                self.list.append(getConfigListEntry(_("port (default=19333)"), config.plugins.hyperioncontrol.creatorBoblightServerPort))
            self.list.append(getConfigListEntry("", ))

        self.list.append(getConfigListEntry(_("Bootsequence"), ))
        self.list.append(
            getConfigListEntry(_("start with"), NoSave(config.plugins.hyperioncontrol.creatorBootseqStart)))
        if config.plugins.hyperioncontrol.creatorBootseqStart.value != "off":
            if config.plugins.hyperioncontrol.creatorBootseqStart.value == "effect":
                self.list.append(getConfigListEntry(_("... select start effect (with ok key)"),
                                                    NoSave(config.plugins.hyperioncontrol.creatorBootseqEffect)))
            else:
                self.list.append(getConfigListEntry(_("... select start color (with ok key)"),
                                                    NoSave(config.plugins.hyperioncontrol.creatorBootseqColor)))
            self.list.append(
                getConfigListEntry(_("duration in ms"), NoSave(config.plugins.hyperioncontrol.creatorBootseqDuration)))

        self["config"].list = self.list
        self["config"].l.setList(self.list)

    def readValuesFromConfig(self):
        # read values from config.json
        self.jsonConfig = ""

        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
            config_file_path = "/etc/hyperion/hyperion-above.config.json"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
            config_file_path = "/etc/hyperion/hyperion-below.config.json"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
            config_file_path = "/etc/hyperion/hyperion-left.config.json"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
            config_file_path = "/etc/hyperion/hyperion-right.config.json"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-v4l2":
            config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
        else:
            # Hyperion, Hyperion-aml
            config_file_path = "/etc/hyperion/hyperion.config.json"

        if not os.path.exists(config_file_path):
            config_file_path = "/etc/hyperion/hyperion.config.json-example"
        try:
            if os.path.exists(config_file_path):
                valuesFromConfig = open(config_file_path).read()
                lines = valuesFromConfig.split("\n")
                newlines = []
                for line in lines:
                    cleanline = line.replace("\t", "")
                    if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
                        newlines.append(line)
                valuesFromConfig = "\n".join(newlines)
                self.jsonConfig = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)

                if self.jsonConfig.get("grabber-v4l2"):
                    # Framegrabber v4l2 external grabber
                    config.plugins.hyperioncontrol.creatorV4l2frameDecimation.value = str(self.jsonConfig['grabber-v4l2']['frameDecimation'])
                    config.plugins.hyperioncontrol.creatorV4l2sizeDecimation.value = str(self.jsonConfig['grabber-v4l2']['sizeDecimation'])
                    config.plugins.hyperioncontrol.creatorV4l2cropLeft.value = str(self.jsonConfig['grabber-v4l2']['cropLeft'])
                    config.plugins.hyperioncontrol.creatorV4l2cropRight.value = str(self.jsonConfig['grabber-v4l2']['cropRight'])
                    config.plugins.hyperioncontrol.creatorV4l2cropTop.value = str(self.jsonConfig['grabber-v4l2']['cropTop'])
                    config.plugins.hyperioncontrol.creatorV4l2cropBottom.value = str(self.jsonConfig['grabber-v4l2']['cropBottom'])
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorV4l2redSignalThreshold, self.jsonConfig, ['grabber-v4l2', 'redSignalThreshold'], 100)
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorV4l2greenSignalThreshold, self.jsonConfig, ['grabber-v4l2', 'greenSignalThreshold'], 100)
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorV4l2blueSignalThreshold, self.jsonConfig, ['grabber-v4l2', 'blueSignalThreshold'], 100)

                if self.jsonConfig.get('framegrabber'):
                    # Framgegrabber internal
                    config.plugins.hyperioncontrol.creatorFramegrabberSize.value = str(self.jsonConfig['framegrabber']['width']) + "," + str(self.jsonConfig['framegrabber']['height'])
                    config.plugins.hyperioncontrol.creatorFramegrabberFreq.value = str(int(self.jsonConfig['framegrabber']['frequency_Hz']))

                # BlackborderDetector(not in remote only in json)
                bbdenabled = self.jsonConfig['blackborderdetector']['enable']
                if str(bbdenabled).lower() == "true":
                    config.plugins.hyperioncontrol.creatorBbdEnable.value = True
                else:
                    config.plugins.hyperioncontrol.creatorBbdEnable.value = False
                config.plugins.hyperioncontrol.creatorBbdThreshold.value = str(
                    int(float(self.jsonConfig['blackborderdetector']['threshold']) * 100))

                # webif
                if self.jsonConfig.get('webConfig'):
                    webConfigEnabled = self.jsonConfig['webConfig']['enable']
                    if str(webConfigEnabled).lower() == "true":
                        config.plugins.hyperioncontrol.creatorWebConfigEnable.value = True
                    else:
                        config.plugins.hyperioncontrol.creatorWebConfigEnable.value = False
                    config.plugins.hyperioncontrol.creatorWebConfigPort.value = int(self.jsonConfig['webConfig']['port'])

                # device
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorDeviceType, self.jsonConfig, ['device', 'type'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorConfigName, self.jsonConfig, ['device', 'name'])
                if config.plugins.hyperioncontrol.creatorDeviceType.value in ("udpraw", "philipshue", "philipshueentertainment"):
                    outputIP = setConfigValueFromJson(config.plugins.hyperioncontrol.creatorOutputIP, self.jsonConfig, ['device', 'output'], onlyReturnValue=True)
                    outputIP = outputIP.replace(":19446", "").split(".")
                    outputIP = [int(d) for d in outputIP]
                    config.plugins.hyperioncontrol.creatorOutputIP.value = outputIP
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorColorOrder, self.jsonConfig, ['device', 'colorOrder'])
                if config.plugins.hyperioncontrol.creatorDeviceType.value == "philipshue":
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorPhe_username, self.jsonConfig, ['device', 'username'])
                    lightIDs = setConfigValueFromJson(config.plugins.hyperioncontrol.creatorPhe_lightIds, self.jsonConfig, ['device', 'lightIds'], onlyReturnValue=True)
                if config.plugins.hyperioncontrol.creatorDeviceType.value == "philipshueentertainment":
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorPhe_username, self.jsonConfig, ['device', 'username'])
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorPhe_clientkey, self.jsonConfig, ['device', 'clientkey'])
                    lightIDs = setConfigValueFromJson(config.plugins.hyperioncontrol.creatorPhe_lightIds, self.jsonConfig, ['device', 'lightIds'], onlyReturnValue=True)
                    config.plugins.hyperioncontrol.creatorPhe_lightIds.value = ', '.join(map(str, lightIDs))
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorPhe_groupId, self.jsonConfig, ['device', 'groupId'])
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorDelayafterconnect, self.jsonConfig, ['device', 'delayafterconnect'])
                if config.plugins.hyperioncontrol.creatorDeviceType.value in ("philipshue", "philipshueentertainment"):
                    self.jsonConfig['device']['switchOffOnBlack'] = str(config.plugins.hyperioncontrol.creatorPhe_switchOffOnBlack.value)
                else:
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBaudrate, self.jsonConfig, ['device', 'rate'], calculate=1)
                if config.plugins.hyperioncontrol.creatorDeviceType.value in ("udpraw"):
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorProtocol, self.jsonConfig, ['device', 'protocol'])

                # outputType
                if config.plugins.hyperioncontrol.creatorDeviceType.value in ("sedu", "adalight", "AdalightApa102", "atmo", "karate"):
                    setConfigValueFromJson(config.plugins.hyperioncontrol.creatorOutputTYPE, self.jsonConfig, ['device', 'output'])

                # bootsequence
                effect = self.jsonConfig['bootsequence']['effect']
                color = self.jsonConfig['bootsequence']['color']
                if effect:
                    config.plugins.hyperioncontrol.creatorBootseqStart.value = "effect"
                    config.plugins.hyperioncontrol.creatorBootseqEffect.choices.choices = [(effect, effect)]
                    config.plugins.hyperioncontrol.creatorBootseqEffect.value = str(effect)
                else:
                    if color == [0, 0, 0]:
                        config.plugins.hyperioncontrol.creatorBootseqStart.value = "off"
                    else:
                        config.plugins.hyperioncontrol.creatorBootseqStart.value = "color"
                    effect = config.plugins.hyperioncontrol.creatorBootseqEffect.default
                    config.plugins.hyperioncontrol.creatorBootseqEffect.choices.choices = [(effect, effect)]
                    config.plugins.hyperioncontrol.creatorBootseqEffect.value = str(effect)
                if color:
                    color = "#" + '{:02x}'.format(color[0]).upper() + '{:02x}'.format(
                        color[1]).upper() + '{:02x}'.format(color[2]).upper()
                    colorName = self.ColorNames.getColorName(color)
                    config.plugins.hyperioncontrol.creatorBootseqColor.choices.choices = [(color, colorName)]
                    config.plugins.hyperioncontrol.creatorBootseqColor.value = str(color)
                config.plugins.hyperioncontrol.creatorBootseqDuration.value = str(
                    self.jsonConfig['bootsequence']['duration_ms'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBaudrate, self.jsonConfig,
                                       ['device', 'rate'],
                                       calculate=1, default=config.plugins.hyperioncontrol.creatorBaudrate.value)

                # hsv
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorSaturationGain, self.jsonConfig,
                                       ['color', 'transform', 0, 'hsv', 'saturationGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorValueGain, self.jsonConfig,
                                       ['color', 'transform', 0, 'hsv', 'valueGain'], 100)
                # hsl
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorSaturationLGain, self.jsonConfig,
                                       ['color', 'transform', 0, 'hsl', 'saturationLGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorLuminanceGain, self.jsonConfig,
                                       ['color', 'transform', 0, 'hsl', 'luminanceGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorLuminanceMinimum, self.jsonConfig,
                                       ['color', 'transform', 0, 'hsl', 'luminanceMinimum'], 100)
                # red
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorGammaRed, self.jsonConfig,
                                       ['color', 'transform', 0, 'red', 'gamma'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorThresholdRed, self.jsonConfig,
                                       ['color', 'transform', 0, 'red', 'threshold'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorTemperatureRed, self.jsonConfig,
                                       ['color', 'temperature', 0, 'correctionValues', 'red'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorRedAdjustRed, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureRed', 'redChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorGreenAdjustRed, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureGreen', 'redChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBlueAdjustRed, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureBlue', 'redChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorWhitelevelRed, self.jsonConfig,
                                       ['color', 'transform', 0, 'red', 'whitelevel'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBlacklevelRed, self.jsonConfig,
                                       ['color', 'transform', 0, 'red', 'blacklevel'], 100)

                # green
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorGammaGreen, self.jsonConfig,
                                       ['color', 'transform', 0, 'green', 'gamma'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorThresholdGreen, self.jsonConfig,
                                       ['color', 'transform', 0, 'green', 'threshold'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorTemperatureGreen, self.jsonConfig,
                                       ['color', 'temperature', 0, 'correctionValues', 'green'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorRedAdjustGreen, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureRed', 'greenChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorGreenAdjustGreen, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureGreen', 'greenChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBlueAdjustGreen, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureBlue', 'greenChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorWhitelevelGreen, self.jsonConfig,
                                       ['color', 'transform', 0, 'green', 'whitelevel'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBlacklevelGreen, self.jsonConfig,
                                       ['color', 'transform', 0, 'green', 'blacklevel'], 100)

                # blue
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorGammaBlue, self.jsonConfig,
                                       ['color', 'transform', 0, 'blue', 'gamma'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorThresholdBlue, self.jsonConfig,
                                       ['color', 'transform', 0, 'blue', 'threshold'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorTemperatureBlue, self.jsonConfig,
                                       ['color', 'temperature', 0, 'correctionValues', 'blue'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorRedAdjustBlue, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureRed', 'blueChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorGreenAdjustBlue, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureGreen', 'blueChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBlueAdjustBlue, self.jsonConfig,
                                       ['color', 'channelAdjustment', 0, 'pureBlue', 'blueChannel'])
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorWhitelevelBlue, self.jsonConfig,
                                       ['color', 'transform', 0, 'blue', 'whitelevel'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorBlacklevelBlue, self.jsonConfig,
                                       ['color', 'transform', 0, 'blue', 'blacklevel'], 100)

                setConfigValueFromJson(config.plugins.hyperioncontrol.creatorSmoothingupdateFrequency, self.jsonConfig,
                                       ['color', 'smoothing', 'updateFrequency'], default="20", calculate=1)
        except:
            import traceback, sys
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))

            self.session.open(MessageBox, _("Error loading the '%(file)s'!\n\nError:\n%(err)s") % ({'file': config_file_path, 'err': error}), MessageBox.TYPE_INFO, timeout=5)

    def getConfigName(self, subsection, configelement):
        for item in subsection.dict():
            conf = subsection.__getattr__(item)
            if conf == configelement:
                return item

    def writeWepifIP(self):
        if config.plugins.hyperioncontrol.creatorWebConfigEnable.value:
            ipA = self.getCurrentIP()
            ipA = "http://" + ipA + ":%s" % int(config.plugins.hyperioncontrol.creatorWebConfigPort.value)
            self.list[self.webConfigIndex] = ((_("Webif") + "  (%s)" % ipA,))
            self["config"].l.invalidateEntry(self.webConfigIndex)
        else:
            self.list[self.webConfigIndex] = ((_("Webif"),))
            self["config"].l.invalidateEntry(self.webConfigIndex)

    def writeLedCount(self):
        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
            pos_txt = "Above"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
            pos_txt = "Below"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
            pos_txt = "Left"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
            pos_txt = "Right"
        else:
            # hyperion and aml, v4l2
            pos_txt = "Default"
        
        led_top = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedTop" % pos_txt).value)
        led_bottom = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedDown" % pos_txt).value)
        led_right = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedRight" % pos_txt).value)
        led_left = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedLeft" % pos_txt).value)
        led_gap = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedBottomGap" % pos_txt).value)

        ledCornerList = [config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerTopLeft" % pos_txt),
                         config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerTopRight" % pos_txt),
                         config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerBottomLeft" % pos_txt),
                         config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerBottomRight" % pos_txt)]

        led_count = led_top + led_bottom + led_right + led_left - led_gap
        for ledCornerConfig in ledCornerList:
            if ledCornerConfig.value:
                led_count += 1
        self.list[self.ledSetupIndex] = ((_("LED-Setup (Number of LEDs: %s)") % led_count,))
        self["config"].l.invalidateEntry(self.ledSetupIndex)

    def changed(self):
        current = self["config"].getCurrent()
        configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])
        # print "[HyperionControl] - change:", configName

        if configName in ("creatorBoblightServer", "creatorDeviceType", "creatorAboveLedBegin", "creatorBelowLedBegin", "creatorLeftLedBegin", "creatorRightLedBegin", "creatorDefaultLedBegin", "creatorBootseqStart", "creatorV4l2CheckOnOff", "creatorHyperionModeType", "creatorXbmcVideoChecker"):
            # set to default baudrate for devices
            if configName == "creatorDeviceType" and current[1].value == "sedu":
                config.plugins.hyperioncontrol.creatorBaudrate.value = "500000"
            elif configName == "creatorDeviceType" and current[1].value == "karate":
                config.plugins.hyperioncontrol.creatorBaudrate.value = "57600"
            elif configName == "creatorDeviceType" and current[1].value == "adalight":
                config.plugins.hyperioncontrol.creatorBaudrate.value = "115200"
            elif configName == "creatorDeviceType" and current[1].value == "AdalightApa102":
                config.plugins.hyperioncontrol.creatorBaudrate.value = "115200"
            elif configName == "creatorDeviceType" and current[1].value == "atmo":
                config.plugins.hyperioncontrol.creatorBaudrate.value = "38400"
            elif configName == "creatorDeviceType":
                config.plugins.hyperioncontrol.creatorBaudrate.value = "200000"
            elif configName == "creatorHyperionModeType":
                self.readValuesFromConfig()
            self.buildConfig()
            return

        if configName.startswith("creatorDefaultLed") or configName.startswith("creatorAboveLed") or configName.startswith("creatorBelowLed") or configName.startswith("creatorLeftLed") or configName.startswith("creatorRightLed"):
            self.writeLedCount()

        if configName.startswith("creatorWebConfig"):
            self.writeWepifIP()

    def info(self):
        current = self["config"].getCurrent()
        configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])

        message_txt = ""
        if configName.endswith(("LedBegin", "HPicDeep", "VPicDeep", "VPicDist", "HPicDist", "LedDirection")):
            self.session.open(HyperionControlInfoScreen, configName)
            return
        elif configName in ("creatorHyperionModeType"):
            message_txt = _("Selection of the configuration type to be processed.")
        elif configName in ("creatorXbmcVideoChecker", "creatorXbmcIP"):
            message_txt = _("Activates the possibility to control the LEDs via an external Kodi. For Kodi-IP enter the IP address of the external device on which Kodi is running with addon hyperion.")
        elif configName in ("creatorBoblightServer", "creatorBoblightServerPort"):
            message_txt = _("Activate the Boblight Server mode e.g. Enigmalight (external device) to use as a grabber.")
        elif configName in ("creatorSaturationGain", "creatorSaturationLGain"):
            message_txt = _("Saturation describes the relationship between color and gray content. It is expressed in percentages between 0% (gray) and 100% (fully saturated color).")
        elif configName in ("creatorValueGain", "creatorLuminanceGain"):
            message_txt = _("With the brightness you adjust the brightness of the LED's.")
        elif configName == "creatorLuminanceMinimum":
            message_txt = _("With 'Luminance Minimum' you can  regulate the minimum brightness of the LEDs.")
        elif configName.startswith("creatorWhitelevel"):
            message_txt = _(
                "Whitelevel\n\nThe led strips channels dont have always the same perceived amount of color, that's why when we see a white image the leds are a bit reddish, greenish or bluish. To correct that behaviour, we need to limit the output or one or two channels so we perceive a perfect white.\n\nChange the 'Red', 'Green', 'Blue' values until your 'led white' fits your 'TV white'.\n\nHINT: Don't adjust the brightness of your leds with this setting!")
        elif configName.startswith("creatorGamma"):
            message_txt = _(
                "Gamma correction\nEach gamma channel must be set so that the 'gray' of the LEDs does not look greenish/reddish/bluish.\nThis should be checked with each test image (calibration via Ok button in the setup option).")
        elif configName.startswith("creatorThreshold"):
            message_txt = _(
                "With this setting you delete the color information of the smallest values - so they are interpreted as black. If you, for example, set 'red' to a threshold of 10, all red values below are interpreted as black.")
        elif configName.startswith("creatorTemperature"):
            message_txt = _(
                "The 'temperature' changes the perception of all colors, either they are 'warmer' or 'colder'. With certain combinations you can set the Kelvin values for the color temperature as with LED lamps.\n\nExamples:\nDirect sunlight = 6,000K = 255, 255, 255\nCandle light = 1,900K = 255, 147, 41\nHalogen light = 3,200K = 255, 241, 224")
        elif "Adjust" in configName:
            message_txt = _(
                "Color correction\n\nIn this case, each color channel must be adjusted to match the color TV output in order to correct any color casts in the LEDs.\nThis should be checked with the test images (calibration via the Ok button for the respective setup option).")
        elif configName.startswith("creatorBlacklevel"):
            message_txt = _(
                "The black level determines how bright the LEDs are in black. This can be checked via the test pattern (calibration via the OK key in the setup option).")
        elif configName.startswith("creatorBbd"):
            message_txt = _(
                "With this option, the black border detection can be activated and adjusted. Black frames are ignored (as with some movies) and the image areas after that are used for the color definition of the LEDs.\n\nhint: If the LEDs do not show anything on black border when the black border detection is activated, the threshold value may be too low. If the threshold is set too high, even dark areas of the image in movies can be classified as black borders.")
        elif configName.startswith("creatorWebConfig"):
            ipA = self.getCurrentIP()
            ipA = "http://" + ipA + ":%s" % int(config.plugins.hyperioncontrol.creatorWebConfigPort.value)
            message_txt = _(
                "Webif\n\nMany settings from the plugin (such as transform, colors and effects) can also be changed in the hyperion web interface via browser access.\n\nAttention! If you use several web interfaces, you have to choose a different port for each one.\n\nCall in the web browser via:\n\n%s") % ipA
        elif configName.startswith("creatorWebConfigPort"):
            message_txt = _(
                "Attention! If you use several web interfaces, you have to choose a different port for each one.")
        elif configName.startswith("creatorBootseq"):
            message_txt = _(
                "Bootsequence\n\nWith this options you can setup the bootsequence. Youn can set a color or an effect on boot hyperion (with 'off' you can deactivate this function). You can also set the duration of the bootsequence (0 ms = endless).\nOn 'off' hyperion start directly in live mode.")
        elif configName == "creatorDeviceType":
            message_txt = _(
                "Device-Type\n\nHere you can select the used device type.\n\nIf you use a device via USB (Adalight, Adalight-APA102, Sedulight, Karatelight, Atmolight), you need additional kernel modules:\napt-get install kernel-module-ch341\napt-get install kernel-module-usbserial")
        elif configName == "creatorOutputWLED":
            message_txt = _(
                "Set to yes if you use WLED to control your LEDs. Commands are then sent to WLED to ensure that WLED is on and can receive data from Hyperion.")
        elif configName == "creatorConfigName":
            message_txt = _("Can be chosen freely.")
        elif configName == "creatorOutputIP":
            message_txt = _("IP Adress of Wemos ESP or HueBridge.")
        elif configName == "creatorPhe_username":
            message_txt = _(
                "Generating your username. Visit developers.meethue.com and follow the Getting Started Guide step bystep.\nOnce you establish your connection with the bridge, you will have a 32 characters uniqueusername.")
        elif configName == "creatorPhe_clientkey":
            message_txt = _(
                "Open web interface for developers (API debugging) on the bridge: http: // <IP of the bridge> /debug/clip.html.\nCreate a new user here. Devicetype: myapplication hyperionHue, generateclientkey: true \nThe answer is the trust name and the client key.")
        elif configName == "creatorPhe_lightIds":
            message_txt = _(
                "In PhilipsHue App you can see your Lights.\nMake a note of ordering of the Lights next to the lamp icons in Lights section.\nThis is your LightsIDs.")
        elif configName == "creatorPhe_groupId":
            message_txt = _(
                "Identify entertainment area. \n / api / <user name from step 1> / groups / <number of entertainment area>")
        elif configName == "creatorPhe_switchOffOnBlack":
            message_txt = _(
                "Not all colrs can be displayed by the Philips Hue devices and black will result in abluish, dark color.\nTherefore, you can let Hyperion switch off your lights when the light should be black:")
        elif configName == "creatorBaudrate":
            message_txt = _(
                "Baudrate\n\nThe baudrate is preset with a default value for each device type. In rare cases, however, this value can deviate. If no result is visible, the baud rate must be selected differently.\n\nIf no default baudrate is required, this can be freely selected.\nThe baudrate then depends on the set videograbber frequency (FPS) and the number of LEDs used. An LED requires 24 bits + 8 bits for Control. If, for example, we have installed 100 LEDs, you need 32 * 100 bits per second, which is then multiplied by the set frequency.\n\nWith 10 FPS this would be 32 * 100 * 10 = 32,000 bits.\nA baud rate of 32,000 would thus be sufficient to control all LEDs.\n\nFor most stripes with UDPRaw setting, 800.000 is a good value.")
        elif configName == "creatorProtocol":
            message_txt = _(
                "Protocol\n\nProtocol 0 simply sends raw led data in RGBRGB in binary. UDP listener is compatible with Protocol 0\n\nIf you are using an ESP8266/Arduino device with a long LED strip, you chould use this alternate protocol 2.\n\nProtocol 3 is a simple TPM2.net implementation. More information on TPM2.net")
        elif configName == "creatorColorOrder":
            message_txt = _(
                "If the colors are not displayed correctly. Red e.g. Is blue, you can change the color order here.")
        elif configName in ("creatorDefaultLedTop", "creatorAboveLedTop", "creatorBelowLedTop", "creatorLeftLedTop", "creatorRightLedTop"):
            message_txt = _(
                "LEDs top: Add the amount of leds that you placed at the top of your TV, for lamps below, left, right enter 0 here.")
        elif configName in ("creatorDefaultLedDown", "creatorAboveLedDown", "creatorBelowLedDown", "creatorLeftLedDown", "creatorRightLedDown"):
            message_txt = _(
                "LEDs down: Add the amount of leds that you placed at the bottom of your TV, for lamps top, left, right enter 0 here.")
        elif configName in ("creatorDefaultLedLeft", "creatorAboveLedLeft", "creatorBelowLedLeft", "creatorLeftLedLeft", "creatorRightLedLeft"):
            message_txt = _(
                "LEDs left: Add the amount of leds that you placed at the left of your TV, for lamps top, down, right enter 0 here.")
        elif configName in ("creatorDefaultLedRight", "creatorAboveLedRight", "creatorBelowLedRight", "creatorRightLedLeft", "creatorRightLedRight"):
            message_txt = _(
                "LEDs right: Add the amount of leds that you placed at the right of your TV, for lamps top, down, left enter 0 here.")
        elif configName == "creatorOutputTYPE":
            message_txt = _(
                "Output\n\nDepending on the device type used, the appropriate output must be selected.\n\nIf you are not sure, check the description of your device.\n\nIf you don't find any information then you have to find out by trying.")
        elif configName in ("creatorDefaultPicOverlap", "creatorAbovePicOverlap", "creatorBelowPicOverlap", "creatorLeftPicOverlap", "creatorRightPicOverlap"):
           message_txt = _(
                "For a smoother color transition you could overlap the leds area with each other. HINT: The default values are good, you could just add some overlap (20) if you like.")
        elif configName in ("creatorDefaultLedBottomGap", "creatorAboveLedBottomGap", "creatorBelowLedBottomGap", "creatorLeftLedBottomGap", "creatorRightLedBottomGap"):
           message_txt = _(
                "If your TV is not mounted at a wall you may have a gap where no leds are placed, add it here.")
        elif "LedCorner" in configName:
           message_txt = _(
                "If LEDs are installed exactly in the corner, set to yes.")
        elif configName == "creatorSmoothingupdateFrequency":
            message_txt = _(
                "Update Frequency\n\nThe setting is used to smooth the LED data in order to reduce the flickering of the LEDs, the higher the value the higher the smoothing.\n\nValues up to 30 Hz are possible for APA102 and LPD8806\n\nWith other LED stripes, this must be tested yourself\n\nProblems can possibly arise if the value is selected too high.")
        elif configName == "creatorFramegrabberSize":
            message_txt = _(
                "The width and height of the image that is captured. The default value is sufficient, unless you have more than 160 LEDs in one direction, then use the next higher value.\nAttention! The CPU load increases with larger values.")
        elif configName == "creatorFramegrabberFreq":
            message_txt = _(
                "How often a pictire will be grabbed. Large values lead to a better result.\nAttention! The CPU load increases with larger values.")
        elif configName == "creatorV4l2frameDecimation":
            message_txt = _(
                "Frame decimation will clean out all green and blury stuff. This only makes sense for analog grabbers.")
        elif configName == "creatorV4l2sizeDecimation":
            message_txt = _(
                "Size decimation shrinks your grabbed picture to get more fps.\nA good value for the grabber is between 160 and 320 px. The basic configuration is available in /root/.hyperion/hyperion-v4l2-ctl.conf. Set a value for your grabber there and then share this value with this function to get the value.")
        elif "creatorV4l2crop" in configName:
            message_txt = _(
                "Cropping from the left, right, top and bottom side\nFor analog grabbers that is called dirty edges.")
        elif configName in ("creatorV4l2redSignalThreshold", "creatorV4l2greenSignalThreshold", "creatorV4l2blueSignalThreshold"):
            message_txt = _(
                "Signal threshold for the red, green and blue color\nIf you have red, green or blue colors although it should actually be black.")
                                
        if message_txt:
            self.session.open(MessageBox, message_txt, MessageBox.TYPE_INFO, windowTitle="Hyperion Control Info")

    def getHostname(self):
        if os.path.exists(eEnv.resolve("${sysconfdir}/hostname")):
            fp = open(eEnv.resolve("${sysconfdir}/hostname"), 'r')
            hostname = fp.readline().replace("\n", "")
            fp.close()
            return hostname
        else:
            return "unknown hostname"

    def getCurrentIP(self):
        ipA = self.getHostname()
        if ipA != "unknown hostname":
            return ipA
        services = eNetworkManager.getInstance().getServices()
        ipA = _("IP_of_the_box")
        for service in services:
            ip = self.getServiceIP(service)
            name = service.name()
            if ip != None:
                ipA = "%s" % ip
        return ipA

    def getServiceIP(self, service):
        ip = None
        if service.state() == eNetworkManager.STATE_ONLINE:
            ipv4 = service.ipv4()
            ip = ipv4.get("Address", "0.0.0.0")
            if ip == "0.0.0.0":
                ipv6 = self._service.ipv6()
                ip6 = ipv6.get("Address", "::")
                if ip6 != "::":
                    ip = ip6
        return ip

    def keyOK(self):
        current = self["config"].getCurrent()
        configName = self.getConfigName(config.plugins.hyperioncontrol, current[1])

        if configName == "creatorBootseqColor":
            self.session.openWithCallback(boundFunction(self.colorCallback, current[1]), hyperionControlColors, True)
        elif configName == "creatorBootseqEffect":
            self.session.openWithCallback(boundFunction(self.effectsCallback, current[1]), hyperionControlEffects, True)

    def colorCallback(self, configObj=None, retvalue=False):
        if retvalue and configObj:
            configObj.choices.choices = [(retvalue[1], retvalue[0])]
            configObj.value = str(retvalue[1])

    def effectsCallback(self, configObj=None, retvalue=False):
        if retvalue:
            configObj.choices.choices = [(retvalue, retvalue)]
            configObj.value = str(retvalue)

    def key_red(self):
        self.close()

    def key_green(self):
        self.saveJson()

    def key_exit(self):
        self.saveAll()  # save all config-values which don't use NoSave()
        config.plugins.hyperioncontrol.save()
        configfile.save()
        self.close()

    def saveJson(self):
        try:
            # hsv
            self.jsonConfig['color']['transform'][0]['hsv']['saturationGain'] = int(config.plugins.hyperioncontrol.creatorSaturationGain.value) / 100.0
            self.jsonConfig['color']['transform'][0]['hsv']['valueGain'] = int(config.plugins.hyperioncontrol.creatorValueGain.value) / 100.0

            # hsl
            self.jsonConfig['color']['transform'][0]['hsl'] = {}
            self.jsonConfig['color']['transform'][0]['hsl']['saturationLGain'] = int(config.plugins.hyperioncontrol.creatorSaturationLGain.value) / 100.0
            self.jsonConfig['color']['transform'][0]['hsl']['luminanceGain'] = int(config.plugins.hyperioncontrol.creatorLuminanceGain.value) / 100.0
            self.jsonConfig['color']['transform'][0]['hsl']['luminanceMinimum'] = int(config.plugins.hyperioncontrol.creatorLuminanceMinimum.value) / 100.0

            # red
            self.jsonConfig['color']['transform'][0]['red']['gamma'] = int(config.plugins.hyperioncontrol.creatorGammaRed.value) / 100.0
            self.jsonConfig['color']['transform'][0]['red']['threshold'] = int(config.plugins.hyperioncontrol.creatorThresholdRed.value) / 100.0

            # set as new key
            self.jsonConfig['color']['temperature'] = [{}]
            self.jsonConfig['color']['temperature'][0]['id'] = "default"
            self.jsonConfig['color']['temperature'][0]['leds'] = "*"
            self.jsonConfig['color']['temperature'][0]['correctionValues'] = {}

            self.jsonConfig['color']['temperature'][0]['correctionValues']['red'] = int(config.plugins.hyperioncontrol.creatorTemperatureRed.value)

            # set as new key
            self.jsonConfig['color']['channelAdjustment'] = [{}]
            self.jsonConfig['color']['channelAdjustment'][0]['id'] = "default"
            self.jsonConfig['color']['channelAdjustment'][0]['leds'] = "*"
            self.jsonConfig['color']['channelAdjustment'][0]['pureRed'] = {}
            self.jsonConfig['color']['channelAdjustment'][0]['pureGreen'] = {}
            self.jsonConfig['color']['channelAdjustment'][0]['pureBlue'] = {}

            self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['redChannel'] = int(config.plugins.hyperioncontrol.creatorRedAdjustRed.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['redChannel'] = int(config.plugins.hyperioncontrol.creatorGreenAdjustRed.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['redChannel'] = int(config.plugins.hyperioncontrol.creatorBlueAdjustRed.value)
            self.jsonConfig['color']['transform'][0]['red']['whitelevel'] = int(config.plugins.hyperioncontrol.creatorWhitelevelRed.value) / 100.0
            self.jsonConfig['color']['transform'][0]['red']['blacklevel'] = int(config.plugins.hyperioncontrol.creatorBlacklevelRed.value) / 100.0

            # green
            self.jsonConfig['color']['transform'][0]['green']['gamma'] = int(config.plugins.hyperioncontrol.creatorGammaGreen.value) / 100.0
            self.jsonConfig['color']['transform'][0]['green']['threshold'] = int(config.plugins.hyperioncontrol.creatorThresholdGreen.value) / 100.0
            self.jsonConfig['color']['temperature'][0]['correctionValues']['green'] = int(config.plugins.hyperioncontrol.creatorTemperatureGreen.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['greenChannel'] = int(config.plugins.hyperioncontrol.creatorRedAdjustGreen.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['greenChannel'] = int(config.plugins.hyperioncontrol.creatorGreenAdjustGreen.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['greenChannel'] = int(config.plugins.hyperioncontrol.creatorBlueAdjustGreen.value)
            self.jsonConfig['color']['transform'][0]['green']['whitelevel'] = int(config.plugins.hyperioncontrol.creatorWhitelevelGreen.value) / 100.0
            self.jsonConfig['color']['transform'][0]['green']['blacklevel'] = int(config.plugins.hyperioncontrol.creatorBlacklevelGreen.value) / 100.0

            # blue
            self.jsonConfig['color']['transform'][0]['blue']['gamma'] = int(config.plugins.hyperioncontrol.creatorGammaBlue.value) / 100.0
            self.jsonConfig['color']['transform'][0]['blue']['threshold'] = int(config.plugins.hyperioncontrol.creatorThresholdBlue.value) / 100.0
            self.jsonConfig['color']['temperature'][0]['correctionValues']['blue'] = int(config.plugins.hyperioncontrol.creatorTemperatureBlue.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureRed']['blueChannel'] = int(config.plugins.hyperioncontrol.creatorRedAdjustBlue.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureGreen']['blueChannel'] = int(config.plugins.hyperioncontrol.creatorGreenAdjustBlue.value)
            self.jsonConfig['color']['channelAdjustment'][0]['pureBlue']['blueChannel'] = int(config.plugins.hyperioncontrol.creatorBlueAdjustBlue.value)
            self.jsonConfig['color']['transform'][0]['blue']['whitelevel'] = int(config.plugins.hyperioncontrol.creatorWhitelevelBlue.value) / 100.0
            self.jsonConfig['color']['transform'][0]['blue']['blacklevel'] = int(config.plugins.hyperioncontrol.creatorBlacklevelBlue.value) / 100.0

            # V4l2 grabber
            if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-v4l2":
                if self.jsonConfig.get('grabber-v4l2'):
                    self.jsonConfig['grabber-v4l2']['frameDecimation'] = int(config.plugins.hyperioncontrol.creatorV4l2frameDecimation.value)
                    self.jsonConfig['grabber-v4l2']['sizeDecimation'] = int(config.plugins.hyperioncontrol.creatorV4l2sizeDecimation.value)
                    self.jsonConfig['grabber-v4l2']['cropLeft'] = int(config.plugins.hyperioncontrol.creatorV4l2cropLeft.value)
                    self.jsonConfig['grabber-v4l2']['cropRight'] = int(config.plugins.hyperioncontrol.creatorV4l2cropRight.value)
                    self.jsonConfig['grabber-v4l2']['cropTop'] = int(config.plugins.hyperioncontrol.creatorV4l2cropTop.value)
                    self.jsonConfig['grabber-v4l2']['cropBottom'] = int(config.plugins.hyperioncontrol.creatorV4l2cropBottom.value)
                    self.jsonConfig['grabber-v4l2']['redSignalThreshold'] = int(config.plugins.hyperioncontrol.creatorV4l2redSignalThreshold.value) / 100.0
                    self.jsonConfig['grabber-v4l2']['greenSignalThreshold'] = int(config.plugins.hyperioncontrol.creatorV4l2greenSignalThreshold.value) / 100.0
                    self.jsonConfig['grabber-v4l2']['blueSignalThreshold'] = int(config.plugins.hyperioncontrol.creatorV4l2blueSignalThreshold.value) / 100.0
                else:
                    self.updateJsonV4l2()
            else:
                if self.jsonConfig.get('grabber-v4l2'):
                    del self.jsonConfig['grabber-v4l2']
            # Framegrabber
            if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-aml":
                if self.jsonConfig.get('framegrabber'):
                    self.jsonConfig['framegrabber']['width'] = int(str(config.plugins.hyperioncontrol.creatorFramegrabberSize.value).split(",")[0])
                    self.jsonConfig['framegrabber']['height'] = int(str(config.plugins.hyperioncontrol.creatorFramegrabberSize.value).split(",")[1])
                    self.jsonConfig['framegrabber']['frequency_Hz'] = int(config.plugins.hyperioncontrol.creatorFramegrabberFreq.value) * 1.0
                else:
                    self.updateJsonAml()
            else:
                if self.jsonConfig.get('framegrabber'):
                    del self.jsonConfig['framegrabber']

            # Smoothing
            self.jsonConfig['color']['smoothing']['updateFrequency'] = int(config.plugins.hyperioncontrol.creatorSmoothingupdateFrequency.value) * 1.0

            # BlackborderDetector
            self.jsonConfig['blackborderdetector']['enable'] = config.plugins.hyperioncontrol.creatorBbdEnable.value
            self.jsonConfig['blackborderdetector']['threshold'] = int(config.plugins.hyperioncontrol.creatorBbdThreshold.value) / 100.0

            # webif
            if self.jsonConfig.get('webConfig'):
                self.jsonConfig['webConfig']['enable'] = config.plugins.hyperioncontrol.creatorWebConfigEnable.value
                self.jsonConfig['webConfig']['port'] = int(config.plugins.hyperioncontrol.creatorWebConfigPort.value)
            else:
                self.updateJsonWebif()

            # bootsequence
            if config.plugins.hyperioncontrol.creatorBootseqStart.value == "off":
                self.jsonConfig['bootsequence']['effect'] = ""
                self.jsonConfig['bootsequence']['color'] = [0, 0, 0]
            elif config.plugins.hyperioncontrol.creatorBootseqStart.value == "effect":
                self.jsonConfig['bootsequence']['effect'] = config.plugins.hyperioncontrol.creatorBootseqEffect.value
                self.jsonConfig['bootsequence']['color'] = [0, 0, 0]
            else:
                self.jsonConfig['bootsequence']['effect'] = str("")
                color = config.plugins.hyperioncontrol.creatorBootseqColor.value
                self.jsonConfig['bootsequence']['color'] = [int(color[1:3], 16), int(color[3:5], 16),
                                                            int(color[5:7], 16)]
            self.jsonConfig['bootsequence']['duration_ms'] = int(
                config.plugins.hyperioncontrol.creatorBootseqDuration.value)

            # device
            self.jsonConfig['device']['type'] = str(config.plugins.hyperioncontrol.creatorDeviceType.value)
            self.jsonConfig['device']['name'] = str(config.plugins.hyperioncontrol.creatorConfigName.value)
            if config.plugins.hyperioncontrol.creatorDeviceType.value in ("sedu", "adalight", "AdalightApa102", "atmo", "karate"):
                self.jsonConfig['device']['output'] = str(config.plugins.hyperioncontrol.creatorOutputTYPE.value)
                self.jsonConfig['device']['delayafterconnect'] = int(config.plugins.hyperioncontrol.creatorDelayafterconnect.value)
            else:
                if config.plugins.hyperioncontrol.creatorDeviceType.value in ("philipshue", "philipshueentertainment"):
                    self.jsonConfig['device']['output'] = ".".join(["%d" % d for d in config.plugins.hyperioncontrol.creatorOutputIP.value])
                elif config.plugins.hyperioncontrol.creatorDeviceType.value == "udpraw":
                    self.jsonConfig['device']['output'] = ".".join(["%d" % d for d in config.plugins.hyperioncontrol.creatorOutputIP.value]) + ":19446"
                    self.jsonConfig['device']['protocol'] = int(config.plugins.hyperioncontrol.creatorProtocol.value)
            if config.plugins.hyperioncontrol.deviceType.value == "philipshue":
                self.jsonConfig['device']['username'] = str(config.plugins.hyperioncontrol.creatorPhe_username.value)
                self.jsonConfig['device']['lightIds'] = [int(s) for s in config.plugins.hyperioncontrol.creatorPhe_lightIds.value.split(',')]
                self.jsonConfig['device']['switchOffOnBlack'] = str(config.plugins.hyperioncontrol.creatorPhe_switchOffOnBlack.value)
            if config.plugins.hyperioncontrol.deviceType.value == "philipshueentertainment":
                self.jsonConfig['device']['username'] = str(config.plugins.hyperioncontrol.creatorPhe_username.value)
                self.jsonConfig['device']['clientkey'] = str(config.plugins.hyperioncontrol.creatorPhe_clientkey.value)
                self.jsonConfig['device']['lightIds'] = [int(s) for s in config.plugins.hyperioncontrol.creatorPhe_lightIds.value.split(',')]
                self.jsonConfig['device']['groupId'] = int(config.plugins.hyperioncontrol.creatorPhe_groupId.value)
                self.jsonConfig['device']['switchOffOnBlack'] = str(config.plugins.hyperioncontrol.creatorPhe_switchOffOnBlack.value)
            if config.plugins.hyperioncontrol.deviceType.value == "philipshueentertainment":
                self.jsonConfig['device']['colorOrder'] = str(config.plugins.hyperioncontrol.colorOrder.value)
            else:
                self.jsonConfig['device']['colorOrder'] = str(config.plugins.hyperioncontrol.colorOrder.value)
                self.jsonConfig['device']['rate'] = int(config.plugins.hyperioncontrol.creatorBaudrate.value)

            # LED-Config
            if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
                led_begin = config.plugins.hyperioncontrol.creatorAboveLedBegin.value
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
                led_begin = config.plugins.hyperioncontrol.creatorBelowLedBegin.value
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
                led_begin = config.plugins.hyperioncontrol.creatorLeftLedBegin.value
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
                led_begin = config.plugins.hyperioncontrol.creatorRightLedBegin.value
            else:
                # hyperion and aml, hyperion-v4l2
                led_begin = config.plugins.hyperioncontrol.creatorDefaultLedBegin.value

            if led_begin != "0":
                LEDconfig = self.createNewLEDConfig()
                if LEDconfig:
                    self.jsonConfig['leds'] = LEDconfig
                else:
                    self.session.open(MessageBox, _("Error when generating the LED config!"), MessageBox.TYPE_INFO,
                                      timeout=5)

            restart = False
            if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
                config_file_path = "/etc/hyperion/hyperion-above.config.json"
                self.writeHyperionAboveServce()
                self.delJsonItems(['xbmcVideoChecker', 'forwarder', 'boblightServer'])
                self.updateJsonServer(19451, 19452)
                restart = readLampAbove_Device()
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
                config_file_path = "/etc/hyperion/hyperion-below.config.json"
                self.writeHyperionBelowService()
                self.delJsonItems(['xbmcVideoChecker', 'forwarder', 'boblightServer'])
                self.updateJsonServer(19453, 19454)
                restart = readLampBelow_Device()
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
                config_file_path = "/etc/hyperion/hyperion-left.config.json"
                self.writeHyperionLeftService()
                self.delJsonItems(['xbmcVideoChecker', 'forwarder', 'boblightServer'])
                self.updateJsonServer(19447, 19448)
                restart = readLampLeft_Device()
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
                config_file_path = "/etc/hyperion/hyperion-right.config.json"
                self.writeHyperionRightService()
                self.delJsonItems(['xbmcVideoChecker', 'forwarder', 'boblightServer'])
                self.updateJsonServer(19449, 19450)
                restart = readLampRight_Device()
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-v4l2":
                config_file_path = "/etc/hyperion/hyperion.config-v4l2.json"
                self.writeV4l2Service()
                self.delJsonItems(['xbmcVideoChecker', 'boblightServer'])
                self.updateJsonForwarder()
                self.updateJsonServer(19444, 19445)
                value = getHyperionInfo(mode="hyperion-v4l2")
                restart = value[0]
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-aml":
                config_file_path = "/etc/hyperion/hyperion.config.json"
                self.writeAmlService()
                self.delJsonItems(['xbmcVideoChecker', 'boblightServer'])
                self.updateJsonForwarder()
                self.updateJsonServer(19444, 19445)
                value = getHyperionInfo(mode="hyperion-aml")
                restart = value[0]
            else:
                # Hyperion
                config_file_path = "/etc/hyperion/hyperion.config.json"
                self.writeHyperionService()
                self.updateJsonXbmcVideoChecker()
                self.updateJsonBoblightServer()
                self.updateJsonForwarder()
                self.updateJsonServer(19444, 19445)
                value = getHyperionInfo(mode="hyperion")
                restart = value[0]

            os.system("systemctl daemon-reload")
            if restart:
                self.restartHyperionService(message=False)

            with open(config_file_path, "w") as write_file:
                json.dump(self.jsonConfig, write_file, ensure_ascii=False, indent=4)
                if restart:
                    txt = _("The settings were saved in '%s'.\n\nservice were restarted." % config_file_path)
                else:
                    txt = _("The settings were saved in '%s'." % config_file_path)
                self.session.openWithCallback(self.saveJsonMessageCallback, MessageBox, txt, MessageBox.TYPE_INFO, timeout=10)

            self.saveAll()  # save all config-values which don't use NoSave()
            config.plugins.hyperioncontrol.save()
            configfile.save()
        except:
            import traceback, sys
            traceback.print_exc()
            exc_type, exc_value, exc_traceback = sys.exc_info()
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
            self.session.open(MessageBox, _("When saving the 'hyperion.config.json' an error has occurred!\n\nError:\n%s") % error, MessageBox.TYPE_INFO, timeout=5)
            self.readValuesFromConfig()

    def restartHyperionService(self, message=True):
        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
            cmd = "systemctl restart hyperion-above"
            txt = _("Hyperion above were restarted.")
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
            cmd = "systemctl restart hyperion-below"
            txt = _("Hyperion below were restarted.")
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
            cmd = "systemctl restart hyperion-left"
            txt = _("Hyperion left were restarted.")
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
            cmd = "systemctl restart hyperion-right"
            txt = _("Hyperion right were restarted.")
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-v4l2":
            cmd = "systemctl restart hyperion-v4l2"
            txt = _("External grabber were restarted.")
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-aml":
            cmd = "systemctl restart hyperion && systemctl restart hyperionaml"
            txt = _("Hyperion and internal grabber were restarted.")
        else:
            cmd = "systemctl restart hyperion"
            txt = _("Hyperion were restarted.")
        os.system(cmd)
        if message:
            self.session.openWithCallback(self.saveJsonMessageCallback, MessageBox, txt, MessageBox.TYPE_INFO, timeout=10)

    def delJsonItems(self, items):
        for item in items:
            if self.jsonConfig.get(item):
                del self.jsonConfig[item]

    def addJsonItem(self, item):
        del self.jsonConfig["endOfJson"]
        self.jsonConfig.update(item)
        self.jsonConfig.update({"endOfJson": "endOfJson"})

    def updateJsonServer(self, port_json, port_proto):
        if self.jsonConfig.get("jsonServer"):
            self.jsonConfig["jsonServer"]["port"] = port_json
        else:
            json_server = {"jsonServer": {
                "port": port_json
            }}
            self.addJsonItem(json_server)

        if self.jsonConfig.get("protoServer"):
            self.jsonConfig["protoServer"]["port"] = port_proto
        else:
            proto_server = {"protoServer": {
                "port": port_proto
            }}
            self.addJsonItem(proto_server)

    def updateJsonForwarder(self):
        forwarder = {"forwarder": {
            "json": [
                "127.0.0.1:19447",
                "127.0.0.1:19449",
                "127.0.0.1:19451",
                "127.0.0.1:19453"
            ],
            "proto": [
                "127.0.0.1:19448",
                "127.0.0.1:19450",
                "127.0.0.1:19452",
                "127.0.0.1:19454"
            ]
        }}
        if self.jsonConfig.get("forwarder"):
            del self.jsonConfig["forwarder"]

        self.addJsonItem(forwarder)

    def updateJsonWebif(self):
        if not self.jsonConfig.get('webConfig'):
            webif = {"webConfig": {
                "enable": True,
                "document_root": "/usr/share/hyperion/webconfig",
                "port": config.plugins.hyperioncontrol.creatorWebConfigPort.value
            }}
            self.addJsonItem(webif)

    def updateJsonBoblightServer(self):
        if not config.plugins.hyperioncontrol.creatorBoblightServer.value and self.jsonConfig.get("boblightServer"):
            del self.jsonConfig['boblightServer']
        else:
            if self.jsonConfig.get('boblightServer'):
                self.jsonConfig['boblightServer']['port'] = config.plugins.hyperioncontrol.creatorBoblightServerPort.value
            else:
                boblight = {"boblightServer": {
                    "port": config.plugins.hyperioncontrol.creatorBoblightServerPort.value,
                    "priority": 900}}
                self.addJsonItem(boblight)

    def updateJsonAml(self):
        internal = {"framegrabber": {
            "width": int(str(config.plugins.hyperioncontrol.creatorFramegrabberSize.value).split(",")[0]),
            "height": int(str(config.plugins.hyperioncontrol.creatorFramegrabberSize.value).split(",")[1]),
            "frequency_Hz": int(config.plugins.hyperioncontrol.creatorFramegrabberFreq.value) * 1.0,
            "priority": 890}}
        self.addJsonItem(internal)

    def updateJsonV4l2(self):
        v4l2 = {'grabber-v4l2': {
            'frameDecimation': int(config.plugins.hyperioncontrol.creatorV4l2frameDecimation.value),
            'sizeDecimation': int(config.plugins.hyperioncontrol.creatorV4l2sizeDecimation.value),
            'cropLeft': int(config.plugins.hyperioncontrol.creatorV4l2cropLeft.value),
            'cropRight': int(config.plugins.hyperioncontrol.creatorV4l2cropRight.value),
            'cropTop': int(config.plugins.hyperioncontrol.creatorV4l2cropTop.value),
            'cropBottom': int(config.plugins.hyperioncontrol.creatorV4l2cropBottom.value),
            'redSignalThreshold': int(config.plugins.hyperioncontrol.creatorV4l2redSignalThreshold.value) / 100.0,
            'greenSignalThreshold': int(config.plugins.hyperioncontrol.creatorV4l2greenSignalThreshold.value) / 100.0,
            'blueSignalThreshold': int(config.plugins.hyperioncontrol.creatorV4l2blueSignalThreshold.value) / 100.0}}
        self.addJsonItem(v4l2)

    def updateJsonXbmcVideoChecker(self):
        if not config.plugins.hyperioncontrol.creatorXbmcVideoChecker.value and self.jsonConfig.get('xbmcVideoChecker'):
            del self.jsonConfig['xbmcVideoChecker']
        else:
            if self.jsonConfig.get('xbmcVideoChecker'):
                self.jsonConfig['xbmcVideoChecker']['xbmcAddress'] = str(".".join(["%d" % d for d in config.plugins.hyperioncontrol.creatorXbmcIP.value]))
            else:
                # add xbmc to json
                xbmc_ip = ".".join(["%d" % d for d in config.plugins.hyperioncontrol.creatorXbmcIP.value])
                xbmc_json = {"xbmcVideoChecker": {
                    "xbmcAddress": "%s" % xbmc_ip,
                    "xbmcTcpPort": 9090,
                    "grabVideo": True,
                    "grabPictures": True,
                    "grabAudio": True,
                    "grabMenu": True,
                    "grabPause": True,
                    "grabScreensaver": True,
                    "enable3DDetection": True}
                }
                self.addJsonItem(xbmc_json)

    def saveJsonMessageCallback(self, ret=None):
        # check installed packages for device-type
        if config.plugins.hyperioncontrol.creatorDeviceType.value in ("sedu", "adalight", "AdalightApa102", "atmo", "karate"):
            command = Command("apt list --installed kernel-module-usbserial kernel-module-ch341 kernel-module-ftdi-sio")
            command.run(timeout=5)

            # read real settings from hyperion-remote -l
            if command.timeout or command.process.returncode != 0:
                print "[HyperionControl] timeout/error on check packages"
            else:
                cmdValue = command.out.strip().split("\n")
                if len(cmdValue) < 4:
                    self.session.openWithCallback(self.installPackages, MessageBox, _("You need more kernel-moduls for the selected device-type:\n  - kernel-module-usbserial\n  - kernel-module-ch341\n  - kernel-module-ftdi-sio\n\nDo you want to install this needed modules?"), MessageBox.TYPE_YESNO)

    def installPackages(self, ret=None):
        if ret:
            self.session.openWithCallback(self.installPackagesCallback, Console, _("HyperionControl - install needed modules"), ["apt-get -y --assume-yes install kernel-module-usbserial kernel-module-ch341 kernel-module-ftdi-sio", "modprobe usbserial", "modprobe ch341", "modprobe ftdi-sio", "systemctl restart hyperion && systemctl restart hyperionaml"])

    def installPackagesCallback(self, ret=None):
        self.session.open(MessageBox, _("Needed modules was installed and loaded.\n\nHyperion and internal grabber were restarted."), MessageBox.TYPE_INFO)

    def writeAmlService(self):
        # /lib/systemd/system/hyperionaml.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperionaml Systemd service\n\n"

        script_txt += "Conflicts=hyperion-v4l2.service\n"
        script_txt += "Wants=enigma2.service\n"
        script_txt += "Wants=hyperion.service\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=simple\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"

        width = int(str(config.plugins.hyperioncontrol.creatorFramegrabberSize.value).split(",")[0])
        height = int(str(config.plugins.hyperioncontrol.creatorFramegrabberSize.value).split(",")[1])
        freq = config.plugins.hyperioncontrol.creatorFramegrabberFreq.value
        script_txt += "ExecStart=/usr/bin/hyperion-aml -f %s --width %s --height %s\n" % (freq, width, height)
        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        amlservice_file = open("/lib/systemd/system/hyperionaml.service", "w+")
        amlservice_file.write(script_txt)
        amlservice_file.close()
        os.chmod("/lib/systemd/system/hyperionaml.service", 0644)

    def writeHyperionService(self):
        # /lib/systemd/system/hyperion.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperion Systemd service\n\n"

        script_txt += "Conflicts=hyperion-v4l2.service\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=simple\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"
        script_txt += "ExecStart=/usr/bin/hyperiond /etc/hyperion/hyperion.config.json\n"
        if config.plugins.hyperioncontrol.creatorOutputWLED.value:
            script_txt += 'ExecStartPost=/usr/bin/curl -X SET "http://%s/win&T=1&RD=1&SN=0&RN=0"\n' % '%d.%d.%d.%d' % tuple(config.plugins.hyperioncontrol.creatorOutputIP.value)

        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        v4l2service_file = open("/lib/systemd/system/hyperion.service", "w+")
        v4l2service_file.write(script_txt)
        v4l2service_file.close()
        os.chmod("/lib/systemd/system/hyperion.service", 0644)

    def writeV4l2Service(self):
        # /lib/systemd/system/hyperion-v4l2.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperion-v4l2 Systemd service\n\n"

        script_txt += "Conflicts=hyperionaml.service\n"
        script_txt += "Conflicts=hyperion.service\n\n"
        script_txt += "After=network-online.target\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=simple\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"
#        script_txt += "ExecStartPre=/usr/bin/v4l2-ctl -d /dev/video0 --set-fmt-video=width=320,height=240 -p 30\n"
        script_txt += "ExecStartPre=/root/.hyperion/hyperion-v4l2-ctl.conf\n"
        script_txt += "ExecStart=/usr/bin/hyperiond /etc/hyperion/hyperion.config-v4l2.json\n"
        if config.plugins.hyperioncontrol.creatorOutputWLED.value:
            script_txt += 'ExecStartPost=/usr/bin/curl -X SET "http://%s/win&T=1&RD=1&SN=0&RN=0"\n' % '%d.%d.%d.%d' % tuple(config.plugins.hyperioncontrol.creatorOutputIP.value)

        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        amlservice_file = open("/lib/systemd/system/hyperion-v4l2.service", "w+")
        amlservice_file.write(script_txt)
        amlservice_file.close()
        os.chmod("/lib/systemd/system/hyperion-v4l2.service", 0644)

    def writeHyperionAboveServce(self):
        # /lib/systemd/system/hyperion-above.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperion-above Systemd service\n"
        script_txt += "After=network-online.target\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=idle\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"
        script_txt += "ExecStart=/usr/bin/hyperiond /etc/hyperion/hyperion-above.config.json\n"
        if config.plugins.hyperioncontrol.creatorOutputWLED.value:
            script_txt += 'ExecStartPost=/usr/bin/curl -X SET "http://%s/win&T=1&RD=1&SN=0&RN=0"\n' % '%d.%d.%d.%d' % tuple(config.plugins.hyperioncontrol.creatorOutputIP.value)

        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        amlservice_file = open("/lib/systemd/system/hyperion-above.service", "w+")
        amlservice_file.write(script_txt)
        amlservice_file.close()
        os.chmod("/lib/systemd/system/hyperion-above.service", 0644)

    def writeHyperionBelowService(self):
        # /lib/systemd/system/hyperion-below.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperion-below Systemd servic\n"
        script_txt += "After=network-online.target\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=idle\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"
        script_txt += "ExecStart=/usr/bin/hyperiond /etc/hyperion/hyperion-below.config.json\n"
        if config.plugins.hyperioncontrol.creatorOutputWLED.value:
            script_txt += 'ExecStartPost=/usr/bin/curl -X SET "http://%s/win&T=1&RD=1&SN=0&RN=0"\n' % '%d.%d.%d.%d' % tuple(config.plugins.hyperioncontrol.creatorOutputIP.value)

        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        amlservice_file = open("/lib/systemd/system/hyperion-below.service", "w+")
        amlservice_file.write(script_txt)
        amlservice_file.close()
        os.chmod("/lib/systemd/system/hyperion-below.service", 0644)

    def writeHyperionLeftService(self):
        # /lib/systemd/system/hyperion-left.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperion-left Systemd servic\n"
        script_txt += "After=network-online.target\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=idle\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"
        script_txt += "ExecStart=/usr/bin/hyperiond /etc/hyperion/hyperion-left.config.json\n"
        if config.plugins.hyperioncontrol.creatorOutputWLED.value:
            script_txt += 'ExecStartPost=/usr/bin/curl -X SET "http://%s/win&T=1&RD=1&SN=0&RN=0"\n' % '%d.%d.%d.%d' % tuple(config.plugins.hyperioncontrol.creatorOutputIP.value)

        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        amlservice_file = open("/lib/systemd/system/hyperion-left.service", "w+")
        amlservice_file.write(script_txt)
        amlservice_file.close()
        os.chmod("/lib/systemd/system/hyperion-left.service", 0644)

    def writeHyperionRightService(self):
        # /lib/systemd/system/hyperion-right.service
        script_txt = "[Unit]\n"
        script_txt += "Description=Hyperion-right Systemd servic\n"
        script_txt += "After=network-online.target\n\n"

        script_txt += "[Service]\n"
        script_txt += "Type=idle\n"
        script_txt += "User=root\n"
        script_txt += "Group=root\n"
        script_txt += "UMask=007\n"
        script_txt += "ExecStart=/usr/bin/hyperiond /etc/hyperion/hyperion-right.config.json\n"
        if config.plugins.hyperioncontrol.creatorOutputWLED.value:
            script_txt += 'ExecStartPost=/usr/bin/curl -X SET "http://%s/win&T=1&RD=1&SN=0&RN=0"\n' % '%d.%d.%d.%d' % tuple(config.plugins.hyperioncontrol.creatorOutputIP.value)

        script_txt += "ExecReload=/bin/kill -HUP $MAINPID\n"
        script_txt += "Restart=on-failure\n"
        script_txt += "TimeoutStopSec=10\n\n"
        script_txt += "[Install]\n"
        script_txt += "WantedBy=multi-user.target\n"

        amlservice_file = open("/lib/systemd/system/hyperion-right.service", "w+")
        amlservice_file.write(script_txt)
        amlservice_file.close()
        os.chmod("/lib/systemd/system/hyperion-right.service", 0644)

    def getNewFixScanValue(self, ledNr, changeNumber, changes, currentPosition):
        # print "== scanValue", ledNr, changes
        if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
            pos_txt = "Above"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
            pos_txt = "Below"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
            pos_txt = "Left"
        elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
            pos_txt = "Right"
        else:
            # hyperion and aml, v4l2
            pos_txt = "Default"
        
        hPicDeep = int(config.plugins.hyperioncontrol.__getattr__("creator%sHPicDeep" % pos_txt).value) / 100.0
        vPicDeep = int(config.plugins.hyperioncontrol.__getattr__("creator%sVPicDeep" % pos_txt).value) / 100.0
        led_direction = config.plugins.hyperioncontrol.__getattr__("creator%sLedDirection" % pos_txt).value

        ledCornerTopLeft = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedTop" % pos_txt).value else 0
        ledCornerTopRight = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedRight" % pos_txt).value else 0
        ledCornerBottomLeft = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedLeft" % pos_txt).value else 0
        ledCornerBottomRight = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedBottomGap" % pos_txt).value else 0


        hPicDist = self.hPicDist
        vPicDist = self.vPicDist

        # reset scan_min-Values after change position
        if changeNumber > 0 and ledNr == changes[changeNumber - 1] + 1:
            # print "== new position '%s' on ledNr '%s', index '%s'" % (currentPosition, ledNr, ledNr-1)
            if currentPosition == "bottom":
                if led_direction == "1":  # UZ
                    self.hscan_min = 1.0000 - vPicDist
                elif led_direction == "2":  # GUZ
                    self.hscan_min = 0.0000 - self.hb_diff + vPicDist

            elif currentPosition == "top":
                if led_direction == "1":  # UZ
                    self.hscan_min = 0.0000 - self.ht_diff + vPicDist
                elif led_direction == "2":  # GUZ
                    self.hscan_min = 1.0000 - vPicDist

            elif currentPosition == "right":
                if led_direction == "1":  # UZ
                    self.vscan_min = 0.0000 - self.vr_diff + hPicDist
                elif led_direction == "2":  # GUZ
                    self.vscan_min = 1.0000 - hPicDist

            elif currentPosition == "left":
                if led_direction == "1":  # UZ
                    self.vscan_min = 1.0000 - hPicDist
                elif led_direction == "2":  # GUZ
                    self.vscan_min = 0.0000 - self.vl_diff + hPicDist

        # set scan-Values on Corner LED
        if changeNumber < 4 and ledNr == changes[changeNumber]:
            if currentPosition == "bottom":
                if led_direction == "1" and ledCornerBottomLeft:  # UZ
                    # print "== corner led BottomLeft", ledNr, ledNr-1
                    return (0.0000 + vPicDist, vPicDeep + vPicDist, 1.0000 - hPicDeep - hPicDist,
                            1.0000 - hPicDist)  # fix value for corner led BottomLeft
                elif led_direction == "2" and ledCornerBottomRight:  # GUZ
                    # print "== corner led BottomRight", ledNr, ledNr-1
                    return (1.0000 - vPicDeep - vPicDist, 1.0000 - vPicDist, 1.0000 - hPicDeep - hPicDist,
                            1.0000 - hPicDist)  # fix value for corner led BottomRight
            elif currentPosition == "top":
                if led_direction == "1" and ledCornerTopRight:  # UZ
                    # print "== corner led TopRight", ledNr, ledNr-1
                    return (1.0000 - vPicDeep - vPicDist, 1.0000 - vPicDist, 0.0000 + hPicDist,
                            hPicDeep + hPicDist)  # fix value for corner led TopRight
                elif led_direction == "2" and ledCornerTopLeft:  # GUZ
                    # print "== corner led TopLeft", ledNr, ledNr-1
                    return (0.0000 + vPicDist, vPicDeep + vPicDist, 0.0000 + hPicDist,
                            hPicDeep + hPicDist)  # fix value for corner LED TopLeft
            elif currentPosition == "right":
                if led_direction == "1" and ledCornerBottomRight:  # UZ
                    # print "== corner led BottomRight", ledNr, ledNr-1
                    return (1.0000 - vPicDeep - vPicDist, 1.0000 - vPicDist, 1.0000 - hPicDeep - hPicDist,
                            1.0000 - hPicDist)  # fix value for corner led BottomRight
                elif led_direction == "2" and ledCornerTopRight:  # GUZ
                    # print "== corner led TopRight", ledNr, ledNr-1
                    return (1.0000 - vPicDeep - vPicDist, 1.0000 - vPicDist, 0.0000 + hPicDist,
                            hPicDeep + hPicDist)  # fix value for corner led TopRight
            elif currentPosition == "left":
                # UZ
                if led_direction == "1" and ledCornerTopLeft:
                    # print "== corner led TopLeft", ledNr, ledNr-1
                    return (0.0000 + vPicDist, vPicDeep + vPicDist, 0.0000 + hPicDist,
                            hPicDeep + hPicDist)  # fix value for corner LED TopLeft
                # GUZ
                elif led_direction == "2" and ledCornerBottomLeft:
                    # print "== corner led BottomLeft", ledNr, ledNr-1
                    return (0.0000 + vPicDist, vPicDeep + vPicDist, 1.0000 - hPicDeep - hPicDist,
                            1.0000 - hPicDist)  # fix value for corner led BottomLeft

        # jump hscan_min on LED-gap
        if self.gap > 0 and ledNr == self.gapLED:
            if led_direction == "1":  # UZ
                self.hscan_min = self.hscan_min - (self.hb_diff * self.gap)
            elif led_direction == "2":  # GUZ
                self.hscan_min = self.hscan_min + (self.hb_diff * self.gap)

        # Overlap in %
        overlap_ht_diff = self.ht_diff / 100 / 2 * int(config.plugins.hyperioncontrol.__getattr__("creator%sPicOverlap" % pos_txt).value)
        overlap_hb_diff = self.hb_diff / 100 / 2 * int(config.plugins.hyperioncontrol.__getattr__("creator%sPicOverlap" % pos_txt).value)
        overlap_vr_diff = self.vr_diff / 100 / 2 * int(config.plugins.hyperioncontrol.__getattr__("creator%sPicOverlap" % pos_txt).value)
        overlap_vl_diff = self.vl_diff / 100 / 2 * int(config.plugins.hyperioncontrol.__getattr__("creator%sPicOverlap" % pos_txt).value)

        # set scanValues
        if currentPosition == "bottom":
            if led_direction == "1":  # UZ
                self.hscan_min = self.hscan_min - self.hb_diff
            elif led_direction == "2":  # GUZ
                self.hscan_min = self.hscan_min + self.hb_diff
            self.hscan_max = self.hscan_min + self.hb_diff
            hscan_min = self.hscan_min - overlap_hb_diff
            hscan_max = self.hscan_max + overlap_hb_diff
            return (hscan_min, hscan_max, 1.000 - hPicDeep - hPicDist, 1.0000 - hPicDist)  # vscan bottom fix

        elif currentPosition == "top":
            if led_direction == "1":  # UZ
                self.hscan_min = self.hscan_min + self.ht_diff
            elif led_direction == "2":  # GUZ
                self.hscan_min = self.hscan_min - self.ht_diff
            self.hscan_max = self.hscan_min + self.ht_diff
            hscan_min = self.hscan_min - overlap_ht_diff
            hscan_max = self.hscan_max + overlap_ht_diff
            return (hscan_min, hscan_max, 0.0000 + hPicDist, hPicDeep + hPicDist)  # vscan top fix

        elif currentPosition == "left":
            if led_direction == "1":  # UZ
                self.vscan_min = self.vscan_min - self.vl_diff
            elif led_direction == "2":  # GUZ
                self.vscan_min = self.vscan_min + self.vl_diff
            self.vscan_max = self.vscan_min + self.vl_diff
            vscan_min = self.vscan_min - overlap_vl_diff
            vscan_max = self.vscan_max + overlap_vl_diff
            return (0.0000 + vPicDist, vPicDeep + vPicDist, vscan_min, vscan_max)  # hscan left fix

        elif currentPosition == "right":
            if led_direction == "1":  # UZ
                self.vscan_min = self.vscan_min + self.vr_diff
            elif led_direction == "2":  # GUZ
                self.vscan_min = self.vscan_min - self.vr_diff
            self.vscan_max = self.vscan_min + self.vr_diff
            vscan_min = self.vscan_min - overlap_vr_diff
            vscan_max = self.vscan_max + overlap_vr_diff
            return (1.0000 - vPicDeep - vPicDist, 1.0000 - vPicDist, vscan_min, vscan_max)  # hscan right fix

    def createNewLEDConfig(self):
        try:
            if config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-above":
                pos_txt = "Above"
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-below":
                pos_txt = "Below"
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-left":
                pos_txt = "Left"
            elif config.plugins.hyperioncontrol.creatorHyperionModeType.value == "hyperion-right":
                pos_txt = "Right"
            else:
                # hyperion and aml, v4l2
                pos_txt = "Default"
        
            led_top = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedTop" % pos_txt).value)
            led_bottom = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedDown" % pos_txt).value)
            led_right = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedRight" % pos_txt).value)
            led_left = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedLeft" % pos_txt).value)
            self.gap = int(config.plugins.hyperioncontrol.__getattr__("creator%sLedBottomGap" % pos_txt).value)

            ledCornerTopLeft = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerTopLeft" % pos_txt).value else 0
            ledCornerTopRight = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerTopRight" % pos_txt).value else 0
            ledCornerBottomLeft = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerBottomLeft" % pos_txt).value else 0
            ledCornerBottomRight = 1 if config.plugins.hyperioncontrol.__getattr__("creator%sLedCornerBottomRight" % pos_txt).value else 0
            
            led_begin = config.plugins.hyperioncontrol.__getattr__("creator%sLedBegin" % pos_txt).value
            led_direction = config.plugins.hyperioncontrol.__getattr__("creator%sLedDirection" % pos_txt).value

            # image area - distance
            self.hPicDist = int(config.plugins.hyperioncontrol.__getattr__("creator%sHPicDist" % pos_txt).value) / 100.0
            self.vPicDist = int(config.plugins.hyperioncontrol.__getattr__("creator%sVPicDist" % pos_txt).value) / 100.0

            led_count = led_top + led_bottom + led_right + led_left - self.gap + ledCornerTopLeft + ledCornerTopRight + ledCornerBottomLeft + ledCornerBottomRight
            # print "=== led-count", led_count

            self.ht_diff = (1.0 - 2 * self.vPicDist) / led_top if led_top else 0
            self.hb_diff = (1.0 - 2 * self.vPicDist) / led_bottom if led_bottom else 0
            self.vr_diff = (1.0 - 2 * self.hPicDist) / led_right if led_right else 0
            self.vl_diff = (1.0 - 2 * self.hPicDist) / led_left if led_left else 0

            self.hscan_min = 0
            self.hscan_max = 0
            self.vscan_min = 0
            self.vscan_max = 0

            self.gapLED = 0

            # ledBegin "unten Mitte"
            if led_begin == "1":
                if led_direction == "1":  # UZ
                    change0 = (led_bottom // 2) - (self.gap // 2) + ledCornerBottomLeft
                    change1 = change0 + led_left + ledCornerTopLeft
                    change2 = change1 + led_top + ledCornerTopRight
                    change3 = change2 + led_right + ledCornerBottomRight
                    positions = ["bottom", "left", "top", "right", "bottom"]
                    self.hscan_min = 0.5000 - (self.hb_diff * (self.gap // 2))  # UZ = unten mitte nach links
                    self.gapLED = 0
                else:  # GUZ
                    change0 = (led_bottom // 2) - (self.gap // 2) + ledCornerBottomRight
                    change1 = change0 + led_right + ledCornerTopRight
                    change2 = change1 + led_top + ledCornerTopLeft
                    change3 = change2 + led_left + ledCornerBottomLeft
                    positions = ["bottom", "right", "top", "left", "bottom"]
                    self.hscan_min = 0.5000 - self.hb_diff + (
                            self.hb_diff * (self.gap // 2))  # GUZ = unten mitte nach rechts
                    self.gapLED = 0
                changes = [change0, change1, change2, change3]

            # ledBegin "rechts unten"
            if led_begin == "2":
                if led_direction == "1":  # UZ
                    change0 = led_bottom - self.gap + ledCornerBottomLeft
                    change1 = change0 + led_left + ledCornerTopLeft
                    change2 = change1 + led_top + ledCornerTopRight
                    change3 = change2 + led_right + ledCornerBottomRight
                    self.hscan_min = 1.0000 - self.vPicDist  # UZ = unten von rechts nach links
                    positions = ["bottom", "left", "top", "right"]
                    if self.gap:
                        self.gapLED = (led_bottom // 2) - (self.gap // 2) + 1
                else:  # GUZ
                    change0 = led_right + ledCornerTopRight
                    change1 = change0 + led_top + ledCornerTopLeft
                    change2 = change1 + led_left + ledCornerBottomLeft
                    change3 = change2 + led_bottom - self.gap + ledCornerBottomRight
                    positions = ["right", "top", "left", "bottom"]
                    self.vscan_min = 1.0000 - self.hPicDist  # GUZ = rechte Seite nach oben
                    if self.gap:
                        self.gapLED = change2 + (led_bottom // 2) - (self.gap // 2) + 1
                changes = [change0, change1, change2, change3]

            # ledBegin "rechts oben"
            if led_begin == "3":
                if led_direction == "1":  # UZ
                    change0 = led_right + ledCornerBottomRight
                    change1 = change0 + led_bottom - self.gap + ledCornerBottomLeft
                    change2 = change1 + led_left + ledCornerTopLeft
                    change3 = change2 + led_top + ledCornerTopRight
                    self.vscan_min = 0.0000 - self.vr_diff + self.hPicDist  # UZ = rechts von oben nach unten
                    positions = ["right", "bottom", "left", "top"]
                    if self.gap:
                        self.gapLED = change0 + (led_bottom // 2) - (self.gap // 2) + 1
                else:  # GUZ
                    change0 = led_top + ledCornerTopLeft
                    change1 = change0 + led_left + ledCornerBottomLeft
                    change2 = change1 + led_bottom - self.gap + ledCornerBottomRight
                    change3 = change2 + led_right + ledCornerTopRight
                    positions = ["top", "left", "bottom", "right"]
                    self.hscan_min = 1.0000 - self.vPicDist  # GUZ = oben von rechts nach links
                    if self.gap:
                        self.gapLED = change1 + (led_bottom // 2) - (self.gap // 2) + 1
                changes = [change0, change1, change2, change3]

            # ledBegin "links oben"
            if led_begin == "4":
                if led_direction == "1":  # UZ
                    change0 = led_top + ledCornerTopRight
                    change1 = change0 + led_right + ledCornerBottomRight
                    change2 = change1 + led_bottom - self.gap + ledCornerBottomLeft
                    change3 = change2 + led_left + ledCornerTopLeft
                    self.hscan_min = 0.0000 - self.ht_diff + self.vPicDist  # UZ = oben von links nach rechts
                    positions = ["top", "right", "bottom", "left"]
                    if self.gap:
                        self.gapLED = change1 + (led_bottom // 2) - (self.gap // 2) + 1
                else:  # GUZ
                    change0 = led_left + ledCornerBottomLeft
                    change1 = change0 + led_bottom - self.gap + ledCornerBottomRight
                    change2 = change1 + led_right + ledCornerTopRight
                    change3 = change2 + led_top + ledCornerTopLeft
                    positions = ["left", "bottom", "right", "top"]
                    self.vscan_min = 0.0000 - self.vl_diff + self.hPicDist  # GUZ = links von oben nach unten
                    if self.gap:
                        self.gapLED = change0 + (led_bottom // 2) - (self.gap // 2) + 1
                changes = [change0, change1, change2, change3]

            # ledBegin "links unten"
            if led_begin == "5":
                if led_direction == "1":  # UZ
                    change0 = led_left + ledCornerTopLeft
                    change1 = change0 + led_top + ledCornerTopRight
                    change2 = change1 + led_right + ledCornerBottomRight
                    change3 = change2 + led_bottom - self.gap + ledCornerBottomLeft
                    self.vscan_min = 1.000 - self.hPicDist  # UZ = links von unten nach oben
                    positions = ["left", "top", "right", "bottom"]
                    if self.gap:
                        self.gapLED = change2 + (led_bottom // 2) - (self.gap // 2) + 1
                else:  # GUZ
                    change0 = led_bottom - self.gap + ledCornerBottomRight
                    change1 = change0 + led_right + ledCornerTopRight
                    change2 = change1 + led_top + ledCornerTopLeft
                    change3 = change2 + led_left + ledCornerBottomLeft
                    positions = ["bottom", "right", "top", "left"]
                    self.hscan_min = 0.0000 - self.hb_diff + self.vPicDist  # GUZ = unten von links nach rechts
                    if self.gap:
                        self.gapLED = (led_bottom // 2) - (self.gap // 2) + 1
                changes = [change0, change1, change2, change3]

            # print "=== changes, positions, gapLED", changes, positions, self.gapLED
            leds = []
            for i in range(1, led_count + 1):
                position = 0
                for x in changes:
                    if i <= x:
                        break
                    position += 1
                changeNumber = position

                (hscan_min, hscan_max, vscan_min, vscan_max) = self.getNewFixScanValue(i, changeNumber, changes, positions[position])

                # if use overlap cut to min/max
                if hscan_min < 0.0000: hscan_min = 0.0000
                if hscan_min > 1.0000: hscan_min = 1.0000
                if hscan_max < 0.0000: hscan_max = 0.0000
                if hscan_max > 1.0000: hscan_max = 1.0000
                if vscan_min < 0.0000: vscan_min = 0.0000
                if vscan_min > 1.0000: vscan_min = 1.0000
                if vscan_max < 0.0000: vscan_max = 0.0000
                if vscan_max > 1.0000: vscan_max = 1.0000

                led = OrderedDict()
                led['index'] = int(i - 1)
                led['hscan'] = {'minimum': abs(hscan_min), 'maximum': abs(hscan_max)}
                led['vscan'] = {'minimum': abs(vscan_min), 'maximum': abs(vscan_max)}
                leds.append(led)
                # print "=== led dict", led

                # print "index: ", i-1
                # print '"hscan" : { "minimum" : %s, "maximum" : %s },' % (format(round(hscan_min,4),".4f"), format(round(hscan_max,4),".4f"))
                # print '"vscan" : { "minimum" : %s, "maximum" : %s }' % (format(round(vscan_min,4),".4f"), format(round(vscan_max,4),".4f"))
                # print "------------------------------------------------"
            # print "=== leds", leds
            return leds


        except:
            import traceback
            traceback.print_exc()
            return None


def about(self):
    title = _("HyperionControl v. %s \nplugin by\neinfall, murxer, pclin & Sven H") % hyperioncontrol_version
    self.session.open(MessageBox, ("%s") % (title), MessageBox.TYPE_INFO)


def send_CMD(cmd, master=False):
    if config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-above" and not master:
        port = 19451
    elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-below" and not master:
        port = 19453
    elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-left" and not master:
        port = 19447
    elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-right" and not master:
        port = 19449
    else:
        # hyperion, hyperion-aml and hyperion-v4l2
        port = 19444

    raw = "hyperion-remote -a localhost:%s %s" % (str(port), cmd)
    print "[HyperionControl] SEND CMD TO Hyperion Remote: %s" % raw
    os.system(raw)


def hyperionTimer():
    if config.plugins.hyperioncontrol.timerMode.value == "always":
        pass
    elif config.plugins.hyperioncontrol.timerMode.value == "live":
        pass
    else:
        return False


def readLampLeft_Device():
    lampleft_device = False
    command = Command("systemctl status hyperion-left")
    command.run(timeout=3)
    cmd_out = command.out
    if "Active: active" in cmd_out:
        lampleft_device = True
    return lampleft_device


def readLampRight_Device():
    lampright_device = False
    command = Command("systemctl status hyperion-right")
    command.run(timeout=3)
    cmd_out = command.out
    if "Active: active" in cmd_out:
        lampright_device = True
    return lampright_device


def readLampAbove_Device():
    lampabove_device = False
    command = Command("systemctl status hyperion-above")
    command.run(timeout=3)
    cmd_out = command.out
    if "Active: active" in cmd_out:
        lampabove_device = True
    return lampabove_device


def readLampBelow_Device():
    lampbelow_device = False
    command = Command("systemctl status hyperion-below")
    command.run(timeout=3)
    cmd_out = command.out
    if "Active: active" in cmd_out:
        lampbelow_device = True
    return lampbelow_device


def readLampLeftAuto_Device():
    lampleftauto_device = False
    command = Command("systemctl status hyperion-left")
    command.run(timeout=3)
    cmd_out = command.out
    if "enabled;" in cmd_out:
        lampleftauto_device = True
    return lampleftauto_device


def readLampRightAuto_Device():
    lamprightauto_device = False
    command = Command("systemctl status hyperion-right")
    command.run(timeout=3)
    cmd_out = command.out
    if "enabled;" in cmd_out:
        lamprightauto_device = True
    return lamprightauto_device


def readLampAboveAuto_Device():
    lampaboveauto_device = False
    command = Command("systemctl status hyperion-above")
    command.run(timeout=3)
    cmd_out = command.out
    if "enabled;" in cmd_out:
        lampaboveauto_device = True
    return lampaboveauto_device


def readLampBelowAuto_Device():
    lampbelowauto_device = False
    command = Command("systemctl status hyperion-below")
    command.run(timeout=3)
    cmd_out = command.out
    if "enabled;" in cmd_out:
        lampbelowauto_device = True
    return lampbelowauto_device


def readV4l2_Device():
    v4l2_device = None
    # v4l2 Device
    command = Command("usb-devices")
    command.run(timeout=3)
    cmd_out = command.out
    if "Driver=uvcvideo" in cmd_out:
        v4l2_device = True
    return v4l2_device


def getHyperionInfo(mode="hyperion"):
    autostart_hyperion = False
    autostart_hyperionaml = False
    autostart_hyperion_v4l2 = False
    running = False
    autostart = False

    if mode == "hyperion":
        # hyperion-status
        command = Command("systemctl status hyperion")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            running = True

        # hyperion autostart
        command = Command("systemctl status hyperion.service")
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            autostart = True
        return running, autostart

    elif mode == "hyperion-aml":
        # hyperion-aml-status
        command = Command("systemctl status hyperionaml")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            running = True

        # systemd-hyperionaml.timer
        command = Command("systemctl status systemd-hyperionaml.timer")
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            autostart = True
        return running, autostart

    elif mode == "hyperion-v4l2":
        # hyperion-v4l2-status
        command = Command("systemctl status hyperion-v4l2")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            running = True

        # systemd-hyperion-v4l2 autostart
        command = Command("systemctl status hyperion-v4l2.service")
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            autostart = True
        return running, autostart

    elif mode == "autostart":
        # hyperion autostart
        command = Command("systemctl status hyperion.service")
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            autostart_hyperion = True

        # systemd-hyperionaml.timer
        command = Command("systemctl status systemd-hyperionaml.timer")
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            autostart_hyperionaml = True

        # systemd-hyperion-v4l2 autostart
        command = Command("systemctl status hyperion-v4l2.service")
        command.run(timeout=3)
        cmd_out = command.out
        if "enabled;" in cmd_out:
            autostart_hyperion_v4l2 = True
        return autostart_hyperion, autostart_hyperionaml, autostart_hyperion_v4l2

    elif mode == "running":
        hyperion_running = False
        hyperion_aml_running = False
        hyperion_v4l2_running = False
        # hyperion-status
        command = Command("systemctl status hyperion")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            hyperion_running = True

        # hyperion-status
        command = Command("systemctl status hyperionaml")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            hyperion_aml_running = True

        # hyperion-v4l2-status
        command = Command("systemctl status hyperion-v4l2")
        command.run(timeout=3)
        cmd_out = command.out
        if "Active: active" in cmd_out:
            hyperion_v4l2_running = True

        return hyperion_running, hyperion_aml_running, hyperion_v4l2_running


def setConfigValueFromJson(config, jsonData, jsonKey, calculate=None, onlyReturnValue=False, default=None):
    try:
        for item in jsonKey:
            jsonData = jsonData[item]
        if calculate:
            jsonData = int(jsonData * calculate)
        if onlyReturnValue:
            print "[HyperionControl] return value", item, jsonData
            return jsonData
        else:
            if isinstance(config, ConfigSelection) and str(jsonData) not in config.choices and default:
                jsonData = default
            config.value = str(jsonData)
            print "[HyperionControl] set value to", item, str(jsonData)
    except (KeyError, TypeError, IndexError):
        if not default:
            default = config.default
        if onlyReturnValue:
            print "[HyperionControl] error on item - return default", item, default
            return default
        else:
            print "[HyperionControl] error on item - set default", item, default
            config.value = default


def setChoicesHyperionRemote():
    # build choices
    hyperion = getHyperionInfo(mode="running")
    choices = []
    default = "hyperion-aml"
    if hyperion[1]:
        choices.append(("hyperion-aml", _("Hyperion-internal grabber")))
        default = "hyperion-aml"
    elif hyperion[2]:
        choices.append(("hyperion-v4l2", _("Hyperion-external grabber")))
        default = "hyperion-v4l2"
    elif hyperion[0]:
        choices.append(("hyperion", _("Hyperion Boblight/KodiServer")))
        default = "hyperion"
    if not choices:
        if config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion-v4l2":
            choices.append(("hyperion-v4l2", _("Hyperion-external grabber")))
            default = "hyperion-v4l2"
        elif config.plugins.hyperioncontrol.hyperionModeType.value == "hyperion":
            choices.append(("hyperion-aml", _("Hyperion-internal grabber")))
            default = "hyperion"
        else:
            choices.append(("hyperion-aml", _("Hyperion-internal grabber")))
            default = "hyperion-aml"

    if config.plugins.hyperioncontrol.extraLamp.value:
        if readLampAbove_Device():
            choices.append(("hyperion-above", _("Hyperion Above")))
        if readLampBelow_Device():
            choices.append(("hyperion-below", _("Hyperion Below")))
        if readLampRight_Device():
            choices.append(("hyperion-left", _("Hyperion Left")))
        if readLampLeft_Device():
            choices.append(("hyperion-right", _("Hyperion Right")))

    config.plugins.hyperioncontrol.hyperionRemoteType = NoSave(ConfigSelection(choices=choices, default=default))


def getWledIP(config_file_path):
    try:
        if os.path.exists(config_file_path):
            valuesFromConfig = open(config_file_path).read()
            lines = valuesFromConfig.split("\n")
            newlines = []
            for line in lines:
                cleanline = line.replace("\t", "")
                if not cleanline.rstrip().startswith("//") and cleanline.strip() != "":
                    newlines.append(line)
            valuesFromConfig = "\n".join(newlines)
            jsonConfig = json.loads(valuesFromConfig, object_pairs_hook=OrderedDict)
            output_ip = jsonConfig['device']['output'].split(":")[0]
            return output_ip
        return None
    except:
        import traceback, sys
        traceback.print_exc()
        exc_type, exc_value, exc_traceback = sys.exc_info()
        error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
        print "[HyperionControl] error on readValuesFromRemote:", str(error)
        return None


def readValuesFromConfig(jsonConfig):
    if jsonConfig:
        # hsv
        setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, jsonConfig,
                               ['color', 'transform', 0, 'hsv', 'saturationGain'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, jsonConfig,
                               ['color', 'transform', 0, 'hsv', 'valueGain'], 100)
        # hsl
        setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, jsonConfig,
                               ['color', 'transform', 0, 'hsl', 'saturationLGain'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, jsonConfig,
                               ['color', 'transform', 0, 'hsl', 'luminanceGain'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, jsonConfig,
                               ['color', 'transform', 0, 'hsl', 'luminanceMinimum'], 100)
        # red
        setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, jsonConfig,
                               ['color', 'transform', 0, 'red', 'gamma'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, jsonConfig,
                               ['color', 'transform', 0, 'red', 'threshold'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, jsonConfig,
                               ['color', 'temperature', 0, 'correctionValues', 'red'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureRed', 'redChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureGreen', 'redChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureBlue', 'redChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, jsonConfig,
                               ['color', 'transform', 0, 'red', 'whitelevel'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, jsonConfig,
                               ['color', 'transform', 0, 'red', 'blacklevel'], 100)

        # green
        setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, jsonConfig,
                               ['color', 'transform', 0, 'green', 'gamma'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, jsonConfig,
                               ['color', 'transform', 0, 'green', 'threshold'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, jsonConfig,
                               ['color', 'temperature', 0, 'correctionValues', 'green'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureRed', 'greenChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureGreen', 'greenChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureBlue', 'greenChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, jsonConfig,
                               ['color', 'transform', 0, 'green', 'whitelevel'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, jsonConfig,
                               ['color', 'transform', 0, 'green', 'blacklevel'], 100)

        # blue
        setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, jsonConfig,
                               ['color', 'transform', 0, 'blue', 'gamma'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, jsonConfig,
                               ['color', 'transform', 0, 'blue', 'threshold'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, jsonConfig,
                               ['color', 'temperature', 0, 'correctionValues', 'blue'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureRed', 'blueChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureGreen', 'blueChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, jsonConfig,
                               ['color', 'channelAdjustment', 0, 'pureBlue', 'blueChannel'])
        setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, jsonConfig,
                               ['color', 'transform', 0, 'blue', 'whitelevel'], 100)
        setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, jsonConfig,
                               ['color', 'transform', 0, 'blue', 'blacklevel'], 100)


def readValuesFromRemote(self, showMessage=True, setConfig=True, jsonConfig=None):
    print "[HyperionControl] readValuesFromRemote"
    if config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-above":
        port = 19451
    elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-below":
        port = 19453
    elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-left":
        port = 19447
    elif config.plugins.hyperioncontrol.hyperionRemoteType.value == "hyperion-right":
        port = 19449
    else:
        # hyperion, hyperion-aml and hyperion-v4l2
        port = 19444
    try:
        #command = Command("hyperion-remote -l")
        command = Command("hyperion-remote -a localhost:%s -l" % str(port))
        command.run(timeout=5)

        # read real settings from hyperion-remote -l
        if command.timeout:
            print "[HyperionControl] timeout on readValuesFromRemote"
            if showMessage:
                self.session.open(MessageBox, _("Timeout on load remote-Data!\n\n"), MessageBox.TYPE_INFO, timeout=5)
            if setConfig and jsonConfig:
                readValuesFromConfig(jsonConfig)
            return None
        elif command.process.returncode != 0:
            print "[HyperionControl] error on readValuesFromRemote:", str(command.process.returncode), str(command.err)
            if showMessage:
                self.session.open(MessageBox, _("Error on load remote-Data!\n\nError:\n%s") % str(command.err),
                                  MessageBox.TYPE_INFO, timeout=5)
            if setConfig and jsonConfig:
                readValuesFromConfig(jsonConfig)
            return None

        valuesFromRemote = command.out

        pos = valuesFromRemote.find("Server info:")
        if pos:
            valuesFromRemote = valuesFromRemote[pos + 12:]
            jsonRemote = json.loads(valuesFromRemote)
            if setConfig:
                setConfigValueFromJson(config.plugins.hyperioncontrol.saturationGain, jsonRemote,
                                       ['transform', 0, 'saturationGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.valueGain, jsonRemote,
                                       ['transform', 0, 'valueGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.saturationLGain, jsonRemote,
                                       ['transform', 0, 'saturationLGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceGain, jsonRemote,
                                       ['transform', 0, 'luminanceGain'], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.luminanceMinimum, jsonRemote,
                                       ['transform', 0, 'luminanceMinimum'], 100)

                setConfigValueFromJson(config.plugins.hyperioncontrol.gammaRed, jsonRemote,
                                       ['transform', 0, 'gamma', 0], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.gammaGreen, jsonRemote,
                                       ['transform', 0, 'gamma', 1], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.gammaBlue, jsonRemote,
                                       ['transform', 0, 'gamma', 2], 100)

                setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdRed, jsonRemote,
                                       ['transform', 0, 'threshold', 0], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdGreen, jsonRemote,
                                       ['transform', 0, 'threshold', 1], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.thresholdBlue, jsonRemote,
                                       ['transform', 0, 'threshold', 2], 100)

                setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustRed, jsonRemote,
                                       ['adjustment', 0, 'redAdjust', 0])
                setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustGreen, jsonRemote,
                                       ['adjustment', 0, 'redAdjust', 1])
                setConfigValueFromJson(config.plugins.hyperioncontrol.redAdjustBlue, jsonRemote,
                                       ['adjustment', 0, 'redAdjust', 2])
                setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustRed, jsonRemote,
                                       ['adjustment', 0, 'greenAdjust', 0])
                setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustGreen, jsonRemote,
                                       ['adjustment', 0, 'greenAdjust', 1])
                setConfigValueFromJson(config.plugins.hyperioncontrol.greenAdjustBlue, jsonRemote,
                                       ['adjustment', 0, 'greenAdjust', 2])
                setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustRed, jsonRemote,
                                       ['adjustment', 0, 'blueAdjust', 0])
                setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustGreen, jsonRemote,
                                       ['adjustment', 0, 'blueAdjust', 1])
                setConfigValueFromJson(config.plugins.hyperioncontrol.blueAdjustBlue, jsonRemote,
                                       ['adjustment', 0, 'blueAdjust', 2])
                setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureRed, jsonRemote,
                                       ['temperature', 0, 'correctionValues', 0])
                setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureGreen, jsonRemote,
                                       ['temperature', 0, 'correctionValues', 1])
                setConfigValueFromJson(config.plugins.hyperioncontrol.temperatureBlue, jsonRemote,
                                       ['temperature', 0, 'correctionValues', 2])

                setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelRed, jsonRemote,
                                       ['transform', 0, 'blacklevel', 0], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelGreen, jsonRemote,
                                       ['transform', 0, 'blacklevel', 1], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.blacklevelBlue, jsonRemote,
                                       ['transform', 0, 'blacklevel', 2], 100)

                setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelRed, jsonRemote,
                                       ['transform', 0, 'whitelevel', 0], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelGreen, jsonRemote,
                                       ['transform', 0, 'whitelevel', 1], 100)
                setConfigValueFromJson(config.plugins.hyperioncontrol.whitelevelBlue, jsonRemote,
                                       ['transform', 0, 'whitelevel', 2], 100)

            return jsonRemote

    except:
        import traceback, sys
        traceback.print_exc()
        exc_type, exc_value, exc_traceback = sys.exc_info()
        if showMessage:
            error = "\n".join(traceback.format_exception_only(exc_type, exc_value))
            self.session.open(MessageBox, _("Error on load remote-Data!\n\nError:\n%s") % error, MessageBox.TYPE_INFO,
                              timeout=10)
        if setConfig and jsonConfig:
            readValuesFromConfig(jsonConfig)
        return None

